@@ -870,7 +870,7 @@ int TMainForm::removeModule(const char *modName)
 				modDir += entry->second.c_str();
 				entry = module->second.find("ModDrv");
 				if (entry != module->second.end()) {
-					if (!strcmp(entry->second.c_str(), "RawLD")) {
+					if (!strcmp(entry->second.c_str(), "RawLD") || !strcmp(entry->second.c_str(), "RawLD4") || !strcmp(entry->second.c_str(), "zLD") || !strcmp(entry->second.c_str(), "RawGenBook") || !strcmp(entry->second.c_str(), "zGenBook")) {
 						char *buf = new char [ strlen(modDir.c_str()) + 1 ];
 	
 						strcpy(buf, modDir.c_str());
@@ -926,7 +926,7 @@ int TMainForm::removeModule(const char *modName)
 int TMainForm::createParent(const char *pName)
 {
 	char *buf = new char [ strlen(pName) + 1 ];
-	int retCode;
+	int retCode = 0;
 	
 	strcpy(buf, pName);
 	int end = strlen(buf) - 1;
@@ -936,13 +936,16 @@ int TMainForm::createParent(const char *pName)
 		end--;
 	}
 	buf[end] = 0;
-	if (!strlen(buf))
-		return -1;
-		
-	if ((retCode = mkdir(buf, 0755))) {
-		createParent(buf);
-		retCode = mkdir(buf, 0755);
-	}
+
+	if (strlen(buf)>0) {
+          if (access(buf, 02)) {  // not exists with write access?
+          	if ((retCode = mkdir(buf, 0755))) {
+		         	createParent(buf);
+                    retCode = mkdir(buf, 0755);
+               }
+		}
+     }
+     else retCode = -1;
 	delete [] buf;
 	return retCode;
 }
@@ -952,7 +955,7 @@ int TMainForm::createPathAndFile(const char *fName)
 {
 	int fd;
 	
-	fd = open(fName, O_CREAT|O_WRONLY|O_BINARY, 00644);
+	fd = open(fName, O_CREAT|O_WRONLY|O_BINARY, S_IREAD|S_IWRITE);
 	if (fd < 1) {
 		createParent(fName);
 		fd = open(fName, O_CREAT|O_WRONLY|O_BINARY, S_IREAD|S_IWRITE);
@@ -1069,7 +1072,7 @@ int TMainForm::installModule(const char *modName, InstallSourceTab *ist)
 				modDir = entry->second.c_str();
 				entry = module->second.find("ModDrv");
 				if (entry != module->second.end()) {
-					if (!strcmp(entry->second.c_str(), "RawLD")) {
+					if (!strcmp(entry->second.c_str(), "RawLD") || !strcmp(entry->second.c_str(), "RawLD4") || !strcmp(entry->second.c_str(), "zLD") || !strcmp(entry->second.c_str(), "RawGenBook") || !strcmp(entry->second.c_str(), "zGenBook")) {
 						char *buf = new char [ strlen(modDir.c_str()) + 1 ];
 	
 						strcpy(buf, modDir.c_str());
