@@ -36,8 +36,8 @@
 class TTreeNodes;
 
 class TCustomTreeView : public TWinControl {
-friend TTreeNodes;
-friend TTreeNode;
+friend class TTreeNodes;
+friend class TTreeNode;
 protected:
 	void init(TComponent *AOwner);
 
