@@ -29,6 +29,8 @@
 #include <thmlplain.h>
 #include <string.h>
 
+using namespace sword;
+using namespace std;
 
 MainWindow *cheatahWindow = NULL;
 
