@@ -542,7 +542,7 @@ void MainWindow::lookupTextChanged() {
 		keyText += entryText;
 
 		if (curMod) {
-			curMod->SetKey(keyText.c_str());
+			curMod->setKey(keyText.c_str());
 			curMod->Display();
 			sprintf(tmpBuf, "%s (%s)", curMod->KeyText(), curMod->Name());
 			gtk_label_set(GTK_LABEL(statusbar1), tmpBuf);
