@@ -7,10 +7,8 @@
 #include "corediatheke.h"
 #include <regex.h>
 #include <iostream>
-#include <string>
 #include <list>
 
-using std::string;
 using std::list;
 using std::cout;
 using std::endl;
@@ -25,9 +23,9 @@ void systemquery(const char * key, ostream* output){
 	bool types = false, descriptions = false, names = false;
 
 	if (!stricmp(key, "localelist")) {		
-		static LocaleMgr lm = LocaleMgr::systemLocaleMgr;
-		list<string> loclist =	lm.getAvailableLocales();
-		list<string>::iterator li = loclist.begin();
+		LocaleMgr *lm = &LocaleMgr::systemLocaleMgr;
+		list<SWBuf> loclist =	lm->getAvailableLocales();
+		list<SWBuf>::iterator li = loclist.begin();
 		for (;li != loclist.end(); li++) {
 		  *output << li->c_str() << endl;
 		}
@@ -90,7 +88,7 @@ void doquery(unsigned long maxverses = -1, unsigned char outputformat = FMT_PLAI
 	SWModule * target;
 	char *font = 0;
 	char inputformat = 0;
-	string encoding;
+	SWBuf encoding;
 	char querytype = 0;	
 
 	if (locale) {
@@ -126,7 +124,7 @@ void doquery(unsigned long maxverses = -1, unsigned char outputformat = FMT_PLAI
 			else if (!stricmp((char *)(*eit).second.c_str(), "ThML"))
 				inputformat = FMT_THML;
 		}
-		encoding = ((eit = (*sit).second.find("Encoding")) != (*sit).second.end()) ? (*eit).second : (string)"";
+		encoding = ((eit = (*sit).second.find("Encoding")) != (*sit).second.end()) ? (*eit).second : (SWBuf)"";
 	}
 
 
@@ -192,7 +190,10 @@ void doquery(unsigned long maxverses = -1, unsigned char outputformat = FMT_PLAI
 		manager.setGlobalOption("Scripture Cross-References","On");
 	else
 		manager.setGlobalOption("Scripture Cross-References","Off");
-
+	if (optionfilters & OP_RED)
+		manager.setGlobalOption("Words of Christ in Red","On");
+	else
+		manager.setGlobalOption("Words of Christ in Red","Off");
 	if (optionfilters & OP_VARIANTS && variants) {
 			if (variants == -1)
                 manager.setGlobalOption("Variants", "All Readings");
