@@ -13,6 +13,8 @@
 #include <vcl\Buttons.hpp>
 #include <vcl\ExtCtrls.hpp>
 #include <vcl\ComCtrls.hpp>
+
+#include "rxricheditx.h"
 //----------------------------------------------------------------------------
 class TAboutBox : public TForm
 {
@@ -22,15 +24,19 @@ __published:
 	TLabel *ProductName;
 	TLabel *Version;
 	TLabel *Copyright;
-	TRichEdit *CreditAbout;
-	TRichEdit *ModulesAbout;
 	TPanel *Panel2;
 	TButton *OKButton;
+	TPanel *ModAboutPnl;
+	TPanel *CreditAboutPnl;
 	void __fastcall FormShow(TObject *Sender);
 	void __fastcall ProgramIconMouseDown(TObject *Sender, TMouseButton Button,
           TShiftState Shift, int X, int Y);
+	void __fastcall FormCreate(TObject *Sender);
 private:
+	TRxRichEditX *ModulesAbout;
+	TRxRichEditX *CreditAbout;
 public:
+	void __fastcall RTFURLClick(TObject *Sender, const AnsiString URLText, TMouseButton Button);
 	virtual __fastcall TAboutBox(TComponent* AOwner);
 };
 //----------------------------------------------------------------------------
