@@ -10,19 +10,26 @@
 #include <ComCtrls.hpp>
 #include <ExtCtrls.hpp>
 #include <Graphics.hpp>
+#include "rxricheditx.h"
+
 //---------------------------------------------------------------------------
 class TDevOfTheDay2 : public TForm
 {
 __published:	// IDE-managed Components
-        TRichEdit *RichTip;
-        TCheckBox *ckShowTip;
-        TButton *btnOK;
-        TPanel *Panel1;
-        TImage *bmpDev;
-        void __fastcall btnOKClick(TObject *Sender);
+	TRichEdit *RichTipOld;
+	TPanel *Panel1;
+	TPanel *Panel3;
+	TCheckBox *ckShowTip;
+	TButton *btnOK;
+	TPanel *Panel4;
+	TImage *bmpDev;
+	TLabel *Label1;
+	void __fastcall btnOKClick(TObject *Sender);
+	void __fastcall FormShow(TObject *Sender);
 private:	// User declarations
 public:		// User declarations
-        __fastcall TDevOfTheDay2(TComponent* Owner);
+	TRxRichEditX *RichTip;
+	__fastcall TDevOfTheDay2(TComponent* Owner);
 };
 //---------------------------------------------------------------------------
 extern PACKAGE TDevOfTheDay2 *DevOfTheDay2;
