@@ -11,4 +11,13 @@ __fastcall TModInstFrm::TModInstFrm(TComponent* Owner)
 	: TForm(Owner)
 {
 }
-//--------------------------------------------------------------------------- 
\ No newline at end of file
+//---------------------------------------------------------------------------
+void __fastcall TModInstFrm::FormCreate(TObject *Sender)
+{
+	AboutText = new TRxRichEditX(this);
+	AboutText->Parent = Panel2;
+	AboutText->Align = alClient;
+	AboutText->ScrollBars = ssVertical;
+	AboutText->ReadOnly = true;	
+}
+//---------------------------------------------------------------------------
