@@ -8,17 +8,20 @@
 #include <vcl\Forms.hpp>
 #include <vcl\ComCtrls.hpp>
 #include <vcl\ExtCtrls.hpp>
+#include "rxricheditx.h"
 //---------------------------------------------------------------------------
 class TModInstFrm : public TForm
 {
 __published:	// IDE-managed Components
-	TRichEdit *AboutText;
 	TPanel *Panel1;
 	TButton *OkBtn;
 	TLabel *ModText;
+	TPanel *Panel2;
+	void __fastcall FormCreate(TObject *Sender);
 private:	// User declarations
 public:		// User declarations
 	__fastcall TModInstFrm(TComponent* Owner);
+	TRxRichEditX *AboutText;
 };
 //---------------------------------------------------------------------------
 extern TModInstFrm *ModInstFrm;
