@@ -44,7 +44,7 @@ void __fastcall TModVisForm::FormClose(TObject *Sender,
 {
 	if (this->ModalResult == mrOk) {
      	for (int i = 0; i < modList->Items->Count; i++) {
-          	string name = modList->Items->Item[i]->Caption.c_str();
+          	SWBuf name = modList->Items->Item[i]->Caption.c_str();
                (*section)[name] = (modList->Items->Item[i]->Checked) ? "true" : "false";
           }
      }
