@@ -37,6 +37,7 @@ __fastcall TPrintForm::TPrintForm(TComponent* Owner)
 
 
 void __fastcall TPrintForm::FormShow(TObject *Sender) {
+      /* TODO -oDavid T -cPrinting : Remove this once Printing is in a finished form */
 	if(Form1->optionsconf->Sections["Printing"]["Notice"] != "1"){
 		Application->MessageBox("This release of The Sword Printer is preliminary and is not yet fully functional, therefore some of the features here have been disabled. Please be sure to take a look at the formatting options to be able to fully take control of how your page will look. Also be sure to use the \"Print Preview\" feature to preview how your page(s) will look on paper before printing. Enjoy! From, The Sword Project Development Team!", "The Sword Printer", MB_ICONINFORMATION | MB_OK);
 		Form1->optionsconf->Sections["Printing"]["Notice"] = "1";
@@ -81,8 +82,10 @@ void TPrintForm::print(ListKey *keys, SWModule *module) {
 			cmbModule->ItemIndex = count;
 	}
 	SetHint();
-	fontName = ((eit = Form1->optionsconf->Sections["PrintPrefs"].find("PrintFontName")) != Form1->optionsconf->Sections["PrintPrefs"].end())? (*eit).second.c_str() : "Times New Roman";
-	cmbFont->ItemIndex = cmbFont->Items->IndexOf(fontName);
+	// Removed this because I want to place it in a central location which will load all
+	// the printed page settings -- David Trotz
+	// fontName = ((eit = Form1->optionsconf->Sections["PrintedPage"].find("TextFontName")) != Form1->optionsconf->Sections["PrintedPage"].end())? (*eit).second.c_str() : "Times New Roman";
+	// cmbFont->ItemIndex = cmbFont->Items->IndexOf(fontName);
 	PageControl1->ActivePage = shtSelection;
 
 	SetDefaultOps();
@@ -148,7 +151,7 @@ void __fastcall TPrintForm::radLDClick(TObject *Sender)
 }
 //---------------------------------------------------------------------------
 
-void __fastcall TPrintForm::spinLeftClick(TObject *Sender,
+void __fastcall TPrintForm::spnLeftClick(TObject *Sender,
       TUDBtnType Button)
 {
 	if(Button == Comctrls::btNext)
@@ -159,7 +162,7 @@ void __fastcall TPrintForm::spinLeftClick(TObject *Sender,
 }
 //---------------------------------------------------------------------------
 
-void __fastcall TPrintForm::spinRightClick(TObject *Sender,
+void __fastcall TPrintForm::spnRightClick(TObject *Sender,
       TUDBtnType Button)
 {
 	if(Button == Comctrls::btNext)
@@ -169,7 +172,7 @@ void __fastcall TPrintForm::spinRightClick(TObject *Sender,
 }
 //---------------------------------------------------------------------------
 
-void __fastcall TPrintForm::spinBottomClick(TObject *Sender,
+void __fastcall TPrintForm::spnBottomClick(TObject *Sender,
       TUDBtnType Button)
 {
 	if(Button == Comctrls::btNext)
@@ -179,7 +182,7 @@ void __fastcall TPrintForm::spinBottomClick(TObject *Sender,
 }
 //---------------------------------------------------------------------------
 
-void __fastcall TPrintForm::spinTopClick(TObject *Sender,
+void __fastcall TPrintForm::spnTopClick(TObject *Sender,
       TUDBtnType Button)
 {
 	if(Button == Comctrls::btNext)
@@ -191,6 +194,7 @@ void __fastcall TPrintForm::spinTopClick(TObject *Sender,
 
 void __fastcall TPrintForm::btnPrintClick(TObject *Sender)
 {
+	SavePrintPrefs();
 	if(cmbModule->Text.data() != NULL && editStart->Text.data() != NULL){
 		PrepareRTF();
 		OnPrint(rtfPrint);
@@ -342,53 +346,39 @@ void __fastcall TPrintForm::btnSetupClick(TObject *Sender)
 	PrinterSetupDialog->Execute();
 }
 //---------------------------------------------------------------------------
-void __fastcall TPrintForm::BitBtn5Click(TObject *Sender)
-{
-		//To do
-}
-//---------------------------------------------------------------------------
 
 void TPrintForm::PrepareRTF()
 {
     BibleCSMGR* manager = Form1->mainmgr;
-    RTF_FORMAT format;
+    //RTF_FORMAT format;
     AnsiString newtext = "";
     AnsiString moduleName = cmbModule->Text.SubString(2,cmbModule->Text.Pos("]") - 2);
     SWModule *module = manager->Modules[moduleName.c_str()];
     ListKey lstKey;
     StrKey testKey;
-
-    format.bkFontHeadSize = 2 * editHeading->Text.ToInt();
-    format.bodyFontSize = 2 * editBody->Text.ToInt();
-    format.vsNumFontSize = 2 * editNumbers->Text.ToInt();
-    format.chFontHeadSize = 2 * editHeading->Text.ToInt();
-	format.fontFace =  cmbFont->Text;
-    format.paragraph = ckParagraph->Checked;
-    format.prBookHeadings = ckBookHead->Checked;
-    format.prChHeadings = ckChHead->Checked;
-    format.prPreFix = ckPrefix->Checked;
-    format.prPostFix = ckPostfix->Checked;
-    format.prVerseNum = ckVSNum->Checked;
-    format.superVSNum = ckSuperVS->Checked;
+    SavePrintPrefs();
 
     lstKey.Persist(1);
     if(!radLD->Checked){
-   		lstKey = VerseKey().ParseVerseList(editStart->Text.c_str(),*(Form1->DefaultVSKey), true);
-	   module->setKey(&lstKey);
-	    rtfPrint->RenderModule(module, format);
-	module->setKey(Form1->DefaultVSKey);
+		lstKey = VerseKey().ParseVerseList(editStart->Text.c_str(),*(Form1->DefaultVSKey), true);
+		module->setKey(&lstKey);
+		rtfPrint->type = "PrintedPage";
+		rtfPrint->RenderModule(module);
+		module->setKey(Form1->DefaultVSKey);
     }
     else{
-    	testKey.setText(WideStringToUTF8(editStart->Text).c_str());
-        lstKey << testKey;
-	   module->setKey(&lstKey);
-    	rtfPrint->RenderModule(module, format);
-	    module->setKey(Form1->DefaultStrKey);
+		testKey.setText(WideStringToUTF8(editStart->Text).c_str());
+		lstKey << testKey;
+		module->setKey(&lstKey);
+		rtfPrint->type = "PrintedPage";
+		rtfPrint->RenderModule(module);
+		module->setKey(Form1->DefaultStrKey);
     }
 }
 
 void __fastcall TPrintForm::btnPreviewClick(TObject *Sender)
 {
+     SavePrintPrefs();
 	if(cmbModule->Text.data() != NULL && editStart->Text.data() != NULL){
 		PrepareRTF();
         PreviewForm->currPage = 0;
@@ -417,6 +407,105 @@ void __fastcall TPrintForm::ckVSNumClick(TObject *Sender)
 
 void TPrintForm::SetDefaultOps()
 {
+	SWBuf 	modType = "",
+                tempVal = "";
+	if(radText->Checked)
+		modType = "Text";
+	else if(radComm->Checked)
+		modType = "Comm";
+	else modType = "LD";
+
+	//**** Print Chapter Headings? ****//
+	tempVal = Form1->optionsconf->Sections["PrintedPage"][(modType + "ChHead")];
+	if (tempVal == "")
+		ckChHead->Checked = true;
+	else	ckChHead->Checked = (atoi(tempVal.c_str())) ? true : false;
+
+	//**** Print Book Name? ****//
+	tempVal = Form1->optionsconf->Sections["PrintedPage"][(modType +"BkName")];
+	if (tempVal == "")
+		ckBookHead->Checked = true;
+	else	ckBookHead->Checked = (atoi(tempVal.c_str())) ? true : false;
+
+	//**** Prefix Each Verse? ****//
+	tempVal = Form1->optionsconf->Sections["PrintedPage"][(modType +"PreFixTag")];
+	if (tempVal == "")
+		ckPrefix->Checked = true;
+	else	ckPrefix->Checked = (atoi(tempVal.c_str())) ? true : false;
+
+	//**** Postfix Each Verse? ****//
+	tempVal = Form1->optionsconf->Sections["PrintedPage"][(modType +"PostFixTag")];
+	if (tempVal == "")
+		ckPostfix->Checked = true;
+	else	ckPostfix->Checked = (atoi(tempVal.c_str())) ? true : false;
+
+	//**** Print the verse number? ****//
+	tempVal = Form1->optionsconf->Sections["PrintedPage"][(modType + "VSNum")];
+	if (tempVal == "")
+		ckVSNum->Checked = true;
+	else	ckVSNum->Checked = (atoi(tempVal.c_str())) ? true : false;
+
+	//**** Superscript the verse number? ****//
+	tempVal = Form1->optionsconf->Sections["PrintedPage"][(modType + "SuperVS")];
+	if (tempVal == "")
+		ckSuperVS->Checked = true;
+	else	ckSuperVS->Checked = (atoi(tempVal.c_str())) ? true : false;
+
+	//**** Print as a paragraph? ****//
+	tempVal = Form1->optionsconf->Sections["PrintedPage"][(modType + "Paragraph")];
+	if (tempVal == "")
+		ckParagraph->Checked = true;
+	else	ckParagraph->Checked = (atoi(tempVal.c_str())) ? true : false;
+
+	//**** Chapter Heading Size ****//
+	tempVal = Form1->optionsconf->Sections["PrintedPage"][(modType + "ChHeadSize")];
+	if (tempVal != "")
+		spnHeading->Position = (atoi(tempVal.c_str()));
+
+	//**** Body Text Size ****//
+	tempVal = Form1->optionsconf->Sections["PrintedPage"][(modType + "FontSize")];
+	if (tempVal != "")
+		spnBody->Position = (atoi(tempVal.c_str()));
+
+	//**** Verse Number Size ****//
+	tempVal = Form1->optionsconf->Sections["PrintedPage"][(modType + "VSNumSize")];
+	if (tempVal != "")
+		spnNumbers->Position = (atoi(tempVal.c_str()));
+
+	//**** Left Margin ****//
+	tempVal = Form1->optionsconf->Sections["PrintedPage"][("LeftMargin")];
+	if (tempVal != ""){
+		spnLeft->Position = GetSpinPos(atof(tempVal.c_str()));
+          editLeft->Text = roundMargin(atof(tempVal.c_str()));
+     }
+
+     //**** Right Margin ****//
+	tempVal = Form1->optionsconf->Sections["PrintedPage"][("RightMargin")];
+	if (tempVal != ""){
+		spnRight->Position = GetSpinPos(atof(tempVal.c_str()));
+          editRight->Text = roundMargin(atof(tempVal.c_str()));
+     }
+
+     //**** Top Margin ****//
+	tempVal = Form1->optionsconf->Sections["PrintedPage"][("TopMargin")];
+	if (tempVal != ""){
+		spnTop->Position = GetSpinPos(atof(tempVal.c_str()));
+          editTop->Text = roundMargin(atof(tempVal.c_str()));
+     }
+
+     //**** Bottom Margin ****//
+	tempVal = Form1->optionsconf->Sections["PrintedPage"][("BottomMargin")];
+	if (tempVal != ""){
+		spnBottom->Position = GetSpinPos(atof(tempVal.c_str()));
+          editBottom->Text = roundMargin(atof(tempVal.c_str()));
+     }
+
+	//**** Font Name ****//
+	tempVal = Form1->optionsconf->Sections["PrintedPage"][(modType + "FontName")];
+	if (tempVal != "")
+		cmbFont->ItemIndex = cmbFont->Items->IndexOf(tempVal.c_str());
+
+/*
 	if(radText->Checked){
 		ckBookHead->Checked = true;
 		ckChHead->Checked = true;
@@ -442,6 +531,60 @@ void TPrintForm::SetDefaultOps()
 		ckPrefix->Checked = false;
 		ckPostfix->Checked = false;
 	}
+*/
 	 
 }
 
+ /* TODO -oDavid -cPrinting : Find a good place to call this */
+void TPrintForm::SavePrintPrefs()
+{
+	ConfigEntMap emap;
+	SWConfig* config = Form1->optionsconf;
+	SWBuf type;
+	if(radText->Checked){
+		type = "Text";
+	}else if(radComm->Checked){
+		type = "Comment";
+	}else if(radLD->Checked){
+		type = "LD";
+	}
+
+	emap = config->Sections["PrintedPage"];
+	emap[(type + "BkName")]		= IntToStr((ckBookHead->Checked)?1:0).c_str();
+	emap[type + "ChHead"]		= IntToStr((ckChHead->Checked)?1:0).c_str();
+	emap[type + "Paragraph"]		= IntToStr((ckParagraph->Checked)?1:0).c_str();
+	emap[type + "VSNum"]		= IntToStr((ckVSNum->Checked)?1:0).c_str();
+	emap[type + "SuperVS"]		= IntToStr((ckSuperVS->Checked)?1:0).c_str();
+	emap[type + "PreFixTag"]		= IntToStr((ckPrefix->Checked)?1:0).c_str();
+	emap[type + "PostFixTag"]	= IntToStr((ckPostfix->Checked)?1:0).c_str();
+	emap[type + "ChHeadSize"]	= editHeading->Text.c_str();
+	emap[type + "FontSize"]		= editBody->Text.c_str();
+	emap[type + "VSNumSize"]		= editNumbers->Text.c_str();
+	emap[type + "FontName"]		= cmbFont->Text.c_str();
+	// Non text type dependent settings
+	emap["LeftMargin"]			= editLeft->Text.c_str();
+	emap["RightMargin"]			= editRight->Text.c_str();
+	emap["TopMargin"]			= editTop->Text.c_str();
+	emap["BottomMargin"]		= editBottom->Text.c_str();
+	emap["PrnHeader"]			= IntToStr((ckHeader->Checked)?1:0).c_str();
+	emap["PrnFooter"]			= IntToStr((ckFooter->Checked)?1:0).c_str();
+	emap["Header"]				= editHeader->Text.c_str();
+	emap["Footer"]				= editFooter->Text.c_str();
+	config->Sections["PrintedPage"] = emap;
+	config->Save();
+}
+
+
+int inline TPrintForm::GetSpinPos(const float & value)
+{
+        return ((int)(roundMargin(value) / .05));
+}
+
+float TPrintForm::roundMargin(float value)
+{
+        value *= 100; 			// Move the decimal right two positions
+        int nVal = Ceil(value);	// Round up
+        if(nVal % 5)
+        	nVal = nVal - (nVal % 5) + 5;
+        return nVal / 100.00;		// Move decimal left two positions
+}
