@@ -121,7 +121,7 @@ void __fastcall TRangeMaintForm::Button1Click(TObject *Sender)
 	for (int i = 0; i < verses.Count(); i++) {
 		VerseKey *element = SWDYNAMIC_CAST(VerseKey, verses.GetElement(i));
 		if (element) {
-			ListBox2->Items->Add((string(element->LowerBound()) + " - " + string(element->UpperBound())).c_str());
+			ListBox2->Items->Add((SWBuf(element->LowerBound()) + " - " + SWBuf(element->UpperBound())).c_str());
 		}
 		else	ListBox2->Items->Add((const char *)*verses.GetElement(i));
 	}
