@@ -52,8 +52,8 @@ public:		// User declarations
 
 class TCustomRange : public TObject {
 public:
-	string name;
-	string text;
+	SWBuf name;
+	SWBuf text;
 	TCustomRange(const char *name, const char *text) {
 		this->name = name;
 		this->text = text;
