@@ -4,12 +4,14 @@
 #pragma hdrstop
 
 #include "RxRichEditX.h"
+#include "PrintFrm.h"
 #include <clipbrd.hpp>
 #include <swdisprtf.h>
 #include <swdisprtfchap.h>
 #include "mainfrm.h"
 #include <localemgr.h>
 #include <unicodertf.h>
+#include <utilxml.h>
 
 char TRxRichEditX::platformID = 0;
 
@@ -23,210 +25,68 @@ TRxRichEditX::TRxRichEditX(TWinControl *parent) : TRxRichEdit(parent)
 	type = "Default";
 }
 
-void TRxRichEditX::RenderModule(SWModule* module, RTF_FORMAT format)
+void TRxRichEditX::RenderModule(SWModule* module/*, RTF_FORMAT format*/)
 {
-	format_ops = format;
-
-    TMemoryStream* RTFStream = new TMemoryStream();
-    AnsiString rtfText = "{\\rtf1\\ansi{\\fonttbl{\\f0\\fdecor\\fprq2 ";
-    rtfText += format_ops.fontFace;
-    rtfText += ";}}{\\colortbl;\\red0\\green0\\blue0;}";
-    rtfText += "{\\fs8\\cf1\\par\\pard} ";
-
-    //if(!strcmp(module->Type(), "Biblical Texts"))
-	    rtfText += GetFormattedBible(module);
-    /*
-    else if(!strcmp(module->Type(), "Commentaries"))
-    	rtfText += GetFormattedComm(module);
-    else if(!strcmp(module->Type(), "Lexicons / Dictionaries"))
-	rtfText += GetFormattedLD(module);
-    else
-	rtfText += GetFormattedBook(module);
-    */
-    rtfText += "{ \\par }}";
-    RTFStream->Clear();
-    RTFStream->WriteBuffer(rtfText.c_str(), rtfText.Length());
-    RTFStream->Position = 0;
-    Lines->LoadFromStream(RTFStream);
-    delete RTFStream;
-
-	// clear HTML link tags
-	while (true) {
-		int start, len, foundAt, endAt;
-
-		start = (SelLength) ? SelStart + SelLength : 0;
-		len = Text.Length() - start;
-		foundAt = this->SearchText("<a href=\"\">", start, len, TRichSearchTypes());
-		if (foundAt == -1)
-			break;
-
-		SelStart = foundAt;
-		SelLength = 11;
-		this->SelText = "";
-		endAt = this->SearchText("</a>", foundAt, len, TRichSearchTypes());
-		if (foundAt == -1)
-			break;
-		SelStart = endAt;
-		SelLength = 4;
-		this->SelText = "";
-		SelStart = foundAt;
-		SelLength = endAt - foundAt;
-		//this->SelAttributes->Link = true;
-	}
-		SelLength = 0;
+	//format_ops = format;
+	this->module = module;
+	TMemoryStream* RTFStream = new TMemoryStream();
+	recalcAppearance();
+	AnsiString rtfText = RTFHeader;
+	rtfText += RTFHeadMargin;
+	rtfText += GetFormattedText(module);
+	rtfText += RTFTrailer;
+	RTFStream->Clear();
+	RTFStream->WriteBuffer(rtfText.c_str(), rtfText.Length());
+	RTFStream->Position = 0;
+	Lines->LoadFromStream(RTFStream);
+	delete RTFStream;
+	clearHTMLTags();
 }
 
-AnsiString TRxRichEditX::GetFormattedBible(SWModule* module)
+
+
+AnsiString TRxRichEditX::GetFormattedText(SWModule* module)
 {
-    AnsiString rtfText = "";
+	AnsiString rtfText = "";
 	VerseKey key;
-    int lastChap, lastBook;
-    AnsiString bookName;
+	int lastChap, lastBook;
+	AnsiString bookName;
 
-    key = (VerseKey)module->Key();
-    lastChap = key.Chapter();
-    lastBook = key.Book();
-    bookName = (AnsiString)key.getText();
-    bookName = bookName.SubString(0,bookName.Pos(lastChap) - 2);
-
-    if(format_ops.prBookHeadings)
-    	rtfText += BookHeading(module, bookName);
-
-    if(format_ops.prChHeadings)
-	    rtfText += ChapterHeading(module, lastChap);
-
-	for ((*module) = TOP; !module->Error(); (*module)++){
 	key = (VerseKey)module->Key();
-        if(key.Book() != lastBook){
-	        rtfText += "\\par\\par";
-	        bookName = (AnsiString)key.getText();
-            bookName = bookName.SubString(0,bookName.Pos(key.Chapter()) - 2);
-    	    if(format_ops.prBookHeadings)
-	    		rtfText += BookHeading(module, bookName);
-		  lastBook = key.Book();
-            lastChap = -1; // Reset the last Chapter since we could jump from Jude 1 to Revelation 1
-        }
-        if(key.Chapter() != lastChap){
-        	rtfText += "\\par";
-            if(format_ops.prChHeadings)
-	            rtfText += ChapterHeading(module, key.Chapter());
-            lastChap = key.Chapter();
-        }
-        rtfText += PrintEntry(module, key.Verse());
-    }
+	lastChap = key.Chapter();
+	lastBook = key.Book();
+	bookName = (AnsiString)key.getText();
+	bookName = bookName.SubString(0,bookName.Pos(lastChap) - 2);
 
-    return rtfText;
-}
-/*
-AnsiString TRxRichEditX::GetFormattedComm(SWModule* module)
-{
-    AnsiString rtfText = "";
-	VerseKey key;
-    int lastChap;
-    AnsiString lastBook;
+	if(dispAttribs.prBookHeadings)
+	rtfText += BookHeading(module, bookName);
 
-    key = (VerseKey)module->Key();
-    lastChap = key.Chapter();
-    lastBook = (AnsiString)key.Book();
-
-    if(format_ops.prBookHeadings)
-    	rtfText += BookHeading(module, lastBook);
-
-    if(format_ops.prChHeadings)
+	if(dispAttribs.prChHeadings)
 	    rtfText += ChapterHeading(module, lastChap);
 
 	for ((*module) = TOP; !module->Error(); (*module)++){
-    	key = (VerseKey)module->Key();
-        if((AnsiString)key.Book() == lastBook){
-    	    if(format_ops.prBookHeadings)
-	    		rtfText += BookHeading(module, lastBook);
-            lastBook = key.Book();
-        }
-        if(key.Chapter() != lastChap){
-        	rtfText += "\\par\\par";
-            if(format_ops.prChHeadings)
-	            rtfText += ChapterHeading(module, key.Chapter());
-            lastChap = key.Chapter();
-        }
-        rtfText += PrintEntry(module, key.Verse());
-    }
+		key = (VerseKey)module->Key();
+		if(key.Book() != lastBook){
+			rtfText += "\\par\\par";
+			bookName = (AnsiString)key.getText();
+			bookName = bookName.SubString(0,bookName.Pos(key.Chapter()) - 2);
+			if(dispAttribs.prBookHeadings)
+				rtfText += BookHeading(module, bookName);
+		lastBook = key.Book();
+		lastChap = -1; // Reset the last Chapter since we could jump from Jude 1 to Revelation 1
+		}
+		if(key.Chapter() != lastChap){
+			rtfText += "\\par";
+			if(dispAttribs.prChHeadings)
+				rtfText += ChapterHeading(module, key.Chapter());
+			lastChap = key.Chapter();
+		}
+		rtfText += PrintEntry(module, key.Verse());
+	}
 
     return rtfText;
 }
 
-AnsiString TRxRichEditX::GetFormattedLD(SWModule* module)
-{
-    AnsiString rtfText = "";
-	VerseKey key;
-    int lastChap;
-    AnsiString lastBook;
-
-    key = (VerseKey)module->Key();
-    lastChap = key.Chapter();
-    lastBook = (AnsiString)key.Book();
-
-    if(format_ops.prBookHeadings)
-    	rtfText += BookHeading(module, lastBook);
-
-    if(format_ops.prChHeadings)
-	    rtfText += ChapterHeading(module, lastChap);
-
-	for ((*module) = TOP; !module->Error(); (*module)++){
-    	key = (VerseKey)module->Key();
-        if((AnsiString)key.Book() == lastBook){
-    	    if(format_ops.prBookHeadings)
-	    		rtfText += BookHeading(module, lastBook);
-            lastBook = key.Book();
-        }
-        if(key.Chapter() != lastChap){
-        	rtfText += "\\par\\par";
-            if(format_ops.prChHeadings)
-	            rtfText += ChapterHeading(module, key.Chapter());
-            lastChap = key.Chapter();
-        }
-        rtfText += PrintEntry(module, key.Verse());
-    }
-
-    return rtfText;
-}
-
-AnsiString TRxRichEditX::GetFormattedBook(SWModule* module)
-{
-    AnsiString rtfText = "";
-	VerseKey key;
-    int lastChap;
-    AnsiString lastBook;
-
-    key = (VerseKey)module->Key();
-    lastChap = key.Chapter();
-    lastBook = (AnsiString)key.Book();
-
-    if(format_ops.prBookHeadings)
-    	rtfText += BookHeading(module, lastBook);
-
-    if(format_ops.prChHeadings)
-	    rtfText += ChapterHeading(module, lastChap);
-
-	for ((*module) = TOP; !module->Error(); (*module)++){
-    	key = (VerseKey)module->Key();
-        if((AnsiString)key.Book() == lastBook){
-	        rtfText += "\\par\\par";
-    	    if(format_ops.prBookHeadings)
-	    		rtfText += BookHeading(module, lastBook);
-            lastBook = key.Book();
-        }
-        if(key.Chapter() != lastChap){
-        	rtfText += "\\par\\par";
-            if(format_ops.prChHeadings)
-	            rtfText += ChapterHeading(module, key.Chapter());
-            lastChap = key.Chapter();
-	   }
-        rtfText += PrintEntry(module, key.Verse());
-    }
-
-    return rtfText;
-}
-*/
 AnsiString TRxRichEditX::PrintEntry(SWModule* module, int nVerse)
 {
 	AnsiString rtfText = "";
@@ -238,23 +98,23 @@ AnsiString TRxRichEditX::PrintEntry(SWModule* module, int nVerse)
 		rtfText = rtfText + "\\rtlpar ";
 	}
 
-	if(!format_ops.paragraph)
+	if(!dispAttribs.paragraph)
 		rtfText += "\\par";
-	if(format_ops.prPreFix)
+	if(dispAttribs.prPreFix)
 		rtfText += PrintFix(module) + (AnsiString)" ";
 
-	if(format_ops.prVerseNum){
-		rtfText += (AnsiString)"{\\fs" + format_ops.vsNumFontSize;
-		if(format_ops.superVSNum)
+	if(dispAttribs.prVerseNum){
+		rtfText += (AnsiString)"{\\fs" + (dispAttribs.vsNumFontSize * 2);
+		if(dispAttribs.superVSNum)
 			rtfText += "\\super ";
 		else rtfText += " ";
 		rtfText += IntToStr(nVerse);
 		rtfText += " }";
 	}
-	rtfText += (AnsiString)"{\\fs" + format_ops.bodyFontSize + (AnsiString)" ";
+	rtfText += (AnsiString)"{\\fs" + (dispAttribs.fontSize * 2) + (AnsiString)" ";
 	rtfText += module->RenderText();
 	rtfText += " }";
-	if(format_ops.prPostFix)
+	if(dispAttribs.prPostFix)
 	rtfText += (AnsiString)" " + PrintFix(module);
 
 	return rtfText;
@@ -263,7 +123,7 @@ AnsiString TRxRichEditX::PrintEntry(SWModule* module, int nVerse)
 AnsiString TRxRichEditX::ChapterHeading(SWModule* module, int nChapter)
 {
 	AnsiString rtfText = "";
-	rtfText += (AnsiString)" \\qc{\\f1\\cf7\\fs" + format_ops.bkFontHeadSize + (AnsiString)"\\b ";
+	rtfText += (AnsiString)" \\qc{\\f1\\cf7\\fs" + (dispAttribs.chFontHeadSize * 2) + (AnsiString)"\\b ";
 	rtfText += _tr("Chapter");
 	rtfText += " ";
 	rtfText += IntToStr(nChapter);
@@ -274,7 +134,7 @@ AnsiString TRxRichEditX::ChapterHeading(SWModule* module, int nChapter)
 AnsiString TRxRichEditX::BookHeading(SWModule* module, AnsiString name)
 {
 	AnsiString rtfText = "";
-	rtfText += (AnsiString)" \\qc{\\f1\\cf7\\fs" + format_ops.bkFontHeadSize + (AnsiString)"\\b\\ul ";
+	rtfText += (AnsiString)" \\qc{\\f1\\cf7\\fs" + (dispAttribs.chFontHeadSize * 2) + (AnsiString)"\\b\\ul ";
 	rtfText += name;
 	rtfText += "\\par\\par}";
 	return rtfText;
@@ -284,7 +144,7 @@ AnsiString TRxRichEditX::BookHeading(SWModule* module, AnsiString name)
 AnsiString TRxRichEditX::PrintFix(SWModule* module)
 {
 	AnsiString rtfText = "";
-	rtfText += (AnsiString)"{\\fs" + format_ops.vsNumFontSize;
+	rtfText += (AnsiString)"{\\fs" + (dispAttribs.vsNumFontSize * 2);
 	rtfText += (AnsiString)"(" + (AnsiString)module->KeyText() + (AnsiString)" " + (AnsiString)module->Name() + (AnsiString)")}";
 	return rtfText;
 }
@@ -322,7 +182,7 @@ int TRxRichEditX::paintTo(HDC hdc, TRect *size, int margin) {
 	LogY = GetDeviceCaps(hdc, LOGPIXELSY);
 
 	do {
-	
+
 		if (calcBestSize) {
 			dispSize = &trySizeRect[trySize++];
 			if (IsRectEmpty(dispSize)) {
@@ -330,7 +190,7 @@ int TRxRichEditX::paintTo(HDC hdc, TRect *size, int margin) {
 				break;
 			}
 		}
-		
+
 		memset(&Range, 0, sizeof(TFormatRange));
 	//  with Printer, Range do begin
 		Range.hdc = hdc;
@@ -378,20 +238,20 @@ int TRxRichEditX::paintTo(HDC hdc, TRect *size, int margin) {
 
 	if (calcBestSize && size)
 		*size = *dispSize;
-		
+
 	return LastChar;
 }
 
 
 void TRxRichEditX::fillWithRTFString(SWModule *module, const char *text, const char *type) {
-	
+
 	TMemoryStream *RTFStream = new TMemoryStream();
 	System::AnsiString newtext, tmptext;
 
 	this->module = module;
 	this->type = type;
 	recalcAppearance();
-	
+
 	newtext = RTFHeader;
 	newtext += RTFHeadMargin;
 	newtext += "\\pard \\nowidctlpar \\cf7\\f0 ";
@@ -409,7 +269,7 @@ void TRxRichEditX::fillWithRTFString(SWModule *module, const char *text, const c
 	newtext += "}";
 
 	newtext += RTFTrailer;
-	
+
 	RTFStream->Clear();
 	RTFStream->WriteBuffer(newtext.c_str(), newtext.Length());
 	RTFStream->Position = 0;
@@ -420,17 +280,18 @@ void TRxRichEditX::fillWithRTFString(SWModule *module, const char *text, const c
 }
 
 
-void TRxRichEditX::fillWithVerses(SWModule *module, ListKey *verses, bool heading, bool verseNum, const char *type, bool stripNewlines) {
-	string fontname;
+bool TRxRichEditX::fillWithVerses(SWModule *module, ListKey *verses, const char *introText, bool heading, bool verseNum, const char *type, bool stripNewlines) {
+	SWBuf fontname;
 	TMemoryStream *RTFStream = new TMemoryStream();
 	System::AnsiString newtext, tmptext;
 	static UnicodeRTF filter;
-	char buf[255];
+	SWBuf buf;
+	bool atLeastOneEntry = false;
 
 	this->module = module;
 	this->type = type;
 	recalcAppearance();
-	
+
 	newtext = RTFHeader;
 
 	newtext += RTFHeadMargin;
@@ -442,7 +303,7 @@ void TRxRichEditX::fillWithVerses(SWModule *module, ListKey *verses, bool headin
     if (module->Direction() == DIRECTION_RTL && (platformID == WINNT && (!strnicmp(module->Lang(), "he", 2) || !strnicmp(module->Lang(), "ar", 2)))) {
 		newtext += "\\rtlpar ";
     }
-	
+
 	verses->Persist(1);
 	(*verses) = TOP;
 	SWKey *saveKey = (SWKey *)(*module);
@@ -451,6 +312,13 @@ void TRxRichEditX::fillWithVerses(SWModule *module, ListKey *verses, bool headin
 		saveKey = 0;
 	module->setKey(verses);
 
+
+	if (introText) {
+		newtext += RTFHeadingPre;
+		newtext += introText;
+		newtext += RTFHeadingPost;
+		newtext += "{\\par ________ \\par\\par}";
+	}
 	SWKey *lastKey = 0;
 	bool first = true;
 	SWKey *testKey = module->CreateKey();
@@ -469,15 +337,16 @@ void TRxRichEditX::fillWithVerses(SWModule *module, ListKey *verses, bool headin
 			}
 			else	{
 				char *buf2 = 0;
-				stdstr(&buf2, (const char *)*verses);
-				toupperstr_utf8(buf2);
+				stdstr(&buf2, (const char *)*verses, 2);
+				toupperstr_utf8(buf2, strlen(buf2)*2);
 				module->getRawEntry();
-				strcpy(buf, module->KeyText());
-				toupperstr_utf8(buf);
+				char *buf3 = 0;
+				stdstr(&buf3, (const char *)module->KeyText(), 2);
+				toupperstr_utf8(buf3, strlen(buf3)*2);
 				int i;
-				char *start1 = buf;
+				char *start1 = buf3;
 				char *start2 = buf2;
-				int len = strlen(buf);
+				int len = strlen(buf3);
 				for (i = 0; i < len; i++) {
 					if (*start1 == '0') {
 						start1++;
@@ -496,22 +365,25 @@ void TRxRichEditX::fillWithVerses(SWModule *module, ListKey *verses, bool headin
 				}
 				if (!i || i < (strlen(start1)/2)) {
 					delete [] buf2;
+					delete [] buf3;
 					verses->Remove();
 					continue;
 				}
 				delete [] buf2;
+				delete [] buf3;
 				if (!first)
 					newtext += "{\\par\\par}";
 				newtext += RTFHeadingPre;
-				
-				strcpy(buf, module->KeyText());
+
+				buf = module->KeyText();
 				// VerseKey locales are not yet UTF8, so don't try to convert them.
 				if (!SWDYNAMIC_CAST(VerseKey, testKey))
-					filter.ProcessText(buf, 253, *module, module);
-				newtext = newtext + RTFHeadingPre + buf + RTFHeadingPost + ":\\par ";
+					filter.processText(buf, *module, module);
+				newtext = newtext + RTFHeadingPre + buf.c_str() + RTFHeadingPost + ":\\par ";
 
 				newtext += RTFHeadingPost;
 				newtext += "{\\par}";
+				atLeastOneEntry = true;
 			}
 //			newtext += RTFHeadingPost;
 		}
@@ -520,11 +392,11 @@ void TRxRichEditX::fillWithVerses(SWModule *module, ListKey *verses, bool headin
 		}
 		lastKey = verses->GetElement();
 		first = false;
-		
+
 		newtext = newtext + "{";
 
 		tmptext = "";
-		for (const char *loop = (const char *)*module; *loop; loop++) {
+		for (const char *loop = module->RenderText(); *loop; loop++) {
 			if (stripNewlines) {
 				if (!strnicmp(loop, "\\par", 4)) {
 					loop += (loop[4] == 'd') ? 4 : 3;
@@ -536,7 +408,7 @@ void TRxRichEditX::fillWithVerses(SWModule *module, ListKey *verses, bool headin
 					tmptext += "{\\par}";
 			}
 			else tmptext += *loop;
-			
+
 		}
 
 		if (tmptext.Length() > 3) {	// make sure we have an entry
@@ -561,40 +433,20 @@ void TRxRichEditX::fillWithVerses(SWModule *module, ListKey *verses, bool headin
 	else module->setKey(origKeyValue);	//remove our persist key
 
 	newtext += RTFTrailer;
-	
+
 	RTFStream->Clear();
 	RTFStream->WriteBuffer(newtext.c_str(), newtext.Length());
 	RTFStream->Position = 0;
 	Lines->LoadFromStream(RTFStream);
 
-	// make links
-	while (true) {
-		int start, len, foundAt, endAt;
-
-		start = (SelLength) ? SelStart + SelLength : 0;
-		len = Text.Length() - start;
-		foundAt = this->SearchText("<a href=\"\">", start, len, TRichSearchTypes());
-		if (foundAt == -1)
-			break;
-
-		SelStart = foundAt;
-		SelLength = 11;
-		this->SelText = "";
-		endAt = this->SearchText("</a>", foundAt, len, TRichSearchTypes());
-		if (foundAt == -1)
-			break;
-		SelStart = endAt;
-		SelLength = 4;
-		this->SelText = "";
-		SelStart = foundAt;
-		SelLength = endAt - foundAt;
-		this->SelAttributes->Link = true;
-	}
+	makeLinks();
+	makeImages();
+	
 	if (Visible) {
 		TComponent *owner = this->Owner;
 		TForm *parentForm = dynamic_cast<TForm *>(owner);
 		TWinControl *focus = 0;
-		
+
 		if (parentForm)
 			focus = parentForm->ActiveControl;
 		this->SetFocus();
@@ -608,90 +460,202 @@ void TRxRichEditX::fillWithVerses(SWModule *module, ListKey *verses, bool headin
 	Repaint();
 
 	delete RTFStream;
+	return atLeastOneEntry;
 }
 
 
 
-void TRxRichEditX::getDisplayPrefs(DISP_ATTRIBS *attribs, SWModule *module /*, const char *type*/) {
+void TRxRichEditX::getDisplayPrefs(DISP_ATTRIBS *attribs, SWModule *module )
+{
+	AnsiString fontSize;
+	SWBuf 	keyFontName,
+			keyFontSize;
+	ConfigEntMap::const_iterator eit;
+	SWBuf modType = (SWBuf)getType().c_str();
 
+	// If we are printing to a page rather than the screen we want to grab the
+	// correct settings from that section of the conf file.
+	SWBuf dispType = (type == "PrintedPage") ? "PrintedPage" : "Appearance";
 
-	string keyBackColor = (string)getType().c_str() + "BackColor";
-	string backColor = Form1->optionsconf->Sections["Appearance"][keyBackColor];	
-	// This portion of the code sets the default values (If needed) of the background 
+	//**** Font Name ****//
+	// If the module uses it's own font load that else we will try the config file
+	// else we will use the default
+	while (true) {
+		if (module) {
+			eit = module->getConfig().find("Font");
+			if (eit != module->getConfig().end()) {
+				attribs->fontName = (*eit).second.c_str();
+				break;
+			}
+		}
+		keyFontName = modType + "FontName";
+		attribs->fontName = (AnsiString)Form1->optionsconf->Sections[dispType][keyFontName].c_str();
+		break;
+	}
+	// If we still have no name we will set it to a default value
+	if (attribs->fontName == "")
+		attribs->fontName = "Times New Roman";
+
+	//**** Font Size ****//
+	// If the module uses it's own font size load that else we will try the config
+	// file else we will use the default
+	while (true) {
+		if (module) {
+			eit = module->getConfig().find("FontSize");
+			if (eit != module->getConfig().end()) {
+				fontSize = (*eit).second.c_str();
+				break;
+			}
+		}
+		keyFontSize = modType + "FontSize";
+		fontSize = (AnsiString)Form1->optionsconf->Sections[dispType][keyFontSize].c_str();
+		break;
+	}
+	
+	// If we still have no size we will set it to a default value
+	if (fontSize == "")
+		attribs->fontSize = 10;
+	else attribs->fontSize = StrToInt(fontSize);
+
+	//**** Chapter Heading Font Size ****//
+	fontSize = "";
+	while (true) {
+		if (module) {
+			eit = module->getConfig().find("ChHeadSize");
+			if (eit != module->getConfig().end()) {
+				fontSize = (*eit).second.c_str();
+				break;
+			}
+		}
+		keyFontSize = modType + "ChHeadSize";
+		fontSize = (AnsiString)Form1->optionsconf->Sections[dispType][keyFontSize].c_str();
+		break;
+	}
+	
+	// If we still have no size we will set it to a default value
+	if (fontSize == "")
+		attribs->chFontHeadSize = 16;
+	else attribs->chFontHeadSize = StrToInt(fontSize);
+
+	//**** Verse Numbering Font Size ****//
+	fontSize = "";
+	while (true) {
+		if (module) {
+			eit = module->getConfig().find("VSNumSize");
+			if (eit != module->getConfig().end()) {
+				fontSize = (*eit).second.c_str();
+				break;
+			}
+		}
+		keyFontSize = modType + "VSNumSize";
+		fontSize = (AnsiString)Form1->optionsconf->Sections[dispType][keyFontSize].c_str();
+		break;
+	}
+	// If we still have no size we will set it to a default value
+	if (fontSize == "")
+		attribs->vsNumFontSize = 10;
+	else attribs->vsNumFontSize = StrToInt(fontSize);
+
+	//**** Main Background Color ****//
+	SWBuf keyBackColor = modType + "BackColor";
+	SWBuf backColor = Form1->optionsconf->Sections[dispType][keyBackColor];
+	// This portion of the code sets the default values (If needed) of the background
 	// for either typical windows or for popup windows
 	if (backColor == "") {
-		if (getType() == "Popup") {
+		if (modType == "Popup") {
 			attribs->backColor = 14680063;
 		}
 		else attribs->backColor = clWindow;
 	}
 	else attribs->backColor = StrToInt((AnsiString)backColor.c_str());
 
-	string keyFontColor = (string)getType().c_str() + "FontColor";
-	string fontColor = Form1->optionsconf->Sections["Appearance"][keyFontColor];	
-	// This portion of the code sets the default values (If needed) of the background 
+	//**** Main Font Color ****//
+	SWBuf keyFontColor = modType + "FontColor";
+	SWBuf fontColor = Form1->optionsconf->Sections[dispType][keyFontColor];
+	// This portion of the code sets the default values (If needed) of the background
 	// for either typical windows or for popup windows
 	if (backColor == "") {
 		attribs->fontColor = clBlack;
 	}
 	else attribs->fontColor = StrToInt((AnsiString)fontColor.c_str());
 
-	// If the module uses it's own font load that else we will try the config file else we will use the default
-	ConfigEntMap::const_iterator eit = module->getConfig().find("Font");
-	if (eit != module->getConfig().end())
-		attribs->fontName = (*eit).second.c_str();
-	else {
-		string keyFontName = (string)getType().c_str() + "FontName";
-		attribs->fontName = (AnsiString)Form1->optionsconf->Sections["Appearance"][keyFontName].c_str();
-	}
-	// If we still have no name we will set it to a default value
-	if (attribs->fontName == "")
-		attribs->fontName = "Times New Roman";
-
-
-	// If the module uses it's own font size load that else we will try the config file else we will use the default
-     AnsiString fontSize;
-	eit = module->getConfig().find("FontSize");
-	if (eit != module->getConfig().end())
-		fontSize = (*eit).second.c_str();
-	else {
-		string keyFontSize = (string)getType().c_str() + "FontSize";
-		fontSize = (AnsiString)Form1->optionsconf->Sections["Appearance"][keyFontSize].c_str();
-	}
-	// If we still have no size we will set it to a default value
-	if (fontSize == "")
-		attribs->fontSize = 10;
-	else attribs->fontSize = StrToInt(fontSize);
-
-	string entryColor = Form1->optionsconf->Sections["Appearance"]["VSNumberColor"];
+	//**** EntryKey Color ****//
+	SWBuf entryColor = Form1->optionsconf->Sections[dispType]["VSNumberColor"];
 	if (entryColor == "")
 		attribs->entryKeyColor = clBlue;
 	else attribs->entryKeyColor = StrToInt((AnsiString)entryColor.c_str());
 
-	string currentVSColor = Form1->optionsconf->Sections["Appearance"]["CurrentVSColor"];
+	//**** Current Verse Color ****//
+	SWBuf currentVSColor = Form1->optionsconf->Sections[dispType]["CurrentVSColor"];
 	if (currentVSColor == "")
 		attribs->currentVSColor = clBlue;
 	else attribs->currentVSColor = StrToInt((AnsiString)currentVSColor.c_str());
 
-	string morphColor = Form1->optionsconf->Sections["Appearance"]["MorphColor"];
+	//**** Morphological Numbers Color ****//
+	SWBuf morphColor = Form1->optionsconf->Sections[dispType]["MorphColor"];
 	if (morphColor == "")
 		attribs->morphColor = clBlue;
 	else attribs->morphColor = StrToInt((AnsiString)morphColor.c_str());
-	
-	string strongColor = Form1->optionsconf->Sections["Appearance"]["StrongsColor"];
+
+	//**** Strongs Numbers Color ****//
+	SWBuf strongColor = Form1->optionsconf->Sections[dispType]["StrongsColor"];
 	if (strongColor == "")
 		attribs->strongsColor = clBlue;
 	else attribs->strongsColor = StrToInt((AnsiString)strongColor.c_str());
 
-	string fieldColor = Form1->optionsconf->Sections["Appearance"]["FieldColor"];
+	//**** Field Lookup Color ****//
+	SWBuf fieldColor = Form1->optionsconf->Sections[dispType]["FieldColor"];
 	if (fieldColor == "")
 		attribs->lookupFieldColor = clBlue;
 	else attribs->lookupFieldColor = StrToInt((AnsiString)fieldColor.c_str());
 
-	string autoVSColor = Form1->optionsconf->Sections["Appearance"]["AutoVSColor"];
+	//**** Mark Current Verse? ****//
+	SWBuf autoVSColor = Form1->optionsconf->Sections[dispType]["AutoVSColor"];
 	if (autoVSColor == "")
 		attribs->markCurrentVerse = true;
 	else	attribs->markCurrentVerse = (atoi(autoVSColor.c_str())) ? true : false;
+
+	//**** Print Chapter Headings? ****//
+	SWBuf prChHeadings = Form1->optionsconf->Sections[dispType][(modType + "ChHead")];
+	if (prChHeadings == "")
+		attribs->prChHeadings = true;
+	else	attribs->prChHeadings = (atoi(prChHeadings.c_str())) ? true : false;
+
+	//**** Print Book Name? ****//
+	SWBuf prBookHeadings = Form1->optionsconf->Sections[dispType][(modType +"BkName")];
+	if (prBookHeadings == "")
+		attribs->prBookHeadings = true;
+	else	attribs->prBookHeadings = (atoi(prBookHeadings.c_str())) ? true : false;
+
+	//**** Prefix Each Verse? ****//
+	SWBuf prPreFix = Form1->optionsconf->Sections[dispType][(modType +"PreFixTag")];
+	if (prPreFix == "")
+		attribs->prPreFix = true;
+	else	attribs->prPreFix = (atoi(prPreFix.c_str())) ? true : false;
+
+	//**** Postfix Each Verse? ****//
+	SWBuf prPostFix = Form1->optionsconf->Sections[dispType][(modType +"PostFixTag")];
+	if (prPostFix == "")
+		attribs->prPostFix = true;
+	else	attribs->prPostFix = (atoi(prPostFix.c_str())) ? true : false;
+
+	//**** Print the verse number? ****//
+	SWBuf prVerseNum = Form1->optionsconf->Sections[dispType][(modType + "VSNum")];
+	if (prVerseNum == "")
+		attribs->prVerseNum = true;
+	else	attribs->prVerseNum = (atoi(prVerseNum.c_str())) ? true : false;
+
+	//**** Superscript the verse number? ****//
+	SWBuf superVSNum = Form1->optionsconf->Sections[dispType][(modType + "SuperVS")];
+	if (superVSNum == "")
+		attribs->superVSNum = true;
+	else	attribs->superVSNum = (atoi(superVSNum.c_str())) ? true : false;
+
+	//**** Print as a paragraph? ****//
+	SWBuf paragraph = Form1->optionsconf->Sections[dispType][(modType + "Paragraph")];
+	if (paragraph == "")
+		attribs->paragraph = true;
+	else	attribs->paragraph = (atoi(paragraph.c_str())) ? true : false;
 }
 
 
@@ -709,7 +673,6 @@ void TRxRichEditX::recalcAppearance() {
 	RTFHeadMargin = "{\\cf1\\pard}";
 	RTFVerseMarkPost = "}";
 	RTFVersePost = " }";
-
 	buildRTFHeader();
 
 	// build headers with proportional font sizes to single user selected
@@ -722,13 +685,14 @@ void TRxRichEditX::recalcAppearance() {
 			(int)(fontBase * 8));
 	RTFTrailer = buf;
 
-	char chapBuf[1024];
-	strcpy(chapBuf, _tr("Chapter"));
-	filter.ProcessText(chapBuf, 1022, *module, module);
-	
-	sprintf(buf,
-			"\\pard \\qc\\nowidctlpar{\\f1\\cf7\\fs%d\\b %s ",
-			(int)(fontBase * 10), chapBuf);
+	SWBuf chapBuf;
+	//chapBuf = WideStringToUTF8(_tr("Chapter")).c_str();
+        chapBuf = _tr("Chapter");
+	filter.processText(chapBuf);
+
+        sprintf(buf,
+                        "\\pard \\qc\\nowidctlpar{\\f1\\cf7\\fs%d\\b %s ",
+			(int)(fontBase * 10), chapBuf.c_str());
 	RTFChapterMarkPre  = buf;
 
 	sprintf(buf,
@@ -762,9 +726,9 @@ void TRxRichEditX::TColorToRGB(const TColor& color, int& red, int& green, int& b
 void TRxRichEditX::buildRTFHeader() {
 	char buf1[1024], buf2[1024];
 	SectionMap::iterator sit;
-	string value;
+	SWBuf value;
 	ConfigEntMap::iterator entry;
-	string tmpval;
+	SWBuf tmpval;
 	int CurrVSRed, CurrVSGreen, CurrVSBlue, BodyRed, BodyGreen, BodyBlue,
 			VSNumRed, VSNumGreen, VSNumBlue, MorphRed, MorphGreen, MorphBlue,
 			StrongsRed, StrongsGreen, StrongsBlue;
@@ -833,17 +797,26 @@ void TRxRichEditX::buildRTFHeader() {
 
 
 AnsiString TRxRichEditX::getType() {
-	AnsiString retVal;
+	AnsiString retVal = "Default";
 	if (!strcmp(type.c_str(), "Default")) {
-		if (!strcmp(module->Type(), "Biblical Texts"))
-			retVal = "Text";
-		if ((!strcmp(module->Type(), "Commentaries")) ||
-				(!strcmp(module->Type(), "Generic Books")))
-			retVal = "Comment";
-		if (!strcmp(module->Type(), "Lexicons / Dictionaries"))
-			retVal = "LD";
+		if (module) {
+			if (!strcmp(module->Type(), "Biblical Texts"))
+				retVal = "Text";
+			else if ((!strcmp(module->Type(), "Commentaries")) ||
+					(!strcmp(module->Type(), "Generic Books")))
+				retVal = "Comment";
+			else if (!strcmp(module->Type(), "Lexicons / Dictionaries"))
+				retVal = "LD";
+		}
 	}
-	else retVal = type;
+	else if(!strcmp(type.c_str(), "PrintedPage")){
+		if(PrintForm->radText->Checked)
+			retVal = "Text";
+		else if(PrintForm->radComm->Checked)
+			retVal = "Comment";
+		else if(PrintForm->radLD->Checked)
+			retVal = "LD";
+	}else retVal = type;
 	
 	return retVal;
 }
@@ -909,7 +882,7 @@ WideString TRxRichEditX::Trim(WideString &src) {
 		start++;
 	}
 	if (start > 1)
-		Result = Result.SubString(start, length - start);
+		Result = Result.SubString(start, (length - start)+1);
 	return Result;	
 }
 
@@ -917,10 +890,12 @@ WideString TRxRichEditX::Trim(WideString &src) {
 WideString __fastcall TRxRichEditX::WordAtCursor(void) {
 
 	TCharRange Range;
+	WideString TestStr = "";
 	WideString Result = "";
 
 	if (HandleAllocated()) {
 		long max = TextLen() - 1;
+		bool pre = true;
 		for (int i = 0; (SelStart + i) < max; i++) {
 			Range.cpMax = SelStart + i;
 			if (!Range.cpMax)
@@ -931,9 +906,43 @@ WideString __fastcall TRxRichEditX::WordAtCursor(void) {
 			while (SendMessage(Handle, EM_FINDWORDBREAK, WB_ISDELIMITER, Range.cpMin))
 				Range.cpMin++;
 			Range.cpMax = SendMessage(Handle, EM_FINDWORDBREAK, WB_RIGHTBREAK, Range.cpMax);
-			Result = Trim(GetTextRange(Range.cpMin, Range.cpMax));
-			if (Range.cpMax - Range.cpMin)
+			TestStr = Trim(GetTextRange(Range.cpMin, Range.cpMax));
+			if (pre)
+				Result = TestStr;
+			else	Result += TestStr;
+			
+			if (Range.cpMin > 1)
+				Range.cpMin--;
+			TestStr = Trim(GetTextRange(Range.cpMin, Range.cpMax));
+			
+			int testlen = TestStr.Length();
+
+			if (!testlen)
+				continue;
+
+			if ((pre) && (TestStr[1] == '-') && (Range.cpMin > 1)) {
+				if (testlen > 1) {
+					if ((isdigit(TestStr[2])) || (toupper(TestStr[2]) == TestStr[2])) {
+						i = 0;
+						SelStart = Range.cpMin-1;
+						continue;
+					}
+				}
+			}
+			
+			if ((TestStr[testlen] == '-') && (Range.cpMax < (max-1))) {
+				if (testlen > 1) {
+					if ((isdigit(TestStr[testlen-1])) || (toupper(TestStr[testlen-1]) == TestStr[testlen-1])) {
+						i = 0;
+						SelStart = Range.cpMax+1;
+						pre = false;
+						continue;
+					}
+				}
+			}
+			if (Result.Length())
 				break;
+				
 		}
 	}
 	int start = 1;
@@ -959,3 +968,328 @@ WideString __fastcall TRxRichEditX::WordAtCursor(void) {
 	return Result;
 }
 
+
+void TRxRichEditX::makeLinks() {
+	while (true) {
+		int start, len, foundAt, endAt;
+
+		start = (SelLength) ? SelStart + SelLength : 0;
+		len = Text.Length() - start;
+		foundAt = this->SearchText("<a href=\"\">", start, len, TRichSearchTypes());
+		if (foundAt == -1)
+			break;
+
+		SelStart = foundAt;
+		SelLength = 11;
+		this->SelText = "";
+		endAt = this->SearchText("</a>", foundAt, len, TRichSearchTypes());
+		if (foundAt == -1)
+			break;
+		SelStart = endAt;
+		SelLength = 4;
+		this->SelText = "";
+		SelStart = foundAt;
+		SelLength = endAt - foundAt;
+		this->SelAttributes->Link = true;
+	}
+}
+
+
+void TRxRichEditX::clearHTMLTags()
+{
+	while (true) {
+		int start, len, foundAt, endAt;
+
+		start = (SelLength) ? SelStart + SelLength : 0;
+		len = Text.Length() - start;
+		foundAt = this->SearchText("<a href=\"\">", start, len, TRichSearchTypes());
+		if (foundAt == -1)
+			break;
+
+		SelStart = foundAt;
+		SelLength = 11;
+		this->SelText = "";
+		endAt = this->SearchText("</a>", foundAt, len, TRichSearchTypes());
+		if (foundAt == -1)
+			break;
+		SelStart = endAt;
+		SelLength = 4;
+		this->SelText = "";
+		SelStart = foundAt;
+		SelLength = endAt - foundAt;
+		//this->SelAttributes->Link = true;
+	}
+	SelLength = 0;
+}
+
+
+void TRxRichEditX::makeImages() {
+	while (true) {
+		int start, len, foundAt, endAt;
+
+		start = (SelLength) ? SelStart + SelLength : 0;
+		len = Text.Length() - start;
+		foundAt = this->SearchText("<figure src=\"", start, len, TRichSearchTypes());
+		if (foundAt == -1)
+			break;
+
+		endAt = this->SearchText("\" />", foundAt, len, TRichSearchTypes());
+		if (foundAt == -1)
+			break;
+		SelStart = foundAt;
+		SelLength = (endAt - foundAt)+4;
+		XMLTag tag(this->SelText.c_str());
+		SelText = "";
+		insertImage(tag.getAttribute("src"));		
+	}
+}
+
+
+void TRxRichEditX::insertImage(const char *filePath) {
+//static char *tmp = "D:/Program Files/Borland/Borland Shared/Images/Buttons/alarm.bmp";
+//filePath=tmp;
+	TImage *image = new TImage(this);
+	TJPEGImage *ji = new TJPEGImage();
+	Graphics::TBitmap *bitmap = new Graphics::TBitmap();
+	try {
+	int len = strlen(filePath);
+	if ((len > 4) && (!stricmp(filePath+(len-4), ".jpg"))) {
+//		ji->LoadFromFile(filePath);
+
+
+
+	TFileStream* imageFile;
+	imageFile = new TFileStream(filePath,fmOpenRead|fmShareCompat);
+	ji->LoadFromStream(imageFile);		// load the jpeg from the disk file stream
+	bitmap->Assign(ji);
+	image->Picture->Assign(bitmap);
+	delete imageFile;
+
+//		image->Width = ji->Width;
+//		image->Height = ji->Height;
+		image->Canvas->Draw(0, 0, ji);
+//		image->Picture->Bitmap->Assign(ji);
+	}
+	else	image->Picture->LoadFromFile(filePath);
+	
+	insertRTF(bitmapToRTF(image->Picture->Bitmap));
+	}
+	catch (...){}
+	delete ji;
+	delete image;
+	delete bitmap;
+}
+
+
+// EditStreamInCallback callback function
+DWORD __stdcall editStreamInCallBack(unsigned long dwCookie, unsigned char *pbBuff, long cb, long *pcb) {
+  unsigned long dataAvail;
+  TStream *stream = (TStream *)dwCookie;
+  dataAvail = stream->Size - stream->Position;
+  unsigned long retVal = 0;
+    if (dataAvail <= cb) {
+	 *pcb = stream->Read(pbBuff, dataAvail);
+	 if (*pcb != dataAvail)
+	   retVal = E_FAIL;
+    }
+    else {
+	 *pcb = stream->Read(pbBuff, cb);
+	 if (*pcb != cb)
+	   retVal = E_FAIL;
+    }
+    return retVal;
+}
+
+
+// Insert Stream into RichEdit
+void TRxRichEditX::insertRTF(const char *rtf) {
+TStringStream *ss = new TStringStream(rtf);
+
+  try {
+    EDITSTREAM editStream;
+    editStream.dwCookie = (unsigned long)ss;
+    editStream.dwError = 0;
+    editStream.pfnCallback = editStreamInCallBack;
+    Perform(EM_STREAMIN, SF_RTF | SFF_SELECTION, (int)&editStream);
+  }
+  catch (...) {}
+  delete ss;
+}
+
+
+/*
+// Example to insert image from Image1 into RxRichEdit1
+procedure TForm1.Button1Click(Sender: TObject);
+var
+  SS: TStringStream;
+  BMP: TBitmap;
+begin
+  BMP := TBitmap.Create;
+  BMP := Image1.Picture.Bitmap;
+  SS  := TStringStream.Create(BitmapToRTF(BMP));
+  try
+    PutRTFSelection(RxRichEdit1, SS);
+  finally
+    SS.Free;
+  end;
+end;
+*/
+
+
+
+
+// Convert Bitmap to RTF Code
+SWBuf TRxRichEditX::bitmapToRTF(Graphics::TBitmap *pict) {
+	unsigned char *bi, *bb;
+	SWBuf rtf;
+	unsigned int bis, bbs;
+	char achar[10];
+	char *hexpict;
+	int i;
+	GetDIBSizes(pict->Handle, bis, bbs);
+	bi = new char [bis + 1];
+	bb = new char [bbs + 1];
+	GetDIB(pict->Handle, pict->Palette, bi, bb);
+	rtf = "{\\rtf1 {\\pict\\dibitmap ";
+	hexpict = new char[((bbs + bis) * 2)+1];
+	i = 0;
+	for (int x = 0; x < bis; x++) {
+		sprintf(achar, "%02x", (unsigned int)bi[x]);
+		hexpict[i++] = achar[0];
+		hexpict[i++] = achar[1];
+	}
+	
+	for (int x = 0; x < bbs; x++) {
+		sprintf(achar, "%02x", (unsigned int)bb[x]);
+		hexpict[i++] = achar[0];
+		hexpict[i++] = achar[1];
+	}
+	hexpict[i] = 0;
+	rtf += hexpict;
+	rtf += " }}";
+	delete [] bi;
+	delete [] bb;
+	delete [] hexpict;
+	return rtf;
+}
+
+
+/*
+ uses
+  RichEdit;
+
+// Stream Callback function
+type
+  TEditStreamCallBack = function(dwCookie: Longint; pbBuff: PByte;
+    cb: Longint; var pcb: Longint): DWORD;
+  stdcall;
+
+  TEditStream = record
+    dwCookie: Longint;
+    dwError: Longint;
+    pfnCallback: TEditStreamCallBack;
+  end;
+
+// RichEdit Type
+type
+  TMyRichEdit = TRxRichEdit;
+
+// EditStreamInCallback callback function
+function EditStreamInCallback(dwCookie: Longint; pbBuff: PByte;
+  cb: Longint; var pcb: Longint): DWORD; stdcall;
+  // by P. Below
+var
+  theStream: TStream;
+  dataAvail: LongInt;
+begin
+  theStream := TStream(dwCookie);
+  with theStream do
+  begin
+    dataAvail := Size - Position;
+    Result := 0;
+    if dataAvail <= cb then
+    begin
+	 pcb := read(pbBuff^, dataAvail);
+	 if pcb <> dataAvail then
+	   Result := UINT(E_FAIL);
+    end
+    else
+    begin
+	 pcb := read(pbBuff^, cb);
+	 if pcb <> cb then
+	   Result := UINT(E_FAIL);
+    end;
+  end;
+end;
+
+// Insert Stream into RichEdit
+procedure PutRTFSelection(RichEdit: TMyRichEdit; SourceStream: TStream);
+  // by P. Below
+var
+  EditStream: TEditStream;
+begin
+  with EditStream do
+  begin
+    dwCookie := Longint(SourceStream);
+    dwError := 0;
+    pfnCallback := EditStreamInCallBack;
+  end;
+  RichEdit.Perform(EM_STREAMIN, SF_RTF or SFF_SELECTION, Longint(@EditStream));
+end;
+
+// Convert Bitmap to RTF Code
+function BitmapToRTF(pict: TBitmap): string;
+// by D3k
+var
+  bi, bb, rtf: string;
+  bis, bbs: Cardinal;
+  achar: ShortString;
+  hexpict: string;
+  I: Integer;
+begin
+  GetDIBSizes(pict.Handle, bis, bbs);
+  SetLength(bi, bis);
+  SetLength(bb, bbs);
+  GetDIB(pict.Handle, pict.Palette, PChar(bi)^, PChar(bb)^);
+  rtf := '{\rtf1 {\pict\dibitmap ';
+  SetLength(hexpict, (Length(bb) + Length(bi)) * 2);
+  I := 2;
+  for bis := 1 to Length(bi) do
+  begin
+    achar := Format('%x', [Integer(bi[bis])]);
+    if Length(achar) = 1 then
+	 achar := '0' + achar;
+    hexpict[I - 1] := achar[1];
+    hexpict[I] := achar[2];
+    Inc(I, 2);
+  end;
+  for bbs := 1 to Length(bb) do
+  begin
+    achar := Format('%x', [Integer(bb[bbs])]);
+    if Length(achar) = 1 then
+	 achar := '0' + achar;
+    hexpict[I - 1] := achar[1];
+    hexpict[I] := achar[2];
+    Inc(I, 2);
+  end;
+  rtf := rtf + hexpict + ' }}';
+  Result := rtf;
+end;
+
+
+// Example to insert image from Image1 into RxRichEdit1
+procedure TForm1.Button1Click(Sender: TObject);
+var
+  SS: TStringStream;
+  BMP: TBitmap;
+begin
+  BMP := TBitmap.Create;
+  BMP := Image1.Picture.Bitmap;
+  SS  := TStringStream.Create(BitmapToRTF(BMP));
+  try
+    PutRTFSelection(RxRichEdit1, SS);
+  finally
+    SS.Free;
+  end;
+end;
+*/
