@@ -47,10 +47,10 @@ BibleCSMGR::~BibleCSMGR()
 
 void BibleCSMGR::AddRenderFilters(SWModule *module, ConfigEntMap &section)
 {
-	string lang;
+	SWBuf lang;
 	ConfigEntMap::iterator entry;
 
-	lang = ((entry = section.find("Lang")) != section.end()) ? (*entry).second : (string)"en";
+	lang = ((entry = section.find("Lang")) != section.end()) ? (*entry).second : (SWBuf)"en";
 
 	   if (module->Direction() == DIRECTION_RTL) {
 #ifdef _ICU_
@@ -73,7 +73,6 @@ char BibleCSMGR::AddModToConfig(int conffd, const char *fname)
 	SectionMap::iterator section;
 	AnsiString abouttext = "";
 	int logflag;
-	TMemoryStream *RTFStream = new TMemoryStream();
 	char retval;
 
 	section = modconf.Sections.begin();
@@ -82,12 +81,8 @@ char BibleCSMGR::AddModToConfig(int conffd, const char *fname)
 	ModInstFrm->ModText->Caption = ModInstFrm->ModText->Caption + "Found new module [ " + (*(*section).second.find("Description")).second.c_str() + " ]. Installing...";
 
 	abouttext = abouttext + "{\\fs20\\cf0 " + (*(*section).second.find("About")).second.c_str() + " }";
-	RTFStream->Clear();
-	RTFStream->WriteBuffer(abouttext.c_str(), abouttext.Length());
-	RTFStream->Position = 0;
-	ModInstFrm->AboutText->Lines->LoadFromStream(RTFStream);
-	delete RTFStream;
-
+	ModInstFrm->AboutText->fillWithRTFString(0, abouttext.c_str());
+	
 	ModInstFrm->ShowModal();
 
 	logflag = SWLog::systemlog->getLogLevel();
