@@ -24,13 +24,13 @@ __fastcall TbookmarkForm::TbookmarkForm(TComponent* Owner)
 	
 	DIR *dir;
 	struct dirent *ent;
-	string conffile;
+	SWBuf conffile;
 	bmdir = "";
 	
 	bmtree->Items->Clear();
 
 	if ((sit = Form1->optionsconf->Sections.find("Bookmarks")) != Form1->optionsconf->Sections.end())
-		bmdir = ((eit = (*sit).second.find("Directory")) != (*sit).second.end()) ? (*eit).second : (string)"";
+		bmdir = ((eit = (*sit).second.find("Directory")) != (*sit).second.end()) ? (*eit).second : (SWBuf)"";
 		
 //	Add Personal Bookmarks first, or if they don't exist, ADD A BLANK BRANCH first in the tree
 //	--------------------------------------------------------------------------
@@ -166,7 +166,7 @@ void TbookmarkForm::SaveBookmarks()
 	char buf[15];
 	bool personal = true, other = false;
 	list <String *>::iterator it;
-	string persfile;
+	SWBuf persfile;
 
 	if (bmtree->Items->Count)
 		tree = bmtree->Items->Item[0];
