@@ -39,7 +39,7 @@ private:	// User declarations
 	
 
 public:		// User declarations
-	string bmdir;
+	SWBuf bmdir;
 	__fastcall TbookmarkForm(TComponent* Owner);
 	__fastcall ~TbookmarkForm();
 	void SaveBookmarks();
