@@ -28,6 +28,7 @@
 #include "sword.h"
 #include "rtfhintfrm.h"
 #include "newbmfilefrm.h"
+#include "paraldisp.h"
 #include "RangeMaintFrm.h"
 #include "PrintFrm.h"
 #include "PreviewForm.h"
@@ -35,6 +36,10 @@
 #pragma link "RxRichEd"
 //#pragma link "MOZILLACONTROLLib_OCX"
 //#pragma link "SHDocVw_OCX"
+#pragma link "TntButtons"
+#pragma link "TntComCtrls"
+#pragma link "TntExtCtrls"
+#pragma link "TntMenus"
 #pragma resource "*.dfm"
 
 static const char *rtfclasses[] = {"RTFDisp", "SWDisplay", "SWObject", 0};
@@ -112,7 +117,8 @@ void TForm1::NavigateVerseURL (char* verStrIN)
 			 verStr2 = 0;
 	   }
 
-	   freeHandLookup->Text = verse;
+	   //freeHandLookup->Text = UTF8ToWideString(verse);
+           freeHandLookup->Text = verse;
 	   freeHandLookupoldKeyPress(0, '\r');
 
 	   delete [] verStr;
@@ -141,13 +147,13 @@ __fastcall TForm1::TForm1(TComponent* Owner)
 	lbDictKeys = new TTntListBox(this);
 	lbDictKeys->Parent = Panel2;
 	lbDictKeys->Align = alClient;
-	lbDictKeys->Font->Name = "Code2000";
+//	lbDictKeys->Font->Name = "Code2000";
 	lbDictKeys->OnClick = lbDictKeysOldClick;
 	
 	DictKeyEdit = new TTntEdit(this);
 	DictKeyEdit->Parent = Panel2;
 	DictKeyEdit->Align = alTop;
-	DictKeyEdit->Font->Name = "Code2000";
+//	DictKeyEdit->Font->Name = "Code2000";
 	DictKeyEdit->OnChange = DictKeyEditOldChange;
 	renderingHint = false;	
 	closing = false;
@@ -210,6 +216,7 @@ char TForm1::CreateTextPane(SWModule *mod) {
 	newrtf->ReadOnly = true;
 	newrtf->PopupMenu = (strcmp(mod->Name(), "N27U4")) ? PopupMenu2 : PopupMenu1;
 	newrtf->OnMouseDown = RTFMouseDown;
+	newrtf->OnURLClick = RTFURLClick;
 	newrtf->Name = String("TextRTF")+mod->Name();
 
 	newrtf->module = mod;
@@ -222,6 +229,38 @@ char TForm1::CreateTextPane(SWModule *mod) {
 
 	return 0;
 }
+
+
+char TForm1::CreateParallelPane(SWMgr **mgr) {
+	TTabSheet *newtab = new TTabSheet(this);
+	parallelDisp = new ParallelDisp(this);
+	char buf[1024];
+	//SectionMap::iterator sit;
+
+	newtab->Caption = "PARALLEL";
+	newtab->Hint = "View Parallel Bibles";
+	newtab->ParentShowHint = true;
+	newtab->PageControl = TextPageControl;
+	parallelDisp->Parent = newtab;
+	parallelDisp->Align = alClient;
+	parallelDisp->ScrollBars = ssVertical;
+	parallelDisp->ReadOnly = true;
+	parallelDisp->PopupMenu = PopupMenu5;
+	parallelDisp->OnMouseDown = RTFMouseDown;
+	parallelDisp->OnURLClick = RTFURLClick;
+	parallelDisp->Name = "ParallelRTF";
+
+	parallelDisp->mgr = mgr;
+	parallelDisp->recalcAppearance();
+
+	parallelDisp->mod[0] = layoutconf->Sections["History"]["LastParaMod01"];
+	parallelDisp->mod[1] = layoutconf->Sections["History"]["LastParaMod02"];
+	parallelDisp->mod[2] = layoutconf->Sections["History"]["LastParaMod03"];
+
+	return 0;
+}
+
+
 //---------------------------------------------------------------------------
 char TForm1::CreateCommentPane(SWModule *mod) {
 	TTabSheet *newtab = new TTabSheet(this);
@@ -269,7 +308,8 @@ char TForm1::CreateCommentPane(SWModule *mod) {
 		((SWDispRTF *)newrtf)->ReadOnly = true;
 		((SWDispRTF *)newrtf)->Name = String("CommentRTF")+mod->Name();
 
-		if ((*mainmgr->config->Sections[mod->Name()].find("ModDrv")).second == "RawFiles") {
+		if (!strcmp(mod->getConfigEntry("ModDrv"), "RawFiles") ||
+          		((mod->getConfigEntry("Writable")) && (!strcmp(mod->getConfigEntry("Writable"), "true")))) {
 			((SWDispRTF *)newrtf)->PopupMenu = PopupMenu3;
 			((SWDispRTF *)newrtf)->ExpandNewLine = false;
 		}
@@ -387,7 +427,13 @@ void TForm1::RefreshActiveSheet(TPageControl *pc)
 
 	if (mainmgr) {
 		it = mainmgr->Modules.find(pc->ActivePage->Caption.c_str());
-		if (it != mainmgr->Modules.end()) {
+		if (pc->ActivePage->Caption == "PARALLEL") {
+			parallelDisp->Display();
+			pc->Hint = "PARALLEL";
+			if (logmodstate)
+				modstates.insert(modstates.begin(), new ModState(pc, pc->ActivePage, *DefaultVSKey));
+		}
+		else if (it != mainmgr->Modules.end()) {
 			SWModule *module = it->second;
 			module->Display();
 			pc->Hint = module->Description();
@@ -405,7 +451,8 @@ void TForm1::TextKeyChanged()
 	cbBook->ItemIndex = cbBook->Items->IndexOf(DefaultVSKey->books[DefaultVSKey->Testament()-1][DefaultVSKey->Book()-1].name);
 	CHBox->Text  = DefaultVSKey->Chapter();
 	VSBox->Text  = DefaultVSKey->Verse();
-	freeHandLookup->Text = (const char *)*DefaultVSKey;
+	//freeHandLookup->Text = UTF8ToWideString((const char *)*DefaultVSKey);
+	   freeHandLookup->Text = (const char *)*DefaultVSKey;
 	RefreshActiveSheet(TextPageControl);
 	logmodstate = false;		// only log state once
 	RefreshActiveSheet(CommentaryPageControl);
@@ -423,7 +470,8 @@ void __fastcall TForm1::btnLookupClick(TObject *Sender)
 	if (StrToInt(VSBox->Text) < 0)
 		VSBox->Text = StrToInt(VSBox->Text) + 1;
 
-	(*DefaultVSKey)       = (cbBook->Text + " " + CHBox->Text + ":" + VSBox->Text).c_str();
+	//(*DefaultVSKey)       = WideStringToUTF8((cbBook->Text + " " + CHBox->Text + ":" + VSBox->Text)).c_str();
+        (*DefaultVSKey)       = (cbBook->Text + " " + CHBox->Text + ":" + VSBox->Text).c_str();
 	TextKeyChanged();
 	DefaultVSKey->AutoNormalize(1);	
 }
@@ -450,7 +498,7 @@ void TForm1::FillDictKeys() {
 	int index = 0;
 	SWKey saveKey;
 	ConfigEntMap::const_iterator const_eit;
-	string fontname;
+	SWBuf fontname;
 
 	if (mainmgr) {
 		it = mainmgr->Modules.find(LexDictPageControl->ActivePage->Caption.c_str());
@@ -480,6 +528,7 @@ void TForm1::FillDictKeys() {
 			module->Error();
 			for (;!module->Error() && count;count--,(*module)++) {
 				lbDictKeys->Items->Add(UTF8ToWideString(module->KeyText()));
+				//lbDictKeys->Items->Add(module->KeyText());
 				if (saveKey == module->Key())
 					index = lbDictKeys->Items->Count-1;
 			}
@@ -516,23 +565,28 @@ void __fastcall TForm1::About1Click(TObject *Sender)
 
 
 WideString TForm1::TrimJunk(WideString src) {
-	bool hasDigit = false;
-     bool hasAlpha = false;
+	bool hasDigit  = false;
+	bool hasAlpha  = false;
+	bool hasNonCap = false;
 	WideString retVal = "";
 	
 	int len = src.Length();
 	for (int i = 1; i <=len; i++) {
 		if (isdigit(src[i]))
 			hasDigit = true;
-		if (isalpha(src[i]) || (src[i] > 255))
+		if (isalpha(src[i]) || (src[i] > 255)) {
 			hasAlpha = true;
+			if (toupper(src[i]) != src[i])
+				hasNonCap = true;
+		}
 	}
+	
 	for (int i = 1; i <= len; i++) {
 		if (hasDigit && !hasAlpha) {	// number
 			if (isdigit(src[i])) retVal += (WideString)src[i];
 		}
 		else if (hasAlpha) {	// alpha
-			if (isalpha(src[i]) || (src[i] > 255)) retVal += (WideString)src[i];
+			if (isalpha(src[i]) || (src[i] > 255) || ((!hasNonCap) && ((src[i] == '-') || isdigit(src[i])))) retVal += (WideString)src[i];
 		}
 		else	retVal += (WideString)src[i];
 	}
@@ -556,13 +610,15 @@ void __fastcall TForm1::DictionaryLookup1Click(TObject *Sender)
 	}
 	else	{
 		preChar = rtf->Text[(rtf->SelStart)?rtf->SelStart:0];
-		lookupKey = Trim(rtf->SelText);
+		lookupKey = TRxRichEditX::Trim(rtf->GetTextRange(rtf->SelStart, rtf->SelStart + rtf->SelLength));
 	}
 		
 	// check if all digits (strongs)
 	int i;
 	for (i = 0; i < lookupKey.Length(); i++) {
-		if (!isdigit(lookupKey[i+1]) && !strchr("<>()", lookupKey[i+1]))
+		if (!isdigit(lookupKey[i+1])
+				&& (toupper(lookupKey[i+1]) != lookupKey[i+1])
+				&& !strchr("<>()-", lookupKey[i+1]))
 			break;
 	}
 
@@ -574,7 +630,7 @@ void __fastcall TForm1::DictionaryLookup1Click(TObject *Sender)
 		if (!strncmp(rtf->Name.c_str(), "TextRTFLXX", 10))
 			feature = "GreekDef";
 		if (!hasFeature(mainmgr, curLex.c_str(), feature.c_str())) {
-			string tmpval = optionsconf->Sections["ModDefaults"][feature.c_str()];
+			SWBuf tmpval = optionsconf->Sections["ModDefaults"][feature.c_str()];
 			for (int i = 0; i < LexDictPageControl->PageCount; i++) {
 				if (!stricmp(LexDictPageControl->Pages[i]->Caption.c_str(), tmpval.c_str())) {
 					LexDictPageControl->ActivePageIndex = i;
@@ -636,9 +692,9 @@ void __fastcall TForm1::SaveLayout1Click(TObject *Sender)
 		sit["SearchHeight"] = IntToStr(searchForm->Height).c_str();
 	}
 
-	OptionsList options = mainmgr->getGlobalOptions();
-	for (OptionsList::iterator it = options.begin(); it != options.end(); it++) {
-		string value = mainmgr->getGlobalOption(it->c_str());
+	StringList options = mainmgr->getGlobalOptions();
+	for (StringList::iterator it = options.begin(); it != options.end(); it++) {
+		SWBuf value = mainmgr->getGlobalOption(it->c_str());
 		(*layoutconf)["ModuleOptions"][it->c_str()] = value;
 	}
 	layoutconf->Sections["Screen"] = sit;
@@ -654,7 +710,8 @@ void TForm1::fillVKeySelector(VerseKey *vk) {
 	while (!myVk->Error()) {
 		AnsiString s1 = (const char *)(*myVk);
 		s1.SetLength(s1.Length() - 4);
-		cbBook->Items->Add(s1);
+		//cbBook->Items->Add(UTF8ToWideString(s1));
+                cbBook->Items->Add(s1);
 		myVk->Book(myVk->Book() + 1);
 	}
 	delete myVk;
@@ -664,14 +721,14 @@ void TForm1::fillVKeySelector(VerseKey *vk) {
 void TForm1::i12ize(const char *lang) {
 	LocaleMgr::systemLocaleMgr.setDefaultLocaleName(lang);
 	// TODO 5 -cConfiguration Ops -oScribe: add mainmgr->configpath, or something, to beginning of these image names ???
-	const char *tmp;
+	AnsiString tmp;
 	Graphics::TBitmap *bitmap = 0;
 	TPicture *pic = new TPicture();
 
 	ImageList1->Clear();		
 	
-	tmp = _tr("BackBtnImage");
-	if (strcmp("BackBtnImage", tmp)) {
+	tmp = WideStringToUTF8(_tr("BackBtnImage")).c_str();
+	if (strcmp("BackBtnImage", tmp.c_str())) {
 		pic->LoadFromFile(tmp);
 		bitmap = pic->Bitmap;
 	}
@@ -680,8 +737,8 @@ void TForm1::i12ize(const char *lang) {
 	TColor transColor = bitmap->Canvas->Pixels[0][0];
 	ImageList1->AddMasked(bitmap, transColor);
 
-	tmp = _tr("SearchBtnImage");
-	if (strcmp("SearchBtnImage", tmp)) {
+	tmp = WideStringToUTF8(_tr("SearchBtnImage")).c_str();
+	if (strcmp("SearchBtnImage", tmp.c_str())) {
 		pic->LoadFromFile(tmp);
 		bitmap = pic->Bitmap;
 	}
@@ -695,7 +752,16 @@ void TForm1::i12ize(const char *lang) {
 	ImageList2->AddMasked(bitmap, transColor);
 
 	delete pic;
-	
+
+        TFont *UIFont = new TFont();
+	   UIFont->Name = optionsconf->Sections["Appearance"]["UIFontName"].c_str();
+	   
+	   tmp = optionsconf->Sections["Appearance"]["UIFontColor"].c_str();
+	   if (tmp.Length() > 0)
+		   UIFont->Color = StrToInt(tmp);
+	   tmp = optionsconf->Sections["Appearance"]["UIFontSize"].c_str();
+	   if (tmp.Length() > 0)
+		   UIFont->Size = StrToInt(tmp);
 	// MainForm
 	File1->Caption = _tr("&File");
 	SaveLayout1->Caption = _tr("S&ave Layout");
@@ -736,6 +802,14 @@ void TForm1::i12ize(const char *lang) {
 	TextPageControl->Hint = _tr("Biblical Texts");
 	CommentaryPageControl->Hint = _tr("Commentaries");
 	LexDictPageControl->Hint = _tr("Lexicons / Dictionaries");
+
+        cbBook->Font = UIFont;
+        freeHandLookup->Font = UIFont;
+        DictKeyEditOld->Font = UIFont;
+        lbDictKeysOld->Font = UIFont;
+        lbDictKeys->Font = UIFont;
+        DictKeyEdit->Font = UIFont;
+
 	// AboutBox
 	AboutBox->Caption = _tr("About The SWORD Project");
 	AboutBox->OKButton->Caption = _tr("OK");
@@ -904,6 +978,8 @@ void TForm1::i12ize(const char *lang) {
 	// Print Preview Form
 	PreviewForm->btnPrint->Caption = "Print";
 	PreviewForm->btnClose->Caption = "&Close";
+
+        delete UIFont;
 	
 }
 
@@ -924,7 +1000,7 @@ void __fastcall TForm1::FormShow(TObject *Sender)
 	int loop;
 	int maxFiles;
 	TColor fieldColor;
-	string tmpval;
+	SWBuf tmpval;
 	bool showDevos;
 	bool showGlos;
 
@@ -935,6 +1011,7 @@ void __fastcall TForm1::FormShow(TObject *Sender)
 			showLocked = false;
 	}
 */
+	layoutconf = new SWConfig("./layout.conf");
 
 	logmodstate = true;
 	freshdict   = true;
@@ -975,6 +1052,7 @@ void __fastcall TForm1::FormShow(TObject *Sender)
 		Application->Terminate();
 	// Load devotion and show form
 
+	CreateParallelPane((SWMgr **)&mainmgr);
 	for (it = mainmgr->Modules.begin(); it != mainmgr->Modules.end(); it++) {
 		bool devotional = false;
 		bool glossary = false;
@@ -1067,8 +1145,6 @@ void __fastcall TForm1::FormShow(TObject *Sender)
 
 	}
 
-	layoutconf = new SWConfig("./layout.conf");
-
 	if ((sit = layoutconf->Sections.find("Screen")) != layoutconf->Sections.end()) {
 
 		if ((eit = (*sit).second.find("MainTop")) != (*sit).second.end()) {
@@ -1113,8 +1189,8 @@ void __fastcall TForm1::FormShow(TObject *Sender)
 	
 	// Add options to Options Main Menu choice
 	sit = layoutconf->Sections.find("ModuleOptions");
-	OptionsList options = mainmgr->getGlobalOptions();
-	for (OptionsList::iterator it = options.begin(); it != options.end(); it++) {
+	StringList options = mainmgr->getGlobalOptions();
+	for (StringList::iterator it = options.begin(); it != options.end(); it++) {
 		TMenuItem *newitem = new TMenuItem(MainMenu1);
 		newitem->Caption = it->c_str();
 		newitem->Default = false;
@@ -1124,8 +1200,8 @@ void __fastcall TForm1::FormShow(TObject *Sender)
 			if ((eit = sit->second.find(it->c_str())) != sit->second.end())
 				mainmgr->setGlobalOption(it->c_str(), eit->second.c_str());
 		}
-		OptionsList values = mainmgr->getGlobalOptionValues(it->c_str());    
-		for (OptionsList::iterator it2 = values.begin(); it2 != values.end(); it2++) {
+		StringList values = mainmgr->getGlobalOptionValues(it->c_str());    
+		for (StringList::iterator it2 = values.begin(); it2 != values.end(); it2++) {
 			TMenuItem *newitem2 = new TMenuItem(MainMenu1);
 			newitem2->Caption = it2->c_str();
 			newitem2->Default = false;
@@ -1140,7 +1216,8 @@ void __fastcall TForm1::FormShow(TObject *Sender)
 
 		eitend = (*sit).second.upper_bound("SearchText");
 		for (eit = (*sit).second.lower_bound("SearchText"); eit != eitend; eit++)
-			searchForm->SearchText->Items->Add(UTF8ToWideString(eit->second.c_str()));
+			//searchForm->SearchText->Items->Add(UTF8ToWideString(eit->second.c_str()));
+                        searchForm->SearchText->Items->Add(eit->second.c_str());
 
 		eitend = (*sit).second.upper_bound("LookupText");
 		for (eit = (*sit).second.lower_bound("LookupText"); eit != eitend; eit++)
@@ -1172,7 +1249,8 @@ void __fastcall TForm1::FormShow(TObject *Sender)
 			}
 		}
 		if ((eit = (*sit).second.find("LastLDKey")) != (*sit).second.end()) {
-			DictKeyEdit->Text = UTF8ToWideString((*eit).second.c_str());
+			//DictKeyEdit->Text = UTF8ToWideString((*eit).second.c_str());
+                        DictKeyEdit->Text = (*eit).second.c_str();
 		}
 
 	}
@@ -1197,7 +1275,7 @@ void __fastcall TForm1::FormShow(TObject *Sender)
 		}
 		
 		for (int i = 0; i < pc->PageCount; i++) {
-			string heading = pc->Pages[i]->Caption.c_str();
+			SWBuf heading = pc->Pages[i]->Caption.c_str();
 			pc->Pages[i]->TabVisible = !((*section)[heading.c_str()] == "false");  // !false allows default to be true
 		}
 	}
@@ -1292,7 +1370,8 @@ void __fastcall TForm1::UpDown3Click(TObject *Sender, TUDBtnType Button)
 		if (Button == btNext)
 			(*((*it).second))++;
 		else	(*((*it).second))--;
-		DictKeyEdit->Text = UTF8ToWideString(((*it).second)->KeyText());
+		//DictKeyEdit->Text = UTF8ToWideString(((*it).second)->KeyText());
+                DictKeyEdit->Text = ((*it).second)->KeyText();
 	}
 }
 //---------------------------------------------------------------------------
@@ -1303,7 +1382,10 @@ void __fastcall TForm1::UpDown1Click(TObject *Sender, TUDBtnType Button)
 	btnLookupClick(Sender);
 }
 //---------------------------------------------------------------------------
-
+/* TODO -cParallel Disp : Troy could you make sure that I did this correctly?
+					I am not sure what they key was for in the original
+					if statement so I left it out of the PARALLEL statement
+					and it works, I think ;-) */
 void TForm1::RestoreState(ModState *state)
 {
     ModMap::iterator it;
@@ -1320,10 +1402,15 @@ void TForm1::RestoreState(ModState *state)
 					*DefaultVSKey = state->key;
 					TextKeyChanged();
 				}
+				else if (state->pc->ActivePage->Caption == "PARALLEL") {
+					*DefaultVSKey = state->key;
+					TextKeyChanged();
+				}
 			}
 		}
 		if (state->pc == LexDictPageControl) {
-			DictKeyEdit->Text = UTF8ToWideString((const char *)state->key);
+			//DictKeyEdit->Text = UTF8ToWideString((const char *)state->key);
+				    DictKeyEdit->Text = ((const char *)state->key);
 		}
 		if (state->pc->ActivePage->ControlCount){
 			Form1->SetFocusedControl( (TWinControl *)state->pc->ActivePage->Controls[0] );
@@ -1470,12 +1557,18 @@ void __fastcall TForm1::FormKeyDown(TObject *Sender, WORD &Key,
 				if ((Screen->ActiveControl == LexDictPageControl) || (IsChild(LexDictPageControl->Handle, Screen->ActiveControl->Handle)))
 					pc = LexDictPageControl;
 
-				it = mainmgr->Modules.find(pc->ActivePage->Caption.c_str());
-				if (it != mainmgr->Modules.end()) {
+				if (pc->ActivePage->Caption == "PARALLEL") {
 					if (ctrlstates[val])
 						delete ctrlstates[val];
-					
-					ctrlstates[val] = new ModState(pc, pc->ActivePage, (*it).second->KeyText());
+					ctrlstates[val] = new ModState(pc, pc->ActivePage, *DefaultVSKey);
+				}
+				else {					
+					it = mainmgr->Modules.find(pc->ActivePage->Caption.c_str());
+					if (it != mainmgr->Modules.end()) {
+						if (ctrlstates[val])
+							delete ctrlstates[val];
+						ctrlstates[val] = new ModState(pc, pc->ActivePage, (*it).second->KeyText());
+					}
 				}
 			}
 			else	RestoreState(ctrlstates[val]);
@@ -1490,7 +1583,7 @@ void __fastcall TForm1::Options1Click(TObject *Sender)
 	ConfigEntMap::iterator eit;
 	ConfigEntMap emap;
 	int color;
-	string tmpval;
+	SWBuf tmpval;
 
 	   //*********************************
 	   //Initialize Preferences Dialog Box
@@ -1584,16 +1677,44 @@ void __fastcall TForm1::PopupMenuPopup(TObject *Sender) {
 		staticMenuItemsCount = 3;
 	else if (menu == PopupMenu3)
 		staticMenuItemsCount = 7;
+	else if (menu == PopupMenu5)
+		staticMenuItemsCount = 7;
 		
 	while (menu->Items->Count > staticMenuItemsCount)
 		menu->Items->Delete(staticMenuItemsCount);
 
 	SWDispRTF *rtf = (SWDispRTF *)Screen->ActiveControl;
 	if (clickText.Length()) {
-		string highlight = Trim(clickText).c_str();
-		tmpVerseList = DefaultVSKey->ParseVerseList((char *)highlight.c_str(), *DefaultVSKey);
-		if (tmpVerseList.Count())
-			AddVerseChoices(menu, highlight.c_str(), rtf);
+		SWBuf highlight = Trim(clickText).c_str();
+
+		if (clickText.c_str()[0] == '*') {	// '*' = NOTE
+			ModMap::iterator it;
+			it = mainmgr->Modules.find(TextPageControl->ActivePage->Caption.c_str());
+			if (it != mainmgr->Modules.end()) {
+				SWModule *module = it->second;
+				SWBuf verseNum = clickText.c_str()+2;
+				int offset = (strchr(verseNum.c_str(), '.') - verseNum.c_str());
+				verseNum.setSize(offset);
+				SWBuf fnNum = clickText.c_str()+3+offset;
+				int oldVerse = DefaultVSKey->Verse();
+				DefaultVSKey->Verse(atoi(verseNum));
+				module->RenderText();	// force entry attributes to get set
+				SWBuf type = module->getEntryAttributes()["Footnote"][fnNum.c_str()]["type"].c_str();
+				SWBuf body = module->getEntryAttributes()["Footnote"][fnNum.c_str()]["body"].c_str();
+				DefaultVSKey->Verse(oldVerse);
+				if (type == "crossReference") {
+					SWBuf refList = module->getEntryAttributes()["Footnote"][fnNum.c_str()]["refList"].c_str();
+					tmpVerseList = DefaultVSKey->ParseVerseList(refList.c_str(), *DefaultVSKey);
+					if (tmpVerseList.Count())
+						AddVerseChoices(menu, refList.c_str(), rtf);
+				}
+			}                                 
+		}
+		else {
+			tmpVerseList = DefaultVSKey->ParseVerseList(highlight.c_str(), *DefaultVSKey);
+			if (tmpVerseList.Count())
+				AddVerseChoices(menu, highlight.c_str(), rtf);
+		}
 	}
 	else	if (!rtf->SelLength) {
 		int entrylen = strlen(rtf->Text.c_str());
@@ -1655,7 +1776,7 @@ void __fastcall TForm1::PopupMenuPopup(TObject *Sender) {
 		delete [] entrytext;
 	}
 	else {
-		string highlight = Trim(rtf->SelText).c_str();
+		SWBuf highlight = Trim(rtf->SelText).c_str();
 		tmpVerseList = DefaultVSKey->ParseVerseList((char *)highlight.c_str(), *DefaultVSKey);
 		if (tmpVerseList.Count())
 			AddVerseChoices(menu, highlight.c_str(), rtf);
@@ -1683,7 +1804,7 @@ void __fastcall TForm1::createVerseList(TObject *Sender)
 void __fastcall TForm1::freeHandLookupoldKeyPress(TObject *Sender, char &Key)
 {
 	if (Key == '\r') {
-		ListKey tmpVerseList = DefaultVSKey->ParseVerseList(freeHandLookup->Text.c_str(), *DefaultVSKey);
+		ListKey tmpVerseList = DefaultVSKey->ParseVerseList(WideStringToUTF8(freeHandLookup->Text).c_str(), *DefaultVSKey);
 
 		if (tmpVerseList.Count()) {
 			int index = freeHandLookup->Items->IndexOf(freeHandLookup->Text);
@@ -1750,7 +1871,7 @@ void __fastcall TForm1::EditEntry1Click(TObject *Sender)
 	EditEntryForm->ShowModal();
 	if (EditEntryForm->ResultBuf) {
 		if (module)
-//			DictKeyEdit->Text = EditEntry->ResultBuf;  // for testing purposes
+//			DictKeyEdit->Text = UTF8ToWideString(EditEntry->ResultBuf);  // for testing purposes
 			*module << EditEntryForm->ResultBuf;
 	}
 	RefreshActiveSheet(CommentaryPageControl);
@@ -1759,9 +1880,9 @@ void __fastcall TForm1::EditEntry1Click(TObject *Sender)
 
 void __fastcall TForm1::DeleteEntry1Click(TObject *Sender)
 {
-	string message = _tr("Are you sure you wish to delete this entry?");
-	string header = _tr("Delete Entry");
-	if (MessageBox(Handle, message.c_str(), header.c_str(), MB_YESNO) == IDYES) {
+	WideString message = _tr("Are you sure you wish to delete this entry?");
+	WideString header = _tr("Delete Entry");
+	if (MessageBoxW(Handle, message, header, MB_YESNO) == IDYES) {
 		SWDispRTF *rtf = (SWDispRTF *)Screen->ActiveControl;
 		SWModule *module;
 		ModMap::iterator target;
@@ -1821,9 +1942,9 @@ void __fastcall TForm1::GlobalOptionChange(TObject *Sender)
 	mainmgr->setGlobalOption(option, value);
 	
 	if ((!stricmp(option, "Strong's Numbers")) && (!stricmp(value, "On"))) {
-		string curText = TextPageControl->ActivePage->Caption.c_str();
+		SWBuf curText = TextPageControl->ActivePage->Caption.c_str();
 		if (!hasFeature(mainmgr, curText.c_str(), "StrongsNumbers")) {
-			string tmpval = optionsconf->Sections["ModDefaults"]["StrongsNumbers"];
+			SWBuf tmpval = optionsconf->Sections["ModDefaults"]["StrongsNumbers"];
 			for (int i = 0; i < TextPageControl->PageCount; i++) {
 				if (!stricmp(TextPageControl->Pages[i]->Caption.c_str(), tmpval.c_str())) {
 					TextPageControl->ActivePageIndex = i;
@@ -1846,7 +1967,7 @@ void __fastcall TForm1::OptionShowVals(TObject *Sender)
 {
 	TMenuItem *menuChoice = (TMenuItem *)Sender;
 
-	string option = menuChoice->Caption.c_str();
+	SWBuf option = menuChoice->Caption.c_str();
 	int k = 0;
 	const char *source = option.c_str();
 	char *caption = new char [strlen(source) + 1];
@@ -1856,7 +1977,7 @@ void __fastcall TForm1::OptionShowVals(TObject *Sender)
 	}
 	caption[k] = 0;
 		
-	string value = mainmgr->getGlobalOption(caption);
+	SWBuf value = mainmgr->getGlobalOption(caption);
 	delete [] caption;
 	for (int i = 0; i < menuChoice->Count; i++) {
 		int k = 0;
@@ -1877,8 +1998,8 @@ void __fastcall TForm1::OptionShowVals(TObject *Sender)
 
 void __fastcall TForm1::Contents1Click(TObject *Sender)
 {
-	string helpDir = optionsconf->Sections["Help"].getWithDefault("Directory", ".\\help");
-	string helpExe = helpDir + "\\sword.chm";
+	SWBuf helpDir = optionsconf->Sections["Help"].getWithDefault("Directory", ".\\help");
+	SWBuf helpExe = helpDir + "\\sword.chm";
 	
 	if ((int)ShellExecute(this->Handle, "open", helpExe.c_str(), NULL, NULL, SW_SHOWNORMAL) < 33) {
 		helpExe = helpDir + "\\index.html";
@@ -1916,13 +2037,16 @@ void __fastcall TForm1::FormClose(TObject *Sender, TCloseAction &Action)
 
 	section.erase("LookupText");
 	for (int loop = 0; ((loop < saveCount) && (loop <	freeHandLookup->Items->Count)); loop++)
-		 section.insert(ConfigEntMap::value_type("LookupText", freeHandLookup->Items->Strings[loop].c_str()));
+		 section.insert(ConfigEntMap::value_type("LookupText", WideStringToUTF8(freeHandLookup->Items->Strings[loop]).c_str()));
 
 	section["LastVerse"] = (const char *)*DefaultVSKey;
 	section["LastLDKey"] = WideStringToUTF8(DictKeyEdit->Text).c_str();
 	section["LastTextModule"] = TextPageControl->ActivePage->Caption.c_str();
 	section["LastComModule"] = CommentaryPageControl->ActivePage->Caption.c_str();
 	section["LastLDModule"] = LexDictPageControl->ActivePage->Caption.c_str();
+	section["LastParaMod01"] = parallelDisp->mod[0].c_str();
+	section["LastParaMod02"] = parallelDisp->mod[1].c_str();
+	section["LastParaMod03"] = parallelDisp->mod[2].c_str();
 
 	layoutconf->Sections["History"] = section;
 	layoutconf->Save();
@@ -1965,14 +2089,14 @@ void __fastcall TForm1::HideShowModules1Click(TObject *Sender)
      if (!pc) return;	// assert pc
      	
 	for (int i = 0; i < pc->PageCount; i++) {
-     	string heading = pc->Pages[i]->Caption.c_str();
+     	SWBuf heading = pc->Pages[i]->Caption.c_str();
      	(*section)[heading.c_str()] = ((pc->Pages[i]->TabVisible) ? "true" : "false");
      }
      ModVisForm->mgr = mainmgr;
 	ModVisForm->section = section;
 	if (ModVisForm->ShowModal() == mrOk) {
           for (int i = 0; i < pc->PageCount; i++) {
-               string heading = pc->Pages[i]->Caption.c_str();
+               SWBuf heading = pc->Pages[i]->Caption.c_str();
 			pc->Pages[i]->TabVisible = ((*section)[heading.c_str()] == "true");
           }
 	}	
@@ -2013,10 +2137,10 @@ void __fastcall TForm1::CppWebBrowser1BeforeNavigate2(TObject *Sender,
 	 TOLEBOOL *Cancel)
 {
 	AnsiString url = *URL;
-	string stuff = url.c_str();
+	SWBuf stuff = url.c_str();
 	char *offset = strstr(url.c_str(), "passage");
 	if (offset > 0) {
-		string verse = (offset+8);
+		SWBuf verse = (offset+8);
 		char * buf = new char [ verse.length() + 1 ];
 		strcpy(buf, verse.c_str());
 		dehtmlize(buf);
@@ -2037,11 +2161,15 @@ void __fastcall TForm1::InstallManager1Click(TObject *Sender)
 
 void TForm1::refreshAppearance() {
 	ModMap::iterator it;
-	string locale;
-	string tmpval;
+	SWBuf locale;
+	SWBuf tmpval;
 
 	locale = optionsconf->Sections["Appearance"].getWithDefault("locale", "en_us");
-	i12ize(locale.c_str());
+	/* TODO 5 -oDavid -cLocale : Just a note: Something weird happens here when there are no .conf files and it crashes. This is since Daniel added the TNT stuff. Maybe should look more into this later */
+	const char *loc = locale.c_str();
+	i12ize(loc);
+
+
 
 	// Change the panels attributes here
 	for (it = mainmgr->Modules.begin(); it != mainmgr->Modules.end(); it++) {
@@ -2052,7 +2180,6 @@ void TForm1::refreshAppearance() {
 			rtfDisp->recalcAppearance();
 		}
 	}
-
 	// Change the text entry field color
 	tmpval = optionsconf->Sections["Appearance"]["FieldColor"];
 	TColor fieldColor = tmpval.length() ? TColor(atoi(tmpval.c_str())) : clAqua;
@@ -2083,7 +2210,7 @@ void __fastcall TForm1::ApplicationEvents1ShowHint(AnsiString &HintStr,
 	 bool &CanShow, THintInfo &HintInfo)
 {
 	ConfigEntMap::const_iterator const_eit;
-	string fontname;
+	SWBuf fontname;
 	char buf[1024];
 	static TRect lastRect(0,0,0,0);
 	bool selected = false;
@@ -2115,7 +2242,16 @@ void __fastcall TForm1::ApplicationEvents1ShowHint(AnsiString &HintStr,
 			if (pc->PageCount) {
 				TTabSheet *ts = pc->Pages[i];
 				
-				SWModule *module = mainmgr->Modules[ts->Caption.c_str()];
+				SWModule *module = 0;
+				if (ts->Caption == "PARALLEL") {
+					module = parallelDisp->getModules()[0];
+				}
+				else {
+					ModMap::iterator it = mainmgr->Modules.find(ts->Caption.c_str());
+					if (it != mainmgr->Modules.end())
+						module = it->second;
+				}
+					
 				if (module) {
 					HintStr = module->Description();
 					HintInfo.CursorRect = pc->TabRect(tab);
@@ -2128,13 +2264,26 @@ void __fastcall TForm1::ApplicationEvents1ShowHint(AnsiString &HintStr,
 
 	if ((strncmp(HintInfo.HintControl->Name.c_str(), "TextRTF", 7)) &&
 			(strncmp(HintInfo.HintControl->Name.c_str(), "CommentRTF", 10)) &&
+			(strncmp(HintInfo.HintControl->Name.c_str(), "ParallelRTF", 11)) &&
 			(strncmp(HintInfo.HintControl->Name.c_str(), "BookRTF", 7)) &&
 			(strncmp(HintInfo.HintControl->Name.c_str(), "LexDictRTF", 10))) {
 		return;
 	}
 	char *modName = 0;
 	stdstr(&modName, HintInfo.HintControl->Name.c_str());
-	SWModule *module = mainmgr->Modules[(strstr(modName, "RTF")+3)];
+	SWModule *module = 0;
+
+
+	if (!strcmp(modName, "ParallelRTF")) {
+		module = parallelDisp->getModules()[0];
+	}
+	else {
+		ModMap::iterator it = mainmgr->Modules.find(strstr(modName, "RTF")+3);
+		if (it != mainmgr->Modules.end())
+			module = it->second;
+	}
+
+
 	delete [] modName;
 		
 //	Application->HintHidePause = HintDisplayTime;
@@ -2196,7 +2345,7 @@ void __fastcall TForm1::ApplicationEvents1ShowHint(AnsiString &HintStr,
 		}
 		else {
 			selected = true;
-			targetWord = rtf->SelText;
+			targetWord = rtf->SelTextWide;
 		}
 			
 //			TPoint currentChar = rtf->GetCharPos(rtf->SelStart);
@@ -2244,18 +2393,64 @@ void __fastcall TForm1::ApplicationEvents1ShowHint(AnsiString &HintStr,
 		currentRTF = rtf;
 		lastWord = targetWord;
 			
-		// currently assume all links are verselists
-		if (verseList.Length() && hintVLists) {
-			ModMap::iterator it;
-			it = mainmgr->Modules.find(TextPageControl->ActivePage->Caption.c_str());
-			if (it != mainmgr->Modules.end()) {
-				SWModule *module = it->second;
-				ListKey verses = DefaultVSKey->ParseVerseList(verseList.c_str(), *DefaultVSKey, true);
-				if (verses.Count()) {
-					RTFHintForm->rtfDrawer->fillWithVerses(module, &verses, true, true, "Popup");
-					HintStr = "show rtf";
+		// currently assume all links are verselists (added note types still using verseList variable name)
+		if (verseList.Length()) {
+			if (verseList.c_str()[0] == '*') {	// '*' = NOTE
+				ModMap::iterator it;
+				it = mainmgr->Modules.find(TextPageControl->ActivePage->Caption.c_str());
+				if (it != mainmgr->Modules.end()) {
+					SWModule *module = it->second;
+					SWBuf verseNum = verseList.c_str()+2;
+					int offset = (strchr(verseNum.c_str(), '.') - verseNum.c_str());
+					verseNum.setSize(offset);
+					SWBuf fnNum = verseList.c_str()+3+offset;
+					int oldVerse = DefaultVSKey->Verse();
+					DefaultVSKey->Verse(atoi(verseNum));
+					module->RenderText();	// force entry attributes to get set
+					SWBuf type = module->getEntryAttributes()["Footnote"][fnNum.c_str()]["type"].c_str();
+					SWBuf body = module->getEntryAttributes()["Footnote"][fnNum.c_str()]["body"].c_str();
+					SWBuf refList = module->getEntryAttributes()["Footnote"][fnNum.c_str()]["refList"].c_str();
+					body = module->RenderText(body.c_str());
+					DefaultVSKey->Verse(oldVerse);
+ 
+					if ((type == "crossReference")
+					// this doesn't work cuz we never cat into the hint code if a meta key is pressed.  curious.
+					// && (!GetAsyncKeyState(VK_LCONTROL))
+					) {
+						ListKey verses = DefaultVSKey->ParseVerseList(refList.c_str(), *DefaultVSKey, true);
+						if (verses.Count()) {
+							RTFHintForm->rtfDrawer->fillWithVerses(module, &verses, body.c_str(), true, true, "Popup");
+							HintStr = "show rtf";
+						}
+						else HintStr = "";
+					}
+					else {
+						RTFHintForm->rtfDrawer->fillWithRTFString(module, body.c_str());
+						HintStr = "show rtf";
+					}
+				}                                 
+				break;
+
+			}
+			else if (hintVLists) {			// otherwise assume verselist
+				SWModule *module = 0;
+				if (TextPageControl->ActivePage->Caption == "PARALLEL") {
+					module = Form1->parallelDisp->getModules()[0];
+				}
+				else {
+					ModMap::iterator it = mainmgr->Modules.find(TextPageControl->ActivePage->Caption.c_str());
+					if (it != mainmgr->Modules.end()) {
+						module = it->second;
+					}
+				}
+				if (module) {
+					ListKey verses = DefaultVSKey->ParseVerseList(verseList.c_str(), *DefaultVSKey, true);
+					if (verses.Count()) {
+						RTFHintForm->rtfDrawer->fillWithVerses(module, &verses, 0, true, true, "Popup");
+						HintStr = "show rtf";
+					}
+					else HintStr = "";
 				}
-				else HintStr = "";
 				break;
 			}
 		}
@@ -2268,7 +2463,9 @@ void __fastcall TForm1::ApplicationEvents1ShowHint(AnsiString &HintStr,
 			// check if all digits (strongs)
 			int i;
 			for (i = 1; i <= strongsWord.Length(); i++) {
-				if (!isdigit(strongsWord[i]))
+				if (!isdigit(strongsWord[i])
+						&& (toupper(strongsWord[i]) != strongsWord[i])
+						&& !strchr("-", strongsWord[i]))
 					break;
 			}
 
@@ -2277,17 +2474,20 @@ void __fastcall TForm1::ApplicationEvents1ShowHint(AnsiString &HintStr,
 				feature += (preChar == '(')?"Parse":"Def";
 				if (!strncmp(HintInfo.HintControl->Name.c_str(), "TextRTFLXX", 10))
 					feature = "GreekDef";
-				string tmpval = optionsconf->Sections["ModDefaults"][feature.c_str()];
+				SWBuf tmpval = optionsconf->Sections["ModDefaults"][feature.c_str()];
 				if (tmpval.size()) {
-					SWModule *defMod = mainmgr->Modules[tmpval];
+					SWModule *defMod = 0;
+					ModMap::iterator it = mainmgr->Modules.find(tmpval);
+					if (it != mainmgr->Modules.end())
+						defMod = it->second;
 					if (!defMod) {
 						HintStr = "";
 						break;
 					}
 					ListKey key;
 					key << WideStringToUTF8(strongsWord).c_str();
-					RTFHintForm->rtfDrawer->fillWithVerses(defMod, &key, true, false, "Popup");
-					HintStr = "show rtf";
+					bool validEntry = RTFHintForm->rtfDrawer->fillWithVerses(defMod, &key, 0, true, false, "Popup");
+					HintStr = (validEntry) ? "show rtf" : "";
 					break;
 				}
 			}
@@ -2295,15 +2495,18 @@ void __fastcall TForm1::ApplicationEvents1ShowHint(AnsiString &HintStr,
 
 		// try looking up the word in a general dictionary
 		if (!link && hintWords) {
-			string tmpval = optionsconf->Sections["ModDefaults"]["GenDict"];
+			SWBuf tmpval = optionsconf->Sections["ModDefaults"]["GenDict"];
 			if (tmpval.size()) {
-				SWModule *defMod = mainmgr->Modules[tmpval];
+				SWModule *defMod = 0;
+				ModMap::iterator it = mainmgr->Modules.find(tmpval);
+				if (it != mainmgr->Modules.end())
+					defMod = it->second;
 				if (!defMod)
 					break;
 				ListKey key;
 				key << WideStringToUTF8(targetWord).c_str();
-				RTFHintForm->rtfDrawer->fillWithVerses(defMod, &key, true, false, "Popup");
-				HintStr = (key.Count()) ? "show rtf" : "";
+				bool validEntry = RTFHintForm->rtfDrawer->fillWithVerses(defMod, &key, 0, true, false, "Popup");
+				HintStr = (validEntry) ? "show rtf" : "";
 				break;
 			}
 		}
@@ -2361,19 +2564,19 @@ void __fastcall TRTFHintWindow::Paint(void)
 	TRect Rect = ClientRect;
 	
 	TColor penColor;
-	string strColor;
+	SWBuf strColor;
 	strColor = Form1->optionsconf->Sections["Appearance"]["PopupFontColor"];
 	if(strColor == "")
 		penColor = clBlack;
 	else
-		penColor = StrToInt((AnsiString)strColor.c_str());
+		try {penColor = StrToInt((AnsiString)strColor.c_str());} catch (...) {}
 
 	TColor backColor;
 	strColor = Form1->optionsconf->Sections["Appearance"]["PopupBackColor"];
 	if(strColor == "")
 		backColor = 14680063;
 	else
-		backColor = StrToInt((AnsiString)strColor.c_str());
+		try {backColor = StrToInt((AnsiString)strColor.c_str());} catch (...) {}
 
 	Canvas->Brush->Color = penColor;
 	Canvas->FillRect(Rect);
@@ -2520,7 +2723,9 @@ void __fastcall TForm1::SearchForWord1Click(TObject *Sender)
 		int selstart = rtf->SelStart;
 		int versestart, verseend;
 		for (versestart = selstart; versestart; versestart--) {
-			if (!isdigit(entrytext[versestart]))
+			if (!isdigit(entrytext[versestart])
+					&& (toupper(entrytext[versestart]) != entrytext[versestart])
+					&& !strchr("-", entrytext[versestart]))
 				break;
 		}
 		if ((entrytext[versestart] == '<') ||
@@ -2629,3 +2834,59 @@ void __fastcall TForm1::NewVer1Click(TObject *Sender)
 }
 //---------------------------------------------------------------------------
 
+void __fastcall TForm1::PopupMenu5Popup(TObject *Sender)
+{
+	int menuCount = 3;
+	TMenuItem *newitem;
+	
+	TMenuItem **menus = new TMenuItem *[menuCount];
+	TMenu *menu = PopupMenu5;
+
+	while (menu->Items->Count > 4)
+		menu->Items->Delete(4);
+
+	for (int i = 0; i < menuCount; i++) {
+		menus[i] = new TMenuItem(menu);
+		menus[i]->Caption = "Bible " + IntToStr(i+1);
+		menu->Items->Add(menus[i]);
+		newitem = new TMenuItem(menu);
+		newitem->Caption = "[NONE]";
+		newitem->Hint = "Remove This View";
+		newitem->OnClick = ParallelMenuItemClick;
+		menus[i]->Add(newitem);
+	}
+
+	for (ModMap::iterator it = mainmgr->Modules.begin(); it != mainmgr->Modules.end(); it++) {
+		if (!strcmp(it->second->Type(), "Biblical Texts")) {
+			for (int i = 0; i < menuCount; i++) {
+				newitem = new TMenuItem(menu);
+				newitem->Caption = it->second->Name();
+				if(!strcmp(parallelDisp->mod[i].c_str(), newitem->Caption.c_str()))
+					newitem->Checked = true;
+				newitem->Hint = it->second->Description();
+				newitem->OnClick = ParallelMenuItemClick;
+				menus[i]->Add(newitem);
+			}
+		}
+	}
+	delete [] menus;
+	PopupMenuPopup(Sender);	
+}
+//---------------------------------------------------------------------------
+void __fastcall TForm1::ParallelMenuItemClick(TObject *Sender)
+{
+	TMenuItem *menuchoice = (TMenuItem *)Sender;
+	const char *source = menuchoice->Caption.c_str();
+	char *value = new char [strlen(source) + 1];
+	int k = 0;
+	for (unsigned int j = 0; j < strlen(source); j++) {
+		if (source[j] != '&')
+			value[k++] = source[j];
+	}
+	value[k] = 0;
+	int bno = atoi((menuchoice->Parent->Caption.c_str()+menuchoice->Parent->Caption.Length()-2))-1;
+	parallelDisp->mod[bno] = value;
+	parallelDisp->Display();
+	delete [] value;
+}
+
