@@ -4,7 +4,6 @@
 
 #include <localemgr.h>
 #include <list>
-#include <string>
 #include <dirent.h>
 #include <io.h>
 #include <dir.h>
@@ -14,6 +13,11 @@
 
 
 //---------------------------------------------------------------------------
+#pragma link "TntButtons"
+#pragma link "TntComCtrls"
+#pragma link "TntExtCtrls"
+#pragma link "TntMenus"
+#pragma link "TntStdCtrls"
 #pragma resource "*.dfm"
 TOptionsfrm *Optionsfrm;
 
@@ -28,6 +32,7 @@ __fastcall TOptionsfrm::TOptionsfrm(TComponent* Owner)
 	FontDialogLD = new TFontSelFrm(this);
 	FontDialogPopup = new TFontSelFrm(this);
 	FontDialogSearch = new TFontSelFrm(this);
+        FontDialogUI = new TFontSelFrm(this);
 }
 //--------------------------------------------------------------------------- 
 void __fastcall TOptionsfrm::btnCurrVerseClick(TObject *Sender)
@@ -38,20 +43,20 @@ void __fastcall TOptionsfrm::btnCurrVerseClick(TObject *Sender)
 }
 //---------------------------------------------------------------------------
 
-//DONE -oDavid 1 -cOptions: add per module font size selection
+//DONE 1 -oDavid -cOptions: add per module font size selection
 //DONE 1 -cOptions: add 'general' dictionary besides strongs for lookups of words
 //TODO 1 -cOptions: add Individual Modules per module 'general' dictionary selection and support so different dictionaries can be chosen for, say: Chinese and KJV and Thai, etc.
 
 void __fastcall TOptionsfrm::FormCreate(TObject *Sender)
 {
-	list<string> locales;
+	list<SWBuf> locales;
 
 	localeCB->Clear();
 	localeCB->Items->AddObject(defaultLocaleDescription, (TObject *)defaultLocaleName);
 
 	locales = LocaleMgr::systemLocaleMgr.getAvailableLocales();
 
-	for (list<string>::iterator it = locales.begin(); it != locales.end(); it++) {
+	for (list<SWBuf>::iterator it = locales.begin(); it != locales.end(); it++) {
 		const char *name       = LocaleMgr::systemLocaleMgr.getLocale(it->c_str())->getName();
 		const char *description = LocaleMgr::systemLocaleMgr.getLocale(it->c_str())->getDescription();
 		localeCB->Items->AddObject(description, (TObject *)name);
@@ -77,45 +82,52 @@ void __fastcall TOptionsfrm::btnFontClick(TObject *Sender)
 	TFontSelFrm* tempFrm = new TFontSelFrm(this);
 	switch (cmbTextSelect->ItemIndex)
 	{
-	case 0:
+	case 0: // All Windows
 		CopyFontDlg(tempFrm, FontDialogText);
 		if(FontDialogText->ShowModal() == mrOk){
 			CopyFontDlg(FontDialogComment, FontDialogText);
 			CopyFontDlg(FontDialogLD, FontDialogText);
 			CopyFontDlg(FontDialogPopup, FontDialogText);
+                        CopyFontDlg(FontDialogUI, FontDialogText);
 			UpdatePreview();
 		}else CopyFontDlg(FontDialogText, tempFrm);
 		break;
-	case 1:
+	case 1: // Bible Window
 		CopyFontDlg(tempFrm, FontDialogText);
 		if(FontDialogText->ShowModal() == mrOk)
 			UpdatePreview();
 		else CopyFontDlg(FontDialogText, tempFrm);
 		break;
-	case 2:
+	case 2: // Commentary Window
 		CopyFontDlg(tempFrm, FontDialogComment);
 		if(FontDialogComment->ShowModal() == mrOk)
 			UpdatePreview();
 		else CopyFontDlg(FontDialogComment, tempFrm);
 		break;
-	case 3:
+	case 3: // Lexdict Window
 		CopyFontDlg(tempFrm, FontDialogLD);
 		if(FontDialogLD->ShowModal() == mrOk)
 			UpdatePreview();
 		else	CopyFontDlg(FontDialogLD, tempFrm);
 		break;
-	case 4:
+	case 4: // PopUp Windows
 		CopyFontDlg(tempFrm, FontDialogPopup);
 		if(FontDialogPopup->ShowModal() == mrOk)
 			UpdatePreview();
 		else	CopyFontDlg(FontDialogPopup, tempFrm);
 		break;
-	case 5:
+	case 5: // Search Results
 		CopyFontDlg(tempFrm, FontDialogSearch);
 		if(FontDialogSearch->ShowModal() == mrOk)
 			UpdatePreview();
 		else	CopyFontDlg(FontDialogSearch, tempFrm);
 		break;
+        case 6: // User Interface
+		CopyFontDlg(tempFrm, FontDialogUI);
+		if(FontDialogUI->ShowModal() == mrOk)
+			UpdatePreview();
+		else	CopyFontDlg(FontDialogUI, tempFrm);
+		break;
 	}
 	delete tempFrm;
 }
@@ -285,24 +297,28 @@ void __fastcall TOptionsfrm::SaveSchemeClick(TObject *Sender)
 		emap.erase("LDBackColor"); emap.insert(ConfigEntMap::value_type("LDBackColor", IntToStr(FontDialogLD->BackColor).c_str()));
 		emap.erase("PopupBackColor"); emap.insert(ConfigEntMap::value_type("PopupBackColor", IntToStr(FontDialogPopup->BackColor).c_str()));
 		emap.erase("SearchBackColor"); emap.insert(ConfigEntMap::value_type("SearchBackColor", IntToStr(FontDialogSearch->BackColor).c_str()));
+                emap.erase("UIBackColor"); emap.insert(ConfigEntMap::value_type("UIBackColor", IntToStr(FontDialogUI->BackColor).c_str()));
 
 		emap.erase("TextFontName"); emap.insert(ConfigEntMap::value_type("TextFontName", (FontDialogText->Font->Name).c_str()));
 		emap.erase("CommentFontName"); emap.insert(ConfigEntMap::value_type("CommentFontName", (FontDialogComment->Font->Name).c_str()));
 		emap.erase("LDFontName"); emap.insert(ConfigEntMap::value_type("LDFontName", (FontDialogLD->Font->Name).c_str()));
 		emap.erase("PopupFontName"); emap.insert(ConfigEntMap::value_type("PopupFontName", (FontDialogPopup->Font->Name).c_str()));
 		emap.erase("SearchFontName"); emap.insert(ConfigEntMap::value_type("SearchFontName", (FontDialogSearch->Font->Name).c_str()));
+                emap.erase("UIFontName"); emap.insert(ConfigEntMap::value_type("UIFontName", (FontDialogUI->Font->Name).c_str()));
 
 		emap.erase("TextFontSize"); emap.insert(ConfigEntMap::value_type("TextFontSize", IntToStr(FontDialogText->Font->Size).c_str()));
 		emap.erase("CommentFontSize"); emap.insert(ConfigEntMap::value_type("CommentFontSize", IntToStr(FontDialogComment->Font->Size).c_str()));
 		emap.erase("LDFontSize"); emap.insert(ConfigEntMap::value_type("LDFontSize", IntToStr(FontDialogLD->Font->Size).c_str()));
 		emap.erase("PopupFontSize"); emap.insert(ConfigEntMap::value_type("PopupFontSize", IntToStr(FontDialogPopup->Font->Size).c_str()));
 		emap.erase("SearchFontSize"); emap.insert(ConfigEntMap::value_type("SearchFontSize", IntToStr(FontDialogSearch->Font->Size).c_str()));
+                emap.erase("UIFontSize"); emap.insert(ConfigEntMap::value_type("UIFontSize", IntToStr(FontDialogUI->Font->Size).c_str()));
 		
 		emap.erase("TextFontColor"); emap.insert(ConfigEntMap::value_type("TextFontColor", IntToStr(FontDialogText->Font->Color).c_str()));
 		emap.erase("CommentFontColor"); emap.insert(ConfigEntMap::value_type("CommentFontColor", IntToStr(FontDialogComment->Font->Color).c_str()));
 		emap.erase("LDFontColor"); emap.insert(ConfigEntMap::value_type("LDFontColor", IntToStr(FontDialogLD->Font->Color).c_str()));
 		emap.erase("PopupFontColor"); emap.insert(ConfigEntMap::value_type("PopupFontColor", IntToStr(FontDialogPopup->Font->Color).c_str()));
 		emap.erase("SearchFontColor"); emap.insert(ConfigEntMap::value_type("SearchFontColor", IntToStr(FontDialogSearch->Font->Color).c_str()));
+                emap.erase("UIFontColor"); emap.insert(ConfigEntMap::value_type("UIFontColor", IntToStr(FontDialogUI->Font->Color).c_str()));
 		
 		schemeconf.Sections["Appearance"] = emap;
 		schemeconf.Save();
@@ -319,7 +335,7 @@ void TOptionsfrm::FindSchemeNames()
 {
 	DIR *dir;
 	struct dirent *ent;
-	string conffile;
+	SWBuf conffile;
 	AnsiString strName;
 	AnsiString strPath = COLOR_DIR;
 
@@ -511,7 +527,7 @@ void __fastcall TOptionsfrm::shpMorphClrMouseDown(TObject *Sender,
 
 void TOptionsfrm::LoadDispSettings(SWConfig* config) {
 	SectionMap::iterator sit;
-	   string tmpval;
+	   SWBuf tmpval;
 
 	if((tmpval = config->Sections["Appearance"]["AutoVSColor"]) == "0"){
 		AutoVSColor->Checked = false;
@@ -595,6 +611,10 @@ void TOptionsfrm::LoadDispSettings(SWConfig* config) {
 		FontDialogSearch->BackColor = TColor(atoi(tmpval.c_str()));
 	else FontDialogSearch->BackColor = clWhite;
 
+	if((tmpval = config->Sections["Appearance"]["UIBackColor"]) != "")
+		FontDialogUI->BackColor = TColor(atoi(tmpval.c_str()));
+	else FontDialogUI->BackColor = clWhite;
+
 		//****	Font Face Name 	****//
 	if((tmpval = config->Sections["Appearance"]["TextFontName"]) != "")
 		FontDialogText->Font->Name =  tmpval.c_str();
@@ -615,7 +635,11 @@ void TOptionsfrm::LoadDispSettings(SWConfig* config) {
 	if((tmpval = config->Sections["Appearance"]["SearchFontName"]) != "")
 		FontDialogSearch->Font->Name =  tmpval.c_str();
 	else FontDialogSearch->Font->Name = "Times New Roman";
-			
+
+	if((tmpval = config->Sections["Appearance"]["UIFontName"]) != "")
+		FontDialogUI->Font->Name =  tmpval.c_str();
+	else FontDialogUI->Font->Name = "MS Sans Serif";
+
 		//****	Font Size			****//
 	if((tmpval = config->Sections["Appearance"]["TextFontSize"]) != "")
 		FontDialogText->Font->Size =  atoi(tmpval.c_str());
@@ -636,7 +660,11 @@ void TOptionsfrm::LoadDispSettings(SWConfig* config) {
 	if((tmpval = config->Sections["Appearance"]["SearchFontSize"]) != "")
 		FontDialogSearch->Font->Size =  atoi(tmpval.c_str());
 	else FontDialogSearch->Font->Size = 10;
-			
+
+	if((tmpval = config->Sections["Appearance"]["UIFontSize"]) != "")
+		FontDialogUI->Font->Size =  atoi(tmpval.c_str());
+	else FontDialogUI->Font->Size = 8;
+
 		//****	Font ForeColor		****//
 	if((tmpval = config->Sections["Appearance"]["TextFontColor"]) != "")
 		FontDialogText->Font->Color = TColor(atoi(tmpval.c_str()));
@@ -657,6 +685,10 @@ void TOptionsfrm::LoadDispSettings(SWConfig* config) {
 	if((tmpval = config->Sections["Appearance"]["SearchFontColor"]) != "")
 		FontDialogSearch->Font->Color = TColor(atoi(tmpval.c_str()));
 	else FontDialogSearch->Font->Color = clBlack;
+
+	if((tmpval = config->Sections["Appearance"]["UIFontColor"]) != "")
+		FontDialogUI->Font->Color = TColor(atoi(tmpval.c_str()));
+	else FontDialogUI->Font->Color = clBlack;
 }
 
 
@@ -666,7 +698,7 @@ void TOptionsfrm::LoadMiscSettings(SWConfig* config) {
 	ConfigEntMap::iterator eit;
 	ConfigEntMap emap;
 	int color;
-	string tmpval;
+	SWBuf tmpval;
 
 	Optionsfrm->AutoBMPersonal->Checked = true;
 	Optionsfrm->AutoLayout->Checked = true;
@@ -693,7 +725,7 @@ void TOptionsfrm::LoadMiscSettings(SWConfig* config) {
 
 
 	Optionsfrm->devsAsDictsCB->Checked = false;
-	tmpval = ((eit = config->Sections["General"].find("DevsAsDicts")) != config->Sections["General"].end())? (*eit).second : (string)"false";
+	tmpval = ((eit = config->Sections["General"].find("DevsAsDicts")) != config->Sections["General"].end())? (*eit).second : (SWBuf)"false";
 	if (!stricmp(tmpval.c_str(), "true"))
 		Optionsfrm->devsAsDictsCB->Checked = true;
 
@@ -703,7 +735,7 @@ void TOptionsfrm::LoadMiscSettings(SWConfig* config) {
 		Optionsfrm->glosAsDictsCB->Checked = false;
 
 	Optionsfrm->devSplashCB->Checked = false;
-	tmpval = ((eit = config->Sections["General"].find("TipOfTheDay")) != config->Sections["General"].end())? (*eit).second : (string)"";
+	tmpval = ((eit = config->Sections["General"].find("TipOfTheDay")) != config->Sections["General"].end())? (*eit).second : (SWBuf)"";
 	if (!stricmp(tmpval.c_str(), "Devotional"))
 		Optionsfrm->devSplashCB->Checked = true;
 
@@ -711,37 +743,37 @@ void TOptionsfrm::LoadMiscSettings(SWConfig* config) {
 	SWLocale *defLocale = LocaleMgr::systemLocaleMgr.getLocale(localeName);
 	Optionsfrm->localeCB->ItemIndex = Optionsfrm->localeCB->Items->IndexOf((defLocale)?defLocale->getDescription():"English (US)");
 
-	tmpval = ((eit = config->Sections["ModDefaults"].find("GreekDef")) != config->Sections["ModDefaults"].end())? (*eit).second : (string)"";
+	tmpval = ((eit = config->Sections["ModDefaults"].find("GreekDef")) != config->Sections["ModDefaults"].end())? (*eit).second : (SWBuf)"";
 	if ((it = Form1->mainmgr->Modules.find(tmpval)) != Form1->mainmgr->Modules.end()) {
 		Optionsfrm->greekDefCB->ItemIndex = Optionsfrm->greekDefCB->Items->IndexOf(it->second->Description());
 	}
 
-	tmpval = ((eit = config->Sections["ModDefaults"].find("GreekParse")) != config->Sections["ModDefaults"].end())? (*eit).second : (string)"";
+	tmpval = ((eit = config->Sections["ModDefaults"].find("GreekParse")) != config->Sections["ModDefaults"].end())? (*eit).second : (SWBuf)"";
 	if ((it = Form1->mainmgr->Modules.find(tmpval)) != Form1->mainmgr->Modules.end()) {
 		Optionsfrm->greekParseCB->ItemIndex = Optionsfrm->greekParseCB->Items->IndexOf(it->second->Description());
 	}
 
-	tmpval = ((eit = config->Sections["ModDefaults"].find("HebrewDef")) != config->Sections["ModDefaults"].end())? (*eit).second : (string)"";
+	tmpval = ((eit = config->Sections["ModDefaults"].find("HebrewDef")) != config->Sections["ModDefaults"].end())? (*eit).second : (SWBuf)"";
 	if ((it = Form1->mainmgr->Modules.find(tmpval)) != Form1->mainmgr->Modules.end()) {
 		Optionsfrm->hebrewDefCB->ItemIndex = Optionsfrm->hebrewDefCB->Items->IndexOf(it->second->Description());
 	}
 
-	tmpval = ((eit = config->Sections["ModDefaults"].find("HebrewParse")) != config->Sections["ModDefaults"].end())? (*eit).second : (string)"";
+	tmpval = ((eit = config->Sections["ModDefaults"].find("HebrewParse")) != config->Sections["ModDefaults"].end())? (*eit).second : (SWBuf)"";
 	if ((it = Form1->mainmgr->Modules.find(tmpval)) != Form1->mainmgr->Modules.end()) {
 		Optionsfrm->hebrewParseCB->ItemIndex = Optionsfrm->hebrewParseCB->Items->IndexOf(it->second->Description());
 	}
 
-	tmpval = ((eit = config->Sections["ModDefaults"].find("DailyDevotion")) != config->Sections["ModDefaults"].end())? (*eit).second : (string)"";
+	tmpval = ((eit = config->Sections["ModDefaults"].find("DailyDevotion")) != config->Sections["ModDefaults"].end())? (*eit).second : (SWBuf)"";
 	if ((it = Form1->mainmgr->Modules.find(tmpval)) != Form1->mainmgr->Modules.end()) {
 		Optionsfrm->dailyDefaultCB->ItemIndex = Optionsfrm->dailyDefaultCB->Items->IndexOf(it->second->Description());
 	}
 
-	tmpval = ((eit = config->Sections["ModDefaults"].find("GenDict")) != config->Sections["ModDefaults"].end())? (*eit).second : (string)"";
+	tmpval = ((eit = config->Sections["ModDefaults"].find("GenDict")) != config->Sections["ModDefaults"].end())? (*eit).second : (SWBuf)"";
 	if ((it = Form1->mainmgr->Modules.find(tmpval)) != Form1->mainmgr->Modules.end()) {
 		Optionsfrm->genDictCB->ItemIndex = Optionsfrm->genDictCB->Items->IndexOf(it->second->Description());
 	}
 
-	tmpval = ((eit = config->Sections["ModDefaults"].find("StrongsNumbers")) != config->Sections["ModDefaults"].end())? (*eit).second : (string)"";
+	tmpval = ((eit = config->Sections["ModDefaults"].find("StrongsNumbers")) != config->Sections["ModDefaults"].end())? (*eit).second : (SWBuf)"";
 	if ((it = Form1->mainmgr->Modules.find(tmpval)) != Form1->mainmgr->Modules.end()) {
 		Optionsfrm->strongsNumsCB->ItemIndex = Optionsfrm->strongsNumsCB->Items->IndexOf(it->second->Description());
 	}
@@ -754,7 +786,7 @@ void TOptionsfrm::extractSettings(SWConfig* config) {
 	ConfigEntMap::iterator eit;
 	ConfigEntMap emap;
 	int color;
-	string tmpval;
+	SWBuf tmpval;
 
 	emap = config->Sections["Appearance"];
 	emap["AutoVSColor"]      = IntToStr((Optionsfrm->AutoVSColor->Checked)?1:0).c_str();
@@ -769,24 +801,28 @@ void TOptionsfrm::extractSettings(SWConfig* config) {
 	emap["LDBackColor"]      = IntToStr(Optionsfrm->FontDialogLD->BackColor).c_str();
 	emap["PopupBackColor"]   = IntToStr(Optionsfrm->FontDialogPopup->BackColor).c_str();
 	emap["SearchBackColor"]   = IntToStr(Optionsfrm->FontDialogSearch->BackColor).c_str();
-	    
+        emap["UIBackColor"]   = IntToStr(Optionsfrm->FontDialogUI->BackColor).c_str();
+
 	emap["TextFontName"]     = (Optionsfrm->FontDialogText->Font->Name).c_str();
 	emap["CommentFontName"]  = (Optionsfrm->FontDialogComment->Font->Name).c_str();
 	emap["LDFontName"]       = (Optionsfrm->FontDialogLD->Font->Name).c_str();
 	emap["PopupFontName"]    = (Optionsfrm->FontDialogPopup->Font->Name).c_str();
 	emap["SearchFontName"]    = (Optionsfrm->FontDialogSearch->Font->Name).c_str();
+        emap["UIFontName"]    = (Optionsfrm->FontDialogUI->Font->Name).c_str();
 
 	emap["TextFontSize"]     = IntToStr(Optionsfrm->FontDialogText->Font->Size).c_str();
 	emap["CommentFontSize"]  = IntToStr(Optionsfrm->FontDialogComment->Font->Size).c_str();
 	emap["LDFontSize"]       = IntToStr(Optionsfrm->FontDialogLD->Font->Size).c_str();
 	emap["PopupFontSize"]    = IntToStr(Optionsfrm->FontDialogPopup->Font->Size).c_str();
 	emap["SearchFontSize"]    = IntToStr(Optionsfrm->FontDialogSearch->Font->Size).c_str();
-	    
+        emap["UIFontSize"]    = IntToStr(Optionsfrm->FontDialogUI->Font->Size).c_str();
+
 	emap["TextFontColor"]    = IntToStr(Optionsfrm->FontDialogText->Font->Color).c_str();
 	emap["CommentFontColor"] = IntToStr(Optionsfrm->FontDialogComment->Font->Color).c_str();
 	emap["LDFontColor"]      = IntToStr(Optionsfrm->FontDialogLD->Font->Color).c_str();
 	emap["PopupFontColor"]   = IntToStr(Optionsfrm->FontDialogPopup->Font->Color).c_str();
 	emap["SearchFontColor"]   = IntToStr(Optionsfrm->FontDialogSearch->Font->Color).c_str();
+        emap["UIFontColor"]   = IntToStr(Optionsfrm->FontDialogUI->Font->Color).c_str();
 	    
 	emap["locale"]           = (const char *)Optionsfrm->localeCB->Items->Objects[Optionsfrm->localeCB->ItemIndex];
 
