@@ -117,7 +117,8 @@ void __fastcall TsearchForm::ListBox1DblClick(TObject *Sender)
 		SWKey *key = (SWKey *)*target;
 		(*key) = resultsLV->Items->Item[row]->Caption.c_str();
 		if (targetpc == Form1->LexDictPageControl) {
-			Form1->DictKeyEdit->Text = resultsLV->Items->Item[row]->Caption.c_str();
+			//Form1->DictKeyEdit->Text = UTF8ToWideString(resultsLV->Items->Item[row]->Caption.c_str());
+                        Form1->DictKeyEdit->Text = resultsLV->Items->Item[row]->Caption.c_str();
 		}
 		else {
 			Form1->TextKeyChanged();
@@ -138,14 +139,14 @@ void __fastcall TsearchForm::ListBox1Click(TObject *Sender)
 			*key = resultsLV->Items->Item[row]->Caption.c_str();
 			verse << *key;
 			delete key;
-			pvrtf->fillWithVerses(target, &verse, true, false, "Search");
+			pvrtf->fillWithVerses(target, &verse, 0, true, false, "Search");
 		}
 	}
 }
 
 //---------------------------------------------------------------------------
 
-__fastcall TsearchForm::TSearchThread::TSearchThread(TsearchForm *parent, string searchText, int searchType, int searchOptions, SWKey *scope, bool CreateSuspended)
+__fastcall TsearchForm::TSearchThread::TSearchThread(TsearchForm *parent, SWBuf searchText, int searchType, int searchOptions, SWKey *scope, bool CreateSuspended)
 	: TThread(CreateSuspended)
 {
 	this->parent = parent;
@@ -265,14 +266,14 @@ void __fastcall TsearchForm::resultsLVCustomDrawItem(TCustomListView *Sender, TL
 	if (target && !Form1->closing) {
 		TRect rect = Item->DisplayRect(drSelectBounds);
 		static UnicodeRTF filter;
-		char buf[255];
-		strcpy(buf, Item->Caption.c_str());
+		SWBuf buf;
+		buf =  Item->Caption.c_str();
 
 		SWKey *key = *target;
 		if (!SWDYNAMIC_CAST(VerseKey, key))
-			filter.ProcessText(buf, 253, *target, target);
+			filter.processText(buf, *target, target);
 
-		RTFHintForm->searchListDrawer->fillWithRTFString(target, ((String)" \\b  " + buf + " ").c_str(), "Search");
+		RTFHintForm->searchListDrawer->fillWithRTFString(target, ((String)" \\b  " + buf.c_str() + " ").c_str(), "Search");
 		if (resultsLV->Selected == Item) RTFHintForm->searchListDrawer->Color = TColor(0xB4CDBB);
 		Sender->Canvas->Brush->Color = RTFHintForm->searchListDrawer->Color;
 		Sender->Canvas->FillRect(rect);
@@ -282,7 +283,7 @@ void __fastcall TsearchForm::resultsLVCustomDrawItem(TCustomListView *Sender, TL
 
 		ListKey verse;
 		verse << Item->Caption.c_str();
-		RTFHintForm->searchListDrawer->fillWithVerses(target, &verse, false, false, "Search", true);
+		RTFHintForm->searchListDrawer->fillWithVerses(target, &verse, 0, false, false, "Search", true);
 		//set color back in case it got changed in fill*
 		if (resultsLV->Selected == Item) RTFHintForm->searchListDrawer->Color = TColor(0xB4CDBB);
 
@@ -335,9 +336,17 @@ void __fastcall TsearchForm::FormActivate(TObject *Sender)
 				targetpc = Form1->LexDictPageControl;
 
 	
-			ModMap::iterator it = Form1->mainmgr->Modules.find(targetpc->ActivePage->Caption.c_str());
-			if (it != Form1->mainmgr->Modules.end()) {
-				setTarget(it->second);
+			if (targetpc->ActivePage->Caption == "PARALLEL") {
+				SWModule *mod = Form1->parallelDisp->getModules()[0];
+				if (mod) {
+					setTarget(mod);
+				}
+			}
+			else {
+				ModMap::iterator it = Form1->mainmgr->Modules.find(targetpc->ActivePage->Caption.c_str());
+				if (it != Form1->mainmgr->Modules.end()) {
+					setTarget(it->second);
+				}
 			}
 			SearchText->SetFocus();
 		}
@@ -360,7 +369,7 @@ void TsearchForm::fillList()
 	BibleCSMGR *manager;
 	ModMap::iterator mods;
 	TTreeNode *node;
-	string nodeName;
+	SWBuf nodeName;
 	manager = Form1->mainmgr;
 
 	modTreeView->Items->Clear();
@@ -397,7 +406,7 @@ void __fastcall TsearchForm::modTreeViewChange(TObject *Sender, TTreeNode *Node)
 	BibleCSMGR *manager;
 	ModMap::iterator mods;
 	TTreeNode *node;
-	string nodeName;
+	SWBuf nodeName;
 	manager = Form1->mainmgr;
 	TTreeNode* currNode = modTreeView->Selected;
 	AnsiString nodeSubString;
@@ -413,13 +422,13 @@ void __fastcall TsearchForm::modTreeViewChange(TObject *Sender, TTreeNode *Node)
 
 void __fastcall TsearchForm::Button2Click(TObject *Sender)
 {
-	string helpDir;
+	SWBuf helpDir;
 	ConfigEntMap::iterator it = Form1->optionsconf->Sections["Help"].find("Directory");
 	if (it != Form1->optionsconf->Sections["Help"].end())
 		helpDir = (*it).second;
 	else helpDir = ".\\help";
 
-	string helpExe = helpDir + "\\Sword.chm::searchh.html";
+	SWBuf helpExe = helpDir + "\\Sword.chm::searchh.html";
 	if ((int)ShellExecute(this->Handle, "open", "hh", helpExe.c_str(), NULL, SW_SHOWNORMAL) < 33) {
 		helpExe = helpDir + "\\searchh.html";
 		ShellExecute(this->Handle, "open", helpExe.c_str(), NULL, NULL, SW_SHOWNORMAL);
