@@ -75,7 +75,7 @@ __published:	// IDE-managed Components
 private:	// User declarations
 	class TSearchThread : public TThread {
 		ListKey results;
-		string searchText;
+		SWBuf searchText;
 		int searchType;
 		int searchOptions;
 		SWKey *scope;
@@ -83,7 +83,7 @@ private:	// User declarations
 	protected:
 		void __fastcall Execute();
 	public:
-		__fastcall TSearchThread(TsearchForm *parent, string searchText, int searchType, int searchOptions, SWKey *scope = 0, bool CreateSuspended = false);
+		__fastcall TSearchThread(TsearchForm *parent, SWBuf searchText, int searchType, int searchOptions, SWKey *scope = 0, bool CreateSuspended = false);
 		void __fastcall FillListBox(void);
 		static void searchProgressCallback(char status, void *searchFrm);
 		void __fastcall updateStatus(void);
