@@ -35,15 +35,24 @@ __fastcall TVerseListForm::TVerseListForm(TComponent* Owner, ListKey &iVerseList
 //---------------------------------------------------------------------------
 
 void __fastcall TVerseListForm::ListBox1Click(TObject *Sender) {
-	ModMap::iterator target;
-	target = Form1->mainmgr->Modules.find(Form1->TextPageControl->ActivePage->Caption.c_str());
-	if (target != Form1->mainmgr->Modules.end()) {
+	SWModule *target = 0;
+
+	if (Form1->TextPageControl->ActivePage->Caption == "PARALLEL") {
+		target = Form1->parallelDisp->getModules()[0];
+	}
+	else {
+		ModMap::iterator it;
+		it = Form1->mainmgr->Modules.find(Form1->TextPageControl->ActivePage->Caption.c_str());
+		if (it != Form1->mainmgr->Modules.end())
+			target = it->second;
+	}
+	if (target) {
 		ListKey key;
-		SWKey *tkey = target->second->CreateKey();
+		SWKey *tkey = target->CreateKey();
 		*tkey = ListBox1->Items->Strings[ListBox1->ItemIndex].c_str();
 		key << *tkey;
 		delete tkey;
-		pvrtf->fillWithVerses(target->second, &key, true, false);
+		pvrtf->fillWithVerses(target, &key, 0, true, false);
 	}
 }
 //---------------------------------------------------------------------------
@@ -107,9 +116,19 @@ void __fastcall TVerseListForm::SpeedButton3Click(TObject *Sender)
 	if ((Form1->ActiveControl == Form1->LexDictPageControl) || (IsChild(Form1->LexDictPageControl->Handle, Form1->ActiveControl->Handle)))
 		targetpc = Form1->LexDictPageControl;
 
-	ModMap::iterator it = Form1->mainmgr->Modules.find(targetpc->ActivePage->Caption.c_str());
-	if (it != Form1->mainmgr->Modules.end()) {
-		RTFHintForm->rtfDrawer->fillWithVerses(it->second, &verses, true, true);
+	SWModule *target = 0;
+
+	if (targetpc->ActivePage->Caption == "PARALLEL") {
+		target = Form1->parallelDisp->getModules()[0];
+	}
+	else {
+		ModMap::iterator it;
+		it = Form1->mainmgr->Modules.find(targetpc->ActivePage->Caption.c_str());
+		if (it != Form1->mainmgr->Modules.end())
+			target = it->second;
+	}
+	if (target) {
+		RTFHintForm->rtfDrawer->fillWithVerses(target, &verses, 0, true, true);
 		RTFHintForm->rtfDrawer->SelectAll();
 		RTFHintForm->rtfDrawer->CopyToClipboard();
 	}
@@ -132,9 +151,9 @@ void __fastcall TVerseListForm::SpeedButton5Click(TObject *Sender)
 	SetCurrentDir(ExtractFilePath(Application->ExeName));
 	if (result) {
 		if (FileMgr::existsFile(VLSaveDialog->FileName.c_str())) {
-			string message = _tr("File exists.  Overwrite?");
-			string header = _tr("Overwrite File");
-			if (MessageBox(Handle, message.c_str(), header.c_str(), MB_YESNO) != IDYES) {
+			WideString message = _tr("File exists.  Overwrite?");
+			WideString header = _tr("Overwrite File");
+			if (MessageBoxW(Handle, message, header, MB_YESNO) != IDYES) {
 				return;
 			}
 		}
