@@ -2,9 +2,6 @@
 #include <vcl.h>
 #pragma hdrstop
 
-extern "C" {
-#include <untgz.h>
-}
 
 #include "MainFrm.h"
 #include "StatusFrm.h"
@@ -24,84 +21,81 @@ extern "C" {
 #include <sys/stat.h>
 #include <dir.h>
 #include <FileCtrl.hpp>
+#include <installmgr.h>
+#include <filemgr.h>
 
 TMainForm *MainForm;
 
-	
-VersionInfo::VersionInfo(const char *version) {
-	char *buf = new char[ strlen(version) + 1 ];
-	char *tok;
-	major = minor = minor2 = minor3 = 0;
-		
-	strcpy(buf, version);
-	tok = strtok(buf, ".");
-	if (tok)
-		major = atoi(tok);
-	tok = strtok(0, ".");
-	if (tok)
-		minor = atoi(tok);
-	tok = strtok(0, ".");
-	if (tok)
-		minor2 = atoi(tok);
-	tok = strtok(0, ".");
-	if (tok)
-		minor3 = atoi(tok);
-	delete [] buf;
+int InstallMgrWin::FTPCopy(InstallSource *is, const char *src, const char *dest, bool dirTransfer, const char *suffix) {
+	StatusForm->is = is;
+	StatusForm->src = src;
+	StatusForm->dest = dest;
+	StatusForm->suffix = suffix;
+	StatusForm->dirTransfer = dirTransfer;
+	Application->ProcessMessages();
+	if (StatusForm->ShowModal() == mrCancel)
+		return -1;
+	else	return 0;
 }
 
-
-int VersionInfo::compare(const VersionInfo &vi) const {
-	if (major == vi.major)
-		if (minor == vi.minor)
-			if (minor2 == vi.minor2)
-				if (minor3 == vi.minor3)
-					return 0;
-				else return minor3 - vi.minor3;
-			else	return minor2 - vi.minor2;
-		else	return minor - vi.minor;
-	else	return major - vi.major;
+void InstallMgrWin::preDownloadStatus(long totalBytes, long completedBytes, const char *message) {
+	StatusForm->totalBytes = totalBytes;
+	StatusForm->completedBytes = completedBytes;
+	StatusForm->buffer = message;
+	StatusForm->PreDownload2();
+	Application->ProcessMessages();
 }
 
-
-__fastcall InstallSourceTab::InstallSourceTab(TComponent *Owner, const char *confEnt, const char *type) : TControl(Owner) {
-	char *buf = new char [ strlen(confEnt) + 1 ];
+void InstallMgrWin::statusUpdate(double dltotal, double dlnow) {
+	if (!dltotal || !StatusForm->totalBytes)
+		return;	// prevent division by zero error below
+	int filePercent  = (int)((float)(dlnow + 1) / (float)(dltotal) * 100);
+	int totalPercent = (int)((float)(dlnow + StatusForm->completedBytes + 1) / (float)(StatusForm->totalBytes) * 100);
+	StatusForm->statusBar->Caption = ::IntToStr((long)dlnow) + " bytes out of " + ::IntToStr((long)dltotal) + " transferred (file: " + IntToStr(filePercent) + "% / total: " + IntToStr(totalPercent) + "%)";
+	StatusForm->fileProgress->Position = filePercent;
+	StatusForm->totalProgress->Position = totalPercent;
 	
-	strcpy(buf, confEnt);
+	StatusForm->statusBar->Repaint();
+	Application->ProcessMessages();
+}
 
-	Caption = strtok(buf, "|");
-	Source = strtok(0, "|");
-	Directory = strtok(0, "|");
-	delete [] buf;
-	Type = type;
-	mgr = 0;
+bool InstallMgrWin::getCipherCode(const char *modName, SWConfig *config) {
+	CipherForm->modName = modName;
+	CipherForm->config = config;
+	return (CipherForm->ShowModal() == mrCancel);
+}
+
+__fastcall InstallSourceTab::InstallSourceTab(TComponent *Owner, InstallSource *is) : TControl(Owner) {
+	this->is = is;
 }
 
 __fastcall InstallSourceTab::~InstallSourceTab() {
-	if (mgr)
-		delete mgr;
 }
 	
 __fastcall TMainForm::TMainForm(TComponent* Owner)
-	: TForm(Owner)
-{
-     try {
+		: TForm(Owner) {
+	try {
 	    manager = new SWMgr();
-     }
-     catch (...) {
-           createPathAndFile("./mods.d/globals.conf");           
-           manager = new SWMgr();
-     }
-	installConf = new SWConfig("./InstallMgr.conf");
+	}
+	catch (...) {
+		 FileMgr::createPathAndFile("./mods.d/globals.conf");           
+		 manager = new SWMgr();
+	}
+
+	if (FileMgr::existsFile("./InstallMgr.conf")) {
+		FileMgr::copyFile("./InstallMgr.conf", "installMgr/InstallMgr.conf");
+		FileMgr::removeFile("./InstallMgr.conf");
+	}
+	installMgr = new InstallMgrWin("./installMgr");
 	localMgr = 0;
 }
 
 
-__fastcall TMainForm::~TMainForm()
-{
+__fastcall TMainForm::~TMainForm() {
 	delete manager;
-	delete installConf;
 	if (localMgr)
 		delete localMgr;
+	delete installMgr;
 }
 
 
@@ -114,39 +108,26 @@ void __fastcall TMainForm::FormCreate(TObject *Sender) {
 	SWLog::systemlog = new SWWinLog(this->Handle);		// set the system logger to our MSWindows specific SWLog class
 
 	fillInstallTree();
-	fillSourceTree(getLocalDir(), localTree);
+	fillSourceTree(localTree);
 }
 
 
 void TMainForm::refreshPageControl() {
-	SectionMap::iterator sources;
-	ConfigEntMap::iterator sourceBegin;
-	ConfigEntMap::iterator sourceEnd;
+	InstallSourceMap::iterator source;
 
 	while (PageControl1->PageCount > 1) {
 		delete PageControl1->Pages[1];
 	}
 	
-	sources = installConf->Sections.find("Sources");
-	passive = (!stricmp((*installConf)["General"]["PassiveFTP"].c_str(), "true"));
-
-	if (sources != installConf->Sections.end()) {
-		sourceBegin = sources->second.lower_bound("FTPSource");
-		sourceEnd = sources->second.upper_bound("FTPSource");
-
-		if (sourceBegin != sourceEnd) {
-			while (sourceBegin != sourceEnd) {
-				InstallSourceTab *ist = new InstallSourceTab(this, sourceBegin->second.c_str(), "FTP");
-				addSource(ist);
-				sourceBegin++;
-			}
-		}
+	for (source = installMgr->sources.begin(); source != installMgr->sources.end(); source++) {
+		InstallSourceTab *ist = new InstallSourceTab(this, source->second);
+		source->second->userData = (void *)ist;
+		addSource(ist);
 	}
 }
 
 
-void TMainForm::addSource(InstallSourceTab *ist)
-{
+void TMainForm::addSource(InstallSourceTab *ist) {
 	TTabSheet *newtab = new TTabSheet(this);
 	TTreeView *newtree = new TTreeView(this);
 	TSpeedButton *newbutton = new TSpeedButton(this);
@@ -154,8 +135,8 @@ void TMainForm::addSource(InstallSourceTab *ist)
 	char buf[512];
 	SectionMap::iterator sit;
 	
-	newtab->Caption = ist->Caption.c_str();
-	newtab->Hint = ist->Source.c_str();
+	newtab->Caption = ist->is->caption.c_str();
+	newtab->Hint = ist->is->source.c_str();
 	newtab->ShowHint = true;
 	newtab->PageControl = PageControl1;
 	ist->Parent = newtab;
@@ -177,71 +158,44 @@ void TMainForm::addSource(InstallSourceTab *ist)
 	newtree->OnDblClick = localTreeDblClick;
 	newtree->Images = ImageList1;
 	newtree->StateImages = ImageList2;
-	string parent = "./sources/" + ist->Source + "/file";
-	createParent(parent.c_str());
-	parent = "./sources/" + ist->Source;
-	fillSourceTree(parent.c_str(), newtree);
+	fillSourceTree(newtree, ist->is);
 }
 
 
-const char *TMainForm::getLocalDir()
-{
+const char *TMainForm::getLocalDir() {
 	ConfigEntMap::iterator entry;
 
-	entry = installConf->Sections["Sources"].find("LocalPath");
-	if (entry == installConf->Sections["Sources"].end()) {
-		installConf->Sections["Sources"].insert(ConfigEntMap::value_type("LocalPath", "d:/sword"));
-		entry = installConf->Sections["Sources"].find("LocalPath");
+	entry = installMgr->installConf->Sections["Sources"].find("LocalPath");
+	if (entry == installMgr->installConf->Sections["Sources"].end()) {
+		installMgr->installConf->Sections["Sources"].insert(ConfigEntMap::value_type("LocalPath", "d:/sword"));
+		entry = installMgr->installConf->Sections["Sources"].find("LocalPath");
 	}
 	return entry->second.c_str();
-		
 }
 
 
-void TMainForm::setLocalDir(const char *idir)
-{
-	installConf->Sections["Sources"].erase("LocalPath"); installConf->Sections["Sources"].insert(ConfigEntMap::value_type("LocalPath", idir));
+void TMainForm::setLocalDir(const char *idir) {
+	(*(installMgr->installConf))["Sources"]["LocalPath"] = idir;
 }
 
 
 //---------------------------------------------------------------------------
-void __fastcall TMainForm::LocalPath1Click(TObject *Sender)
-{
-     AnsiString Dir = "C:";
-     WideString Root = getLocalDir();
-     SelectDirectory("Select Local Path", Root , Dir);
+void __fastcall TMainForm::LocalPath1Click(TObject *Sender) {
+	AnsiString Dir = "C:";
+	WideString Root = getLocalDir();
+	SelectDirectory("Select Local Path", Root , Dir);
 
-/*
-	localPathDlg->Title = "Select a file in the directory to choose";
-	localPathDlg->InitialDir = getLocalDir();
-	if (localPathDlg->Execute()) {
-		char *buf = new char [ strlen(localPathDlg->FileName.c_str()) + 1 ];
+	setLocalDir(Dir.c_str());
 
-		strcpy(buf, localPathDlg->FileName.c_str());
-		int end = strlen(buf) - 1;
-		while (end) {
-			if ((buf[end] == '/') || (buf[end] == '\\'))
-				break;
-			end--;
-		}
-		buf[end+1] = 0;
-		setLocalDir(buf);
-		delete [] buf;
-	}
-*/
-
-        setLocalDir(Dir.c_str());
-
-	installConf->Save();
-	fillSourceTree(getLocalDir(), localTree);
+	installMgr->installConf->Save();
+	fillSourceTree(localTree);
 }
 //---------------------------------------------------------------------------
 
-void TMainForm::fillInstallTree()
-{
+void TMainForm::fillInstallTree() {
 	ModMap::iterator mods;
 	TTreeNode *node;
-	string nodeName;
+	SWBuf nodeName;
 	
 	installTree->Items->Clear();
 
@@ -273,17 +227,17 @@ void TMainForm::fillInstallTree()
 }
 
 
-void TMainForm::fillSourceTree(const char *sourceConf, TTreeView *tree)
-{
+void TMainForm::fillSourceTree(TTreeView *tree, InstallSource *is) {
 	ModMap::iterator mods;
 	TTreeNode *node;
 	SectionMap::iterator sections, targetSection;
 	ConfigEntMap::iterator entry;
-	string secName;
+	SWBuf secName;
 	const char * modDesc;
-	string nodeName;
-	string targetVersion;
-	string sourceVersion;
+	SWBuf nodeName;
+	SWBuf targetVersion;
+	SWBuf sourceVersion;
+	SWBuf softwareVersion;
 	bool cipher;
 	bool showLocked = LockedModules1->Checked;
 	
@@ -292,14 +246,11 @@ void TMainForm::fillSourceTree(const char *sourceConf, TTreeView *tree)
 	if (tree == localTree) {
 		if (localMgr)
 			delete localMgr;
-		mgr = localMgr = new SWMgr(sourceConf);
+		mgr = localMgr = new SWMgr(getLocalDir());
 	}
 	else {
-     	
 		InstallSourceTab *ist = (InstallSourceTab *) tree->Parent->Controls[0];
-		if (ist->mgr)
-			delete ist->mgr;
-		mgr = ist->mgr = new SWMgr(sourceConf);
+		mgr = ist->is->getMgr();
 	}
 	
 	tree->Items->Clear();
@@ -328,9 +279,9 @@ void TMainForm::fillSourceTree(const char *sourceConf, TTreeView *tree)
 			secName = "Other";
 		}
 
-		string misc1 = ((entry = sections->second.find("Category")) != sections->second.end()) ? (*entry).second : (string)"";
-          if (misc1.length() > 0)
-             secName = misc1;
+		SWBuf misc1 = ((entry = sections->second.find("Category")) != sections->second.end()) ? (SWBuf)(*entry).second.c_str() : (SWBuf)"";
+		if (misc1.length() > 0)
+		   secName = misc1;
 
 		entry = sections->second.find("Description");
 		if (entry != sections->second.end())
@@ -340,11 +291,16 @@ void TMainForm::fillSourceTree(const char *sourceConf, TTreeView *tree)
 		
 		targetVersion = "0.0";
 		sourceVersion = "1.0";
+		softwareVersion = (const char *)SWVersion::currentVersion;
 		
 		entry = sections->second.find("Version");
 		if (entry != sections->second.end())
 			sourceVersion = entry->second.c_str();
 
+		entry = sections->second.find("MinimumVersion");
+		if (entry != sections->second.end())
+			softwareVersion = entry->second.c_str();
+
 		targetSection = manager->config->Sections.find(sections->first);
 		if (targetSection != manager->config->Sections.end()) {
 			targetVersion = "1.0";
@@ -353,7 +309,7 @@ void TMainForm::fillSourceTree(const char *sourceConf, TTreeView *tree)
 				targetVersion = entry->second;
 		}
 
-		if (VersionInfo(sourceVersion.c_str()) > VersionInfo(targetVersion.c_str())) {
+		if (SWVersion(sourceVersion.c_str()) > SWVersion(targetVersion.c_str())) {
 			for (node = tree->Items->GetFirstNode(); node; node = node->getNextSibling()) {
 				if (!strcmp(node->Text.c_str(), secName.c_str())) {
 					break;
@@ -366,9 +322,9 @@ void TMainForm::fillSourceTree(const char *sourceConf, TTreeView *tree)
 				else	node = tree->Items->AddChild(0, secName.c_str());
 				node->ImageIndex = 0;
 			}
-			nodeName = "[" + sections->first + "] " + modDesc;
+			nodeName = (SWBuf)"[" + sections->first + "] " + modDesc;
 			node = tree->Items->AddChildObject(node, nodeName.c_str(), (void *) sections->first.c_str());
-			if (VersionInfo(targetVersion.c_str()) < VersionInfo("1.0")) {
+			if (SWVersion(targetVersion.c_str()) < SWVersion("1.0")) {
 				node->ImageIndex = 1;
 				node->SelectedIndex = 1;
 			}
@@ -380,6 +336,10 @@ void TMainForm::fillSourceTree(const char *sourceConf, TTreeView *tree)
 				node->ImageIndex += 2;
 				node->SelectedIndex += 2;
 			}
+			if (SWVersion::currentVersion < SWVersion(softwareVersion.c_str())) {
+				node->ImageIndex = 5;
+				node->SelectedIndex = 5;
+			}
 			node->StateIndex = 0;
 		}
 	}
@@ -391,8 +351,7 @@ void TMainForm::fillSourceTree(const char *sourceConf, TTreeView *tree)
 }
 
 
-void __fastcall TMainForm::Button5Click(TObject *Sender)
-{
+void __fastcall TMainForm::Button5Click(TObject *Sender) {
 	TTreeView *tree;
 	for (int i = 0; i < PageControl1->ActivePage->ControlCount; i++) {
 		if (PageControl1->ActivePage->Controls[i]->ClassNameIs("TTreeView")) {
@@ -403,6 +362,22 @@ void __fastcall TMainForm::Button5Click(TObject *Sender)
 	TTreeNode *node = tree->Selected;
 	if (node) {
 		if (node->Parent) {
+			if (node->ImageIndex == 5) {
+				SWMgr *mgr = (tree == localTree) ? manager : ((InstallSourceTab *) PageControl1->ActivePage->Controls[0])->is->getMgr();
+				SWBuf minVer = (*mgr->config)[(const char *)node->Data]["MinimumVersion"];
+
+				InfoForm->Caption = "Need To Upgrade";
+				InfoForm->info = "\\qc {\\b \\fs20 Newer Software Version Required.}\\par\\par\\pard ";
+				InfoForm->info += "This module requires a newer version of the SWORD software engine. \\par\\par ";
+				InfoForm->info += "Your Current Version: ";
+				InfoForm->info += (const char *)SWVersion::currentVersion;
+				InfoForm->info += "\\par Required Version: ";
+				InfoForm->info += minVer.c_str();
+				InfoForm->info += "\\par\\par\\b\\qc Please visit http://www.crosswire.org to upgrade.";
+				InfoForm->ShowModal();
+				return;
+			}
+
 			if (node->StateIndex == 1)
 				node->StateIndex = 0; //node->StateIndex;
 			else	node->StateIndex = 1;
@@ -412,30 +387,30 @@ void __fastcall TMainForm::Button5Click(TObject *Sender)
 }
 //---------------------------------------------------------------------------
 
-void __fastcall TMainForm::localTreeDblClick(TObject *Sender)
-{
+void __fastcall TMainForm::localTreeDblClick(TObject *Sender) {
 	Button5Click(Sender);
 }
 //---------------------------------------------------------------------------
 
-int TMainForm::selectAll(TTreeView *tree, bool sel)
-{
+int TMainForm::selectAll(TTreeView *tree, bool sel) {
 	TTreeNode *node;
 	TTreeNode *node2;
 	int retCount = 0;
 	
 	for (node = tree->Items->GetFirstNode(); node; node = node->getNextSibling()) {
 		for (node2 = node->getFirstChild(); node2; node2 = node2->getNextSibling()) {
-			if (node2->StateIndex == 1) {
-				if (!sel) {
-					node2->StateIndex = 0; //node2->StateIndex;
-					retCount++;
+			if (node2->ImageIndex != 5) {
+				if (node2->StateIndex == 1) {
+					if (!sel) {
+						node2->StateIndex = 0; //node2->StateIndex;
+						retCount++;
+					}
 				}
-			}
-			else	{
-				if (sel) {
-					node2->StateIndex = 1;
-					retCount++;
+				else	{
+					if (sel) {
+						node2->StateIndex = 1;
+						retCount++;
+					}
 				}
 			}
 		}
@@ -444,8 +419,8 @@ int TMainForm::selectAll(TTreeView *tree, bool sel)
 }
 
 
-void __fastcall TMainForm::Button6Click(TObject *Sender)	// SELECT ALL
-{
+// SELECT ALL
+void __fastcall TMainForm::Button6Click(TObject *Sender) {
 	TTreeView *tree;
 	for (int i = 0; i < PageControl1->ActivePage->ControlCount; i++) {
 		if (PageControl1->ActivePage->Controls[i]->ClassNameIs("TTreeView")) {
@@ -459,359 +434,27 @@ void __fastcall TMainForm::Button6Click(TObject *Sender)	// SELECT ALL
 }
 //---------------------------------------------------------------------------
 
-void __fastcall TMainForm::Image1Click(TObject *Sender)
-{
+void __fastcall TMainForm::Image1Click(TObject *Sender) {
 	ShellExecute(this->Handle, "open", "http://www.crosswire.org", NULL, NULL, SW_SHOWNORMAL);
-
 }
 //---------------------------------------------------------------------------
 
-int TMainForm::removeModule(const char *modName)
-{
-	SectionMap::iterator module;
-	ConfigEntMap::iterator fileBegin;
-	ConfigEntMap::iterator fileEnd, entry;
 
-	module = manager->config->Sections.find(modName);
-
-	if (module != manager->config->Sections.end()) {
-		entry = module->second.find("CipherKey");
-		if (entry != module->second.end())
-			CipherForm->cipherEdit->Text = entry->second.c_str();
-			
-		fileBegin = module->second.lower_bound("File");
-		fileEnd = module->second.upper_bound("File");
-
-		if (fileBegin != fileEnd) {	// remove each file
-			while (fileBegin != fileEnd) {
-				//remove file
-				remove(fileBegin->second.c_str());
-				fileBegin++;
-			}
-		}
-		else {	//remove all files in DataPath directory
-
-			DIR *dir;
-			struct dirent *ent;
-			ConfigEntMap::iterator entry;
-			string modDir;
-			string modFile;
-
-			entry = module->second.find("DataPath");
-			if (entry != module->second.end()) {
-				modDir = entry->second.c_str();
-				entry = module->second.find("ModDrv");
-				if (entry != module->second.end()) {
-					if (!strcmp(entry->second.c_str(), "RawLD") || !strcmp(entry->second.c_str(), "RawLD4") || !strcmp(entry->second.c_str(), "zLD") || !strcmp(entry->second.c_str(), "RawGenBook") || !strcmp(entry->second.c_str(), "zGenBook")) {
-						char *buf = new char [ strlen(modDir.c_str()) + 1 ];
-	
-						strcpy(buf, modDir.c_str());
-						int end = strlen(buf) - 1;
-						while (end) {
-							if (buf[end] == '/')
-								break;
-							end--;
-						}
-						buf[end] = 0;
-						modDir = buf;
-						delete [] buf;
-					}
-				}
-
-				if (dir = opendir(modDir.c_str())) {
-					rewinddir(dir);
-					while ((ent = readdir(dir))) {
-						if ((strcmp(ent->d_name, ".")) && (strcmp(ent->d_name, ".."))) {
-							modFile = modDir;
-							modFile += "/";
-							modFile += ent->d_name;
-							remove(modFile.c_str());
-						}
-					}
-					closedir(dir);
-				}
-				if (dir = opendir(manager->configPath)) {	// find and remove .conf file
-					rewinddir(dir);
-					while ((ent = readdir(dir))) {
-						if ((strcmp(ent->d_name, ".")) && (strcmp(ent->d_name, ".."))) {
-							modFile = manager->configPath;
-							modFile += "/";
-							modFile += ent->d_name;
-							SWConfig *config = new SWConfig(modFile.c_str());
-							if (config->Sections.find(modName) != config->Sections.end()) {
-								delete config;
-								remove(modFile.c_str());
-							}
-							else	delete config;
-						}
-					}
-					closedir(dir);
-				}
-			}
-		}
-		return 0;
-	}
-	return 1;
-}
-
-
-int TMainForm::createParent(const char *pName)
-{
-	char *buf = new char [ strlen(pName) + 1 ];
-	int retCode = 0;
-	
-	strcpy(buf, pName);
-	int end = strlen(buf) - 1;
-	while (end) {
-		if (buf[end] == '/')
-			break;
-		end--;
-	}
-	buf[end] = 0;
-	if (strlen(buf)>0) {
-          if (access(buf, 02)) {  // not exists with write access?
-          	if ((retCode = mkdir(buf))) {
-		         	createParent(buf);
-                    retCode = mkdir(buf);
-               }
-		}
-     }
-     else retCode = -1;
-	delete [] buf;
-	return retCode;
-}
-	
-
-int TMainForm::createPathAndFile(const char *fName)
-{
-	int fd;
-	
-	fd = open(fName, O_CREAT|O_WRONLY|O_BINARY, S_IREAD|S_IWRITE);
-	if (fd < 1) {
-		createParent(fName);
-		fd = open(fName, O_CREAT|O_WRONLY|O_BINARY, S_IREAD|S_IWRITE);
-	}
-	return fd;
-}
-
-
-int TMainForm::copyFileToCWD(const char *sourceDir, const char *fName)
-{
-	string sourcePath = sourceDir;
-	sourcePath += fName;
-
-     string dest;
-     dest = manager->prefixPath;
-     if ((manager->prefixPath[strlen(manager->prefixPath)-1] != '\\') && ( manager->prefixPath[strlen(manager->prefixPath)-1] != '/'))
-        dest += "/";
-       dest += fName;
-
-	return copyFile(sourcePath.c_str(), dest.c_str());
-}
-
-
-int TMainForm::copyFile(const char *sourceFile, const char *targetFile)
-{
-	int sfd, dfd, len;
-	char buf[4096];
-
-	if ((sfd = open(sourceFile, O_RDONLY|O_BINARY)) < 1)
-		return -1;
-	if ((dfd = createPathAndFile(targetFile)) < 1)
-		return -1;
-
-	do {
-		len = read(sfd, buf, 4096);
-		write(dfd, buf, len);
-	}
-	while(len == 4096);	
-	close(dfd);
-	close(sfd);
-	
-	return 0;
-}
-
-
-int TMainForm::installModule(const char *modName, InstallSourceTab *ist)
-{
-	SectionMap::iterator module, section;
-	ConfigEntMap::iterator fileBegin;
-	ConfigEntMap::iterator fileEnd;
-	ConfigEntMap::iterator entry;
-	string sourceDir;
-	string buffer;
-	bool aborted = false;
-	bool cipher = false;
-
-	if (ist)
-		sourceDir = "./sources/" + ist->Source;
-	else	sourceDir = getLocalDir();
-	
-	SWMgr mgr(sourceDir.c_str());
-	
-	module = mgr.config->Sections.find(modName);
-
-	if (module != mgr.config->Sections.end()) {
-	
-		entry = module->second.find("CipherKey");
-		if (entry != module->second.end())
-			cipher = true;
-		
-		fileEnd = module->second.upper_bound("File");
-		fileBegin = module->second.lower_bound("File");
-
-		if (fileBegin != fileEnd) {	// copy each file
-			if (ist) {
-				while (fileBegin != fileEnd) {	// ftp each file first
-					buffer = sourceDir + "/" + fileBegin->second;
-					if (FTPCopy(ist, fileBegin->second.c_str(), buffer.c_str())) {
-						aborted = true;
-						break;	// user aborted
-					}
-					fileBegin++;
-				}
-				fileBegin = module->second.lower_bound("File");
-			}
-
-			if (!aborted) {
-				// DO THE INSTALL
-				while (fileBegin != fileEnd) {
-					copyFileToCWD(sourceDir.c_str(), fileBegin->second.c_str());
-					fileBegin++;
-				}
-			}
-			//---------------
-
-			if (ist) {
-				fileBegin = module->second.lower_bound("File");
-				while (fileBegin != fileEnd) {	// delete each tmp ftp file
-					buffer = sourceDir + "/" + fileBegin->second;
-					remove(buffer.c_str());
-					fileBegin++;
-				}
-			}
-		}
-		else {	//copy all files in DataPath directory
-			DIR *dir;
-			struct dirent *ent;
-			ConfigEntMap::iterator entry;
-			string modDir;
-			string modFile;
-			string sourceOrig = sourceDir;
-
-			entry = module->second.find("DataPath");
-			if (entry != module->second.end()) {
-				modDir = entry->second.c_str();
-				entry = module->second.find("ModDrv");
-				if (entry != module->second.end()) {
-					if (!strcmp(entry->second.c_str(), "RawLD") || !strcmp(entry->second.c_str(), "RawLD4") || !strcmp(entry->second.c_str(), "zLD") || !strcmp(entry->second.c_str(), "RawGenBook") || !strcmp(entry->second.c_str(), "zGenBook")) {
-						char *buf = new char [ strlen(modDir.c_str()) + 1 ];
-	
-						strcpy(buf, modDir.c_str());
-						int end = strlen(buf) - 1;
-						while (end) {
-							if (buf[end] == '/')
-								break;
-							end--;
-						}
-						buf[end] = 0;
-						modDir = buf;
-						delete [] buf;
-					}
-				}
-
-				if (ist) {
-					buffer = sourceDir + "/" + modDir;
-					if (FTPCopy(ist, modDir.c_str(), buffer.c_str(), true)) {
-						aborted = true;	// user aborted
-					}
-				}
-				sourceDir += "/";
-				sourceDir += modDir;
-				if (!aborted) {
-					if (dir = opendir(sourceDir.c_str())) {
-						rewinddir(dir);
-						while ((ent = readdir(dir))) {
-							if ((strcmp(ent->d_name, ".")) && (strcmp(ent->d_name, ".."))) {
-								modFile = modDir;
-								modFile += "/";
-								modFile += ent->d_name;
-								copyFileToCWD(sourceOrig.c_str(), modFile.c_str());
-							}
-						}
-						closedir(dir);
-					}
-				}
-				if (ist) {		// delete tmp ftp files
-					if (dir = opendir(sourceDir.c_str())) {
-						rewinddir(dir);
-						while ((ent = readdir(dir))) {
-							if ((strcmp(ent->d_name, ".")) && (strcmp(ent->d_name, ".."))) {
-								modFile = sourceOrig + "/" + modDir;
-								modFile += "/";
-								modFile += ent->d_name;
-								remove(modFile.c_str());
-							}
-						}
-						closedir(dir);
-					}
-				}
-				sourceDir = sourceOrig;
-				sourceDir += "/mods.d/";
-				if (!aborted) {
-					if (dir = opendir(sourceDir.c_str())) {	// find and copy .conf file
-						rewinddir(dir);
-						while ((ent = readdir(dir))) {
-							if ((strcmp(ent->d_name, ".")) && (strcmp(ent->d_name, ".."))) {
-								modFile = sourceDir;
-								modFile += ent->d_name;
-								SWConfig *config = new SWConfig(modFile.c_str());
-								if (config->Sections.find(modName) != config->Sections.end()) {
-									delete config;
-									string targetFile = manager->configPath; //"./mods.d/";
-                                             targetFile += "/";
-									targetFile += ent->d_name;
-									copyFile(modFile.c_str(), targetFile.c_str());
-									if (cipher) {
-										CipherForm->modName = modName;
-										CipherForm->confFile = targetFile;
-										if (CipherForm->ShowModal() == mrCancel) {
-											removeModule(modName);
-											aborted = true;
-										}
-									}
-								}
-								else	delete config;
-							}
-						}
-						closedir(dir);
-					}
-				}
-			}
-		}
-		return (aborted) ? -1 : 0;
-	}
-	return 1;
-}
-
-
-void	TMainForm::fillAllSourceTrees()
-{
-	fillSourceTree(getLocalDir(), localTree);
+void	TMainForm::fillAllSourceTrees() {
+	fillSourceTree(localTree);
 	for (int i = 1; i < PageControl1->PageCount; i++) {
 		InstallSourceTab *ist = (InstallSourceTab *) MainForm->PageControl1->Pages[i]->Controls[0];
-		string parent = "./sources/" + ist->Source;
-		fillSourceTree(parent.c_str(), ist->tree);
+		fillSourceTree(ist->tree, ist->is);
 	}
 }
 
 
-void __fastcall TMainForm::Button4Click(TObject *Sender)	// REMOVE MODULE
-{
+// REMOVE MODULE
+void __fastcall TMainForm::Button4Click(TObject *Sender) {
 	TTreeNode *node = installTree->Selected;
 	if (node) {
 		if (node->Parent) {
-			removeModule((const char *)node->Data);
+			installMgr->removeModule(manager, (const char *)node->Data);
 			delete manager;
 			manager = new SWMgr();
 			fillInstallTree();
@@ -821,8 +464,8 @@ void __fastcall TMainForm::Button4Click(TObject *Sender)	// REMOVE MODULE
 }
 //---------------------------------------------------------------------------
 
-void __fastcall TMainForm::Button2Click(TObject *Sender)	// INSTALL
-{
+// INSTALL
+void __fastcall TMainForm::Button2Click(TObject *Sender) {
 	TTreeNode *node;
 	int count = 0;
 	int abort = 0;
@@ -860,14 +503,14 @@ void __fastcall TMainForm::Button2Click(TObject *Sender)	// INSTALL
 		if (node->StateIndex == 1) {	// if selected for install
 			CipherForm->cipherEdit->Text = "";
 			if ((node->ImageIndex == 2) || (node->ImageIndex == 4)) {	// if this is an upgrade
-				removeModule((const char *)node->Data);
+				installMgr->removeModule(manager, (const char *)node->Data);
 			}
 				// install module
 			statusBar->Caption = "Installing: " + node->Text + "...";
 			statusBar->Repaint();
 			if (tree == localTree)
-				abort = installModule((const char *)node->Data);
-			else	abort = installModule((const char *)node->Data, (InstallSourceTab *) PageControl1->ActivePage->Controls[0]);
+				abort = installMgr->installModule(manager, getLocalDir(), (const char *)node->Data, 0);
+			else	abort = installMgr->installModule(manager, 0, (const char *)node->Data, ((InstallSourceTab *) PageControl1->ActivePage->Controls[0])->is);
 			if (abort)
 				break;
 			progressBar->Position++;
@@ -882,85 +525,34 @@ void __fastcall TMainForm::Button2Click(TObject *Sender)	// INSTALL
 }
 
 
-int TMainForm::FTPCopy(InstallSourceTab *ist, const char *src, const char *dest, bool dirTransfer, const char *suffix)
-{
-	StatusForm->ist = ist;
-	StatusForm->src = src;
-	StatusForm->dest = dest;
-        StatusForm->suffix = suffix;
-	StatusForm->dirTransfer = dirTransfer;
-        StatusForm->passive = passive;
-	Application->ProcessMessages();
-	if (StatusForm->ShowModal() == mrCancel)
-		return -1;
-	else	return 0;
-}
-
-
-void __fastcall TMainForm::RefreshRemoteSource(TObject *Sender)
-{
+void __fastcall TMainForm::RefreshRemoteSource(TObject *Sender) {
 	InstallSourceTab *ist = (InstallSourceTab *) MainForm->PageControl1->ActivePage->Controls[0];
-	DIR *dir;
-	struct dirent *ent;
-	ConfigEntMap::iterator entry;
-	string modDir;
-	string modFile;
-	string root = "./sources/";
-	root += ist->Source.c_str();
-	string target = root + "/mods.d";
-
-	if (dir = opendir(target.c_str())) {
-		rewinddir(dir);
-		while ((ent = readdir(dir))) {
-			if ((strcmp(ent->d_name, ".")) && (strcmp(ent->d_name, ".."))) {
-				modFile = target;
-				modFile += "/";
-				modFile += ent->d_name;
-				remove(modFile.c_str());
-			}
-		}
-		closedir(dir);
-	}
-
-
-    	string archive = root + "/mods.d.tar.gz";
-	if (!FTPCopy(ist, "mods.d.tar.gz", archive.c_str(), false)) {
-     	int fd = open(archive.c_str(), O_RDONLY|O_BINARY);
-     	untargz(fd, root.c_str());
-          close(fd);
-     }
-     else	FTPCopy(ist, "mods.d", target.c_str(), true, ".conf");
-
-	target = "./sources/";
-	target += ist->Source.c_str();
-
-	fillSourceTree(target.c_str(), ist->tree);	
+	installMgr->refreshRemoteSource(ist->is);
+	fillSourceTree(ist->tree, ist->is);	
 }
 
 
 
-void __fastcall TMainForm::SpeedButton1Click(TObject *Sender)
-{
+void __fastcall TMainForm::SpeedButton1Click(TObject *Sender) {
 	InfoForm->Caption = "W A R N I N G";
 	InfoForm->info = "\\qc {\\b \\fs20 -=+* WARNING *+=- -=+* WARNING *+=-}\\par\\par\\pard ";
 	InfoForm->info += "Although Install Manager provides a convenient way for installing and upgrading SWORD components, it also uses a systematic method for accessing sites which gives packet sniffers a target to lock into for singling out users. \\par\\par ";
 	InfoForm->info += "\\b\\qc IF YOU LIVE IN A PERSECUTED COUNTRY AND DO NOT WISH TO RISK DETECTION, YOU SHOULD *NOT* USE INSTALL MANAGER'S REMOTE SOURCE FEATURES.";
 	InfoForm->ShowModal();
 	if (RemoteMntForm->ShowModal() == mrOk) {
-		delete installConf;
-		installConf = new SWConfig("./InstallMgr.conf");
-	
+		delete installMgr;
+		installMgr = new InstallMgrWin("./installMgr");
 		refreshPageControl();
 	}
 }
 
-void __fastcall TMainForm::SpeedButton5Click(TObject *Sender)
-{
+
+void __fastcall TMainForm::SpeedButton5Click(TObject *Sender) {
 	TTreeView *tree;
 	InstallSourceTab *ist = 0;
 	SectionMap::iterator module;
 	ConfigEntMap::iterator entry, entryEnd;
-	string sourceDir;
+	SWBuf sourceDir;
 
 	if (PageControl1->ActivePage != PageControl1->Pages[0]) {
 		ist = (InstallSourceTab *) PageControl1->ActivePage->Controls[0];
@@ -972,15 +564,15 @@ void __fastcall TMainForm::SpeedButton5Click(TObject *Sender)
 	if (node) {
 		if (node->Parent) {
 			if (ist)
-				sourceDir = "./sources/" + ist->Source;
+				sourceDir = (SWBuf)"./installMgr/" + ist->is->source;
 			else	sourceDir = getLocalDir();
 			SWMgr *mgr = new SWMgr(sourceDir.c_str());
 	
 			module = mgr->config->Sections.find((const char *)node->Data);
 
 			if (module != mgr->config->Sections.end()) {
-				string targetVersion = "0.0";
-				string sourceVersion = "1.0";
+				SWBuf targetVersion = "0.0";
+				SWBuf sourceVersion = "1.0";
 				SectionMap::iterator targetSection;
 					
 				entry = module->second.find("Version");
@@ -994,15 +586,15 @@ void __fastcall TMainForm::SpeedButton5Click(TObject *Sender)
 					if (entry != targetSection->second.end())
 						targetVersion = entry->second;
 				}
-				if (VersionInfo(targetVersion.c_str()) < VersionInfo("1.0")) {
+				if (SWVersion(targetVersion.c_str()) < SWVersion("1.0")) {
 					InfoForm->info = "\\pard{\\b * Additional Module Available for Install. } \\par ";
 				}
 				else {
-					InfoForm->info = "\\pard{\\b + Upgraded Module Available for Install.}\\par\\tab Current Version:  \\tab " + targetVersion + " \\par\\tab Upgrade Version:\\tab " + sourceVersion + " \\par ";
+					InfoForm->info = (SWBuf)"\\pard{\\b + Upgraded Module Available for Install.}\\par\\tab Current Version:  \\tab " + targetVersion + " \\par\\tab Upgrade Version:\\tab " + sourceVersion + " \\par ";
 					bool changes = false;
 					for (entry = module->second.begin(); entry != module->second.end(); entry++) {
 						if (!strncmp(entry->first.c_str(), "History_", 8)) {
-							if (VersionInfo(&entry->first.c_str()[8]) > VersionInfo(targetVersion.c_str())) {
+							if (SWVersion(&entry->first.c_str()[8]) > SWVersion(targetVersion.c_str())) {
 								if (!changes) {
 									changes = true;
 									InfoForm->info += "\\par{\\b Changes: }\\par ";
@@ -1029,8 +621,7 @@ void __fastcall TMainForm::SpeedButton5Click(TObject *Sender)
 }
 //---------------------------------------------------------------------------
 
-void __fastcall TMainForm::Exit1Click(TObject *Sender)
-{
+void __fastcall TMainForm::Exit1Click(TObject *Sender) {
 	Close();	
 }
 //---------------------------------------------------------------------------
@@ -1052,13 +643,13 @@ void TMainForm::deleteAllModules() {
 
 	ModMap::iterator it;
 	for (it = mgr->Modules.begin(); it != mgr->Modules.end(); it++) {
-		string label = "Uninstalling: [";
+		SWBuf label = "Uninstalling: [";
 		label += it->second->Name();
 		label += "] ";
 		label += it->second->Description();
 		UninstallForm->Label1->Caption = label.c_str();
 		UninstallForm->Label1->Repaint();
-		removeModule(it->second->Name());
+		installMgr->removeModule(mgr, it->second->Name());
 		UninstallForm->ProgressBar1->Position = UninstallForm->ProgressBar1->Position + 1;
 		UninstallForm->ProgressBar1->Repaint();
 	}
@@ -1066,8 +657,9 @@ void TMainForm::deleteAllModules() {
 	UninstallForm->Hide();
 	return;
 }
-void __fastcall TMainForm::FormShow(TObject *Sender)
-{
+
+
+void __fastcall TMainForm::FormShow(TObject *Sender) {
 	for (int i=0;i<=ParamCount();i++) {
 		if (LowerCase(ParamStr(i)) == "-uninstall") {
 			deleteAllModules();
@@ -1094,16 +686,15 @@ void __fastcall TMainForm::LockedModules1Click(TObject *Sender) {
 }
 //---------------------------------------------------------------------------
 
-void __fastcall TMainForm::Contents1Click(TObject *Sender)
-{
+void __fastcall TMainForm::Contents1Click(TObject *Sender) {
 	SWConfig optionsconf("options.conf");
-	string helpDir;
+	SWBuf helpDir;
 	ConfigEntMap::iterator it = optionsconf.Sections["Help"].find("Directory");
 	if (it != optionsconf.Sections["Help"].end())
 		helpDir = (*it).second;
 	else helpDir = ".\\help";
 
-	string helpExe = helpDir + "\\Sword.chm::modulesh.html#instmgr";
+	SWBuf helpExe = helpDir + "\\Sword.chm::modulesh.html#instmgr";
 	ShellExecute(this->Handle, "open", "hh", helpExe.c_str(), NULL, SW_SHOWNORMAL);
 }
 //---------------------------------------------------------------------------
