@@ -17,18 +17,7 @@
 //#include "IdTCPConnection.hpp"
 //---------------------------------------------------------------------------
 
-class TFTPCon {
-public:
-	string Host;
-	string Username;
-	string Password;
-	bool Passive;
-	string cwd;
-	
-};
-
-class TStatusForm : public TForm
-{
+class TStatusForm : public TForm {
 __published:	// IDE-managed Components
 	TPanel *Panel1;
 	TPanel *Panel2;
@@ -48,40 +37,35 @@ __published:	// IDE-managed Components
 //		const int AWorkCount);
 private:	// User declarations
 	class TFTPThread : public TThread {
-		bool abort;
-		InstallSourceTab *ist;
-		string src;
-		string dest;
-		string buffer;
-		string buffer2;
-		string suffix;
+		InstallSource *is;
+		SWBuf src;
+		SWBuf dest;
+		SWBuf suffix;
 		bool dirTransfer;
-		bool passive;
 	protected:
 		void __fastcall Execute();
 	public:
 //		TNMFTP *FTPLink;
-		__fastcall TFTPThread(InstallSourceTab *iist, const char *isrc, const char *idest, bool idirTransfer = false, bool CreateSuspended = false, bool ipassive = false, const char *suffix = "");
+		__fastcall TFTPThread(InstallSource *iis, const char *isrc, const char *idest, bool idirTransfer = false, bool CreateSuspended = false, const char *suffix = "");
 		__fastcall TFTPThread::~TFTPThread();
 		void __fastcall FTPLinkPacketRecvd(TObject *Sender);
 		void __fastcall CreateFTPObject(void);
 		void __fastcall PreConnect(void);
-		void __fastcall PreDownload1(void);
-		void __fastcall PreDownload2(void);
 //		void __fastcall Cleanup(void);
 	} *ftpThread;
 public:		// User declarations
-	TFTPCon *ftpCon;
+	void __fastcall PreDownload1(void);
+	void __fastcall PreDownload2(void);
 	long completedBytes;
 	long totalBytes;
-	InstallSourceTab *ist;
-	string src;
-	string dest;
-	string suffix;
+	InstallSource *is;
+	SWBuf src;
+	SWBuf dest;
+	SWBuf suffix;
+	SWBuf buffer;
+	SWBuf buffer2;
 	bool dirTransfer;
-	bool passive;
-	bool abort;
-		void __fastcall UpdateBytes();
+	void __fastcall UpdateBytes();
 	__fastcall TStatusForm(TComponent* Owner);
 	__fastcall ~TStatusForm();
 };
