@@ -23,17 +23,14 @@ void __fastcall TCipherForm::Button2Click(TObject *Sender)
 {
 	SectionMap::iterator section;
 	ConfigEntMap::iterator entry;
-	SWConfig *modconf = new SWConfig(confFile.c_str());
-	string tmpBuf;
+	SWBuf tmpBuf;
 	
-	section = modconf->Sections.find(modName);
-	if (section != modconf->Sections.end()) {
+	section = config->Sections.find(modName);
+	if (section != config->Sections.end()) {
 		entry = section->second.find("CipherKey");
 		if (entry != section->second.end()) {
 			entry->second = CipherForm->cipherEdit->Text.c_str();
-			modconf->Save();
-			delete modconf;	// close file
-			modconf = 0;
+			config->Save();
 			SWMgr *mgr = new SWMgr();
 			SWModule *mod = mgr->Modules[modName];
 			mod->setKey("Ipet 2:12");
@@ -45,15 +42,12 @@ void __fastcall TCipherForm::Button2Click(TObject *Sender)
 			delete mgr;
 		}
 	}
-	if (modconf)
-		delete modconf;
-	
 }
 //---------------------------------------------------------------------------
 
 void __fastcall TCipherForm::FormShow(TObject *Sender)
 {
-	string tmpCaption;
+	SWBuf tmpCaption;
 	tmpCaption = "Cipher Key: [";
 	tmpCaption += modName;
 	tmpCaption += "]";
