@@ -7,7 +7,10 @@
 #include <StdCtrls.hpp>
 #include <Forms.hpp>
 #include <ExtCtrls.hpp>
-#include <string>
+#include <swbuf.h>
+#include <swconfig.h>
+
+using namespace sword;
 //---------------------------------------------------------------------------
 class TCipherForm : public TForm
 {
@@ -27,8 +30,8 @@ __published:	// IDE-managed Components
 	void __fastcall FormClose(TObject *Sender, TCloseAction &Action);
 private:	// User declarations
 public:		// User declarations
-	std::string confFile;
-	std::string modName;
+	SWConfig *config;
+	SWBuf modName;
 	__fastcall TCipherForm(TComponent* Owner);
 };
 //---------------------------------------------------------------------------
