@@ -112,7 +112,7 @@ char TForm1::CreateTextPane(SWModule *mod, char *font) {
 	}
 
 	mod->Disp(*displays.insert(displays.begin(), new RTFDisp(newrtf)));
-	mod->SetKey(DefaultVSKey);
+	mod->setKey(DefaultVSKey);
 */
 	return 0;
 }
@@ -157,7 +157,7 @@ char TForm1::CreateCommentPane(SWModule *mod) {
 	newrtf->Parent = newtab;
 	newrtf->Align = alClient;
 
-	mod->SetKey(DefaultVSKey);
+	mod->setKey(DefaultVSKey);
 	return 0;
 }
 //---------------------------------------------------------------------------
@@ -176,7 +176,7 @@ char TForm1::CreateLDPane(SWModule *mod) {
 	newrtf->OnMouseDown = RTFMouseDown;
 
 	mod->Disp(*displays.insert(displays.begin(), new RTFDisp(newrtf)));
-	mod->SetKey(DefaultStrKey);
+	mod->setKey(DefaultStrKey);
 	return 0;
 }
 //---------------------------------------------------------------------------
@@ -398,7 +398,7 @@ void __fastcall TForm1::LookupPokeData(TObject *Sender)
 	token = strtok(buf, " ");
 	if ((it = mainmgr->Modules.find(token)) != mainmgr->Modules.end()) {
 		token = strtok(NULL, "");
-		(*it).second->SetKey(token);
+		it->second->setKey(token);
 		Lookup->Text = (const char *)(*(*it).second);
 	}
 	else Lookup->Text = "";
