@@ -7,7 +7,10 @@
 #include <conio.h>
 #include <stdarg.h>
 //---------------------------------------------------------------------------
- 
+
+
+SWORD_NAMESPACE_START
+
 SWWinLog::SWWinLog(HWND iparent) : SWLog() {
 	parent = iparent;
 }
@@ -72,3 +75,5 @@ void SWWinLog::LogInformation(char *fmt, ...)
 		MessageBox(NULL, msg, "Information", MB_OK);
 	}
 }
+
+SWORD_NAMESPACE_END
