@@ -9,6 +9,7 @@
 #include <windows.h>
 #include <string.h>
 
+SWORD_NAMESPACE_START
 
 /******************************************************************************
  * TBDisp Constructor - sets up an instance of TBDisp for use
@@ -53,3 +54,5 @@ char TBDisp::Display(SWModule &imodule)
      delete [] buf;
 	return 0;
 }
+
+SWORD_NAMESPACE_END
