@@ -1,5 +1,6 @@
-/*
+/***************************************************************************
  *
+ * $Id: latin1utf16.h,v 1.5 2003/02/20 07:25:19 scribe Exp $
  *
  * Copyright 1998 CrossWire Bible Society (http://www.crosswire.org)
  *	CrossWire Bible Society
@@ -22,16 +23,14 @@
 
 #include <swfilter.h>
 
-#include <defs.h>
-
 SWORD_NAMESPACE_START
 
-  /** This filter converts Latin-1 encoded text to UTF-8
-  */
-class SWDLLEXPORT Latin1UTF16:public SWFilter {
+/** This filter converts Latin-1 encoded text to UTF-16
+ */
+class SWDLLEXPORT Latin1UTF16 : public SWFilter {
 public:
-  Latin1UTF16();
-  virtual char ProcessText (char *text, int maxlen, const SWKey *key, const SWModule * = 0);
+	Latin1UTF16();
+	virtual char processText(SWBuf &text, const SWKey *key = 0, const SWModule *module = 0);
 };
 
 SWORD_NAMESPACE_END
