@@ -3,7 +3,7 @@
 *		  types of modules (e.g. texts, commentaries, maps, lexicons,
 *		  etc.)
 *
-* $Id: swmodule.h,v 1.52 2002/09/25 22:14:58 scribe Exp $
+* $Id: swmodule.h,v 1.67 2003/07/05 04:58:42 scribe Exp $
 *
 * Copyright 1998 CrossWire Bible Society (http://www.crosswire.org)
 *	CrossWire Bible Society
@@ -31,12 +31,12 @@
 #include <swconfig.h>
 #include <swcacher.h>
 #include <list>
+#include <swbuf.h>
 
 #include <defs.h>
 #include <multimapwdef.h>
 
-using namespace std;
-
+SWORD_NAMESPACE_START
 
 #define SWMODULE_OPERATORS \
 	operator const char *() { return RenderText(); } \
@@ -51,18 +51,17 @@ using namespace std;
 	SWModule &operator =(SW_POSITION p) { setPosition(p); return *this; }
 
 
-typedef list < SWFilter * >FilterList;
-
-typedef map < string, string, less < string > > AttributeValue;
-typedef map < string, AttributeValue, less < string > > AttributeList;
-typedef map < string, AttributeList, less < string > > AttributeTypeList;
+typedef std::list < SWFilter * >FilterList;
+typedef std::map < SWBuf, SWBuf, std::less < SWBuf > > AttributeValue;
+typedef std::map < SWBuf, AttributeValue, std::less < SWBuf > > AttributeList;
+typedef std::map < SWBuf, AttributeList, std::less < SWBuf > > AttributeTypeList;
 
 #define SWTextDirection char
 #define SWTextEncoding char
 #define SWTextMarkup char
 
 enum {DIRECTION_LTR = 0, DIRECTION_RTL, DIRECTION_BIDI};
-enum {FMT_UNKNOWN = 0, FMT_PLAIN, FMT_THML, FMT_GBF, FMT_HTML, FMT_HTMLHREF, FMT_RTF, FMT_OSIS};
+enum {FMT_UNKNOWN = 0, FMT_PLAIN, FMT_THML, FMT_GBF, FMT_HTML, FMT_HTMLHREF, FMT_RTF, FMT_OSIS, FMT_WEBIF};
 enum {ENC_UNKNOWN = 0, ENC_LATIN1, ENC_UTF8, ENC_SCSU, ENC_UTF16, ENC_RTF, ENC_HTML};
 
 /**
@@ -73,6 +72,16 @@ enum {ENC_UNKNOWN = 0, ENC_LATIN1, ENC_UTF8, ENC_SCSU, ENC_UTF16, ENC_RTF, ENC_H
  * SWModule has also functions to write to the data files.
  */
 
+// TODO: should all SWModule decendents be SWCachers?  Only some really
+// cache data.  But if we don't do this, then we need another mechanism to
+// check if we are an SWCacher.  Maybe make SWModule extend SWObject (which
+// it probably should anyway.  But then we need to add all the cheezy
+// heirarchy info to all he decendent classes for our SWDYNAMIC_CAST and
+// then we can see if we implement SWCacher so we know whether or not add
+// to the yet to be developed cachemgr.
+// Just leave for now.  This lets us always able to call module->flush()
+// to manually flush a cache, and doesn't hurt if there is no work done.
+
 class SWDLLEXPORT SWModule : public SWCacher {
 
 protected:
@@ -102,8 +111,7 @@ protected:
 	SWDisplay *disp;
 
 	static SWDisplay rawdisp;
-	char *entrybuf;
-	unsigned long entrybufallocsize;
+	SWBuf entryBuf;
 
 	/** executed to remove all markup (for searches) */
 	FilterList *stripFilters;
@@ -121,31 +129,31 @@ protected:
 	FilterList *encodingFilters;
 
 	int entrySize;
+	mutable long entryIndex;	 // internal common storage for index
 
 	public:
 	/**
 	* This is the default callback function for searching.
 	* This function is a placeholder and does nothing.
 	* You can define your own function for search progress
-	* evaluation, and pass it over to @ref Search.
+	* evaluation, and pass it over to Search().
 	*/
 	static void nullPercent(char percent, void *userData);
 	/**
-	* Set this bool to false to terminate the search which is executed by this module (@ref #Search).
+	* Set this bool to false to terminate the search which is executed by this module (Search()).
 	* This is useful for threaded applications to terminate the search in another thread.
 	*/
 	bool terminateSearch;
 	/** Initializes data for instance of SWModule
 	*
-	* @param imodname Internal name for module; see also @ref Name
-	* @param imoddesc Name to display to user for module;
-	*  see also @ref Description
-	* @param idisp Display object to use for displaying
-	*  see also @ref Disp
-	* @param imodtype Type of Module
-	*  (All modules will be displayed with
-	*  others of same type under their modtype heading)
-	*  see also @ref Type
+	* @param imodname Internal name for module; see also Name().
+	* @param imoddesc Name to display to user for module; see also Description()
+	* @param idisp Display object to use for displaying; see also Disp()
+	* @param imodtype Type of Module (All modules will be displayed with others of same type under their modtype heading); see also Type()
+	* @param encoding Encoding of the module.
+	* @param dir Direction of text flow.
+	* @param markup Markup of the module.
+	* @param modlang Language of the module.
 	*/
 	SWModule(const char *imodname = 0, const char *imoddesc = 0, SWDisplay * idisp = 0, char *imodtype = 0, SWTextEncoding encoding = ENC_UNKNOWN, SWTextDirection dir = DIRECTION_LTR, SWTextMarkup markup = FMT_UNKNOWN, const char* modlang = 0);
 	/** Cleans up instance of SWModule
@@ -178,25 +186,43 @@ protected:
 	*
 	* @param ikey key with which to set this module
 	* @return error status
-	* @deprecated See setKey()
+	* @deprecated Use setKey() instead.
 	*/
 	char SetKey(const SWKey *ikey) { return setKey(ikey); }
-	virtual char setKey(const SWKey *ikey);
-	
 	/**
-	* Sets the key of this module. Similar to @see SetKey(const SWKey*) .
+	* Sets a key to this module for position to a
+	* particular record or set of records
+	*
+	* @param ikey key with which to set this module
+	* @return error status
+	*/
+	virtual char setKey(const SWKey *ikey);
+
+
+	virtual long Index() const { return entryIndex; }
+	virtual long Index(long iindex) { entryIndex = iindex; return entryIndex; }
+
+	/**
+	* Sets the key of this module.
 	* @param ikey The SWKey which should be used as new key.
 	* @return Error status
-	* @deprecated See setKey()
+	* @deprecated Use setKey() instead.
 	*/
 	char SetKey(const SWKey &ikey) { return setKey(ikey); }
+	/**
+	* Sets the key of this module.
+	* @param ikey The SWKey which should be used as new key.
+	* @return Error status
+	*/
 	char setKey(const SWKey &ikey) { return SetKey(&ikey); }
 
 	/** Gets the current module key
 	* @return the current key of this module
-	* @deprecated See getKey()
 	*/
 	SWKey &Key() const { return *getKey(); }
+	/** Gets the current module key
+	* @return the current key of this module
+	*/
 	SWKey *getKey() const;
 
 	/** Sets the current key of the module to ikey, and returns
@@ -204,23 +230,20 @@ protected:
 	*
 	* @param ikey new current key for the module
 	* @return the keytext of the current module key
-	* @deprecated See setKey()
+	* @deprecated Use setKey() instead.
 	*/
 	char Key(const SWKey & ikey) { return setKey(ikey); }
 
-	/******************************************************************************
-	 * SWModule::KeyText - Sets/gets module KeyText
+	/**
+	 * Sets/gets module KeyText
 	 *
-	 * ENT:	ikeytext - value which to set keytext
-	 *		[0] - only get
-	 *
-	 * RET:	pointer to keytext
+	 * @param ikeytext Value which to set keytext; [0]-only get
+	 * @return pointer to keytext
 	 */
 	virtual const char *KeyText(const char *ikeytext = 0) {
 		if (ikeytext) setKey(ikeytext);
 		return *getKey();
 	}
-
 	/** Calls this modules display object and passes itself
 	*
 	* @return error status
@@ -229,57 +252,49 @@ protected:
 
 	/** Sets/gets display driver
 	*
-	* @param idisp value which to set disp;
-	*  [0] - only get
+	* @param idisp Value which to set disp; [0]-only get
 	* @return pointer to disp
 	*/
 	virtual SWDisplay *Disp(SWDisplay * idisp = 0);
 	/** Sets/gets module name
 	*
-	* @param imodname value which to set modname;
-	*  [0] - only get
+	* @param imodname Value which to set modname; [0]-only get
 	* @return pointer to modname
 	*/
 	virtual char *Name(const char *imodname = 0);
 	/** Sets/gets module description
 	*
-	* @param imoddesc value which to set moddesc;
-	*  [0] - only get
+	* @param imoddesc Value which to set moddesc; [0]-only get
 	* @return pointer to moddesc
 	*/
 	virtual char *Description(const char *imoddesc = 0);
 	/** Sets/gets module type
 	*
-	* @param imodtype value which to set modtype;
-	*  [0] - only get
+	* @param imodtype Value which to set modtype; [0]-only get
 	* @return pointer to modtype
 	*/
 	virtual char *Type(const char *imodtype = 0);
 	/** Sets/gets module direction
 	*
-	* @param newdir value which to set direction;
-	*  [-1] - only get
-	* @return char direction
+	* @param newdir Value which to set direction; [-1]-only get
+	* @return new direction
 	*/
 	virtual char Direction(signed char newdir = -1);
 	/** Sets/gets module encoding
 	*
-	* @param newdir value which to set encoding;
-	*  [-1] - only get
-	* @return char encoding
+	* @param enc Value which to set encoding; [-1]-only get
+	* @return Encoding
 	*/
 	virtual char Encoding(signed char enc = -1);
 	/** Sets/gets module markup
 	*
-	* @param newdir value which to set markup;
-	*  [-1] - only get
-	* @return char markup
+	* @param markup Vvalue which to set markup; [-1]-only get
+	* @return Markup
 	*/
-	virtual char Markup(signed char enc = -1);
+	virtual char Markup(signed char markup = -1);
 	/** Sets/gets module language
 	*
-	* @param imodlang value which to set modlang;
-	*  [0] - only get
+	* @param imodlang Value which to set modlang; [0]-only get
 	* @return pointer to modlang
 	*/
 	virtual char *Lang(const char *imodlang = 0);
@@ -290,10 +305,13 @@ protected:
 	*
 	* @param istr string for which to search
 	* @param searchType type of search to perform
-	*   <=0 ->regex;  1->phrase; 2->multiword;
+	*   >=0 ->regex;  -1 ->phrase; -2 ->multiword;
 	* @param flags options flags for search
+	* @param scope Key containing the scope. VerseKey or ListKey are useful here.
 	* @param justCheckIfSupported if set, don't search,
 	* only tell if this function supports requested search.
+	* @param percent Callback function to get the current search status in %.
+	* @param percentUserData User data that is given to the callback function as parameter.
 	*
 	* @return listkey set to verses that contain istr
 	*/
@@ -306,7 +324,7 @@ protected:
 	*
 	*/
 	virtual signed char createSearchFramework() { return 0; }				// special search framework
-	/**
+	/** Not yet useful.
 	*
 	*/
 	virtual bool hasSearchFramework() { return false; }				// special search framework
@@ -332,11 +350,12 @@ protected:
 	* @return the raw module text of the current entry
 	*/
 #ifndef SWIG 
-	virtual char *getRawEntry() = 0;
+	virtual SWBuf &getRawEntryBuf() = 0;
 #else
-	virtual char *getRawEntry() {};
+	virtual SWBuf &getRawEntryBuf() {};
 #endif  
 
+	virtual const char *getRawEntry() { return getRawEntryBuf().c_str(); }
 
 	// write interface ----------------------------
 	/** Is the module writable? :)
@@ -347,18 +366,15 @@ protected:
 	* @param path The first parameter is path of the new module
 	* @return error
 	*/
-	static signed char createModule(const char *) { return -1; }
-	/** Modify the current module entry text
-	* - only if module @ref isWritable
+	static signed char createModule(const char *path) { return -1; }
+	/** Modify the current module entry text - only if module isWritable()
 	* @return *this
 	*/
 	virtual void setEntry(const char *inbuf, long len = -1) { }
-	/** Link the current module entry to another module entry
-	* - only if module @ref isWritable
+	/** Link the current module entry to another module entry - only if module isWritable()
 	*/
 	virtual void linkEntry(const SWKey *sourceKey) { }
-	/** Delete current module entry - only if module @ref isWritable
-	*
+	/** Delete current module entry - only if module isWritable()
 	*/
 	virtual void deleteEntry() {}
 
@@ -366,23 +382,23 @@ protected:
 
 	/** Decrements module key a number of entries
 	*
-	* @param decrement Number of entries to jump backward
+	* @param steps Number of entries to jump backward
 	* @return *this
 	*/
 	virtual void decrement(int steps = 1);
 	/** Increments module key a number of entries
 	*
-	* @param increment Number of entries to jump forward
+	* @param steps Number of entries to jump forward
 	* @return *this
 	*/
 	virtual void increment(int steps = 1);
 	/** Positions this modules to an entry
 	*
-	* @param p position (e.g. TOP, BOTTOM)
+	* @param pos position (e.g. TOP, BOTTOM)
 	* @return *this
 	*/
-	virtual void setPosition(SW_POSITION p);
-	/** Adds a RenderFilter to this module's @see renderfilters queue
+	virtual void setPosition(SW_POSITION pos);
+	/** Adds a RenderFilter to this module's renderfilters queue
 	* @param newfilter the filter to add
 	* @return *this
 	*/
@@ -390,7 +406,7 @@ protected:
 		renderFilters->push_back (newfilter);
 		return *this;
 	}
-	/** Removes a RenderFilter from this module's @see renderfilters queue
+	/** Removes a RenderFilter from this module's renderfilters queue
 	* @param oldfilter the filter to remove
 	* @return *this
 	*/
@@ -398,7 +414,7 @@ protected:
 		renderFilters->remove (oldfilter);
 		return *this;
 	}
-	/** Replaces a RenderFilter in this module's @see renderfilters queue
+	/** Replaces a RenderFilter in this module's renderfilters queue
 	* @param oldfilter the filter to remove
 	* @param newfilter the filter to add in its place
 	* @return *this
@@ -413,12 +429,11 @@ protected:
 	}
 	/** RenderFilter a text buffer
 	* @param buf the buffer to filter
-	* @param size the allocated size of the buffer
 	* @param key key location from where this buffer was extracted
 	* @return *this
 	*/
-	virtual void renderFilter(char *buf, long size, SWKey *key) {
-		filterBuffer(renderFilters, buf, size, key);
+	virtual void renderFilter(SWBuf &buf, SWKey *key) {
+		filterBuffer(renderFilters, buf, key);
 	}
 	/** Adds an EncodingFilter to this module's @see encodingfilters queue
 	* @param newfilter the filter to add
@@ -428,7 +443,7 @@ protected:
 		encodingFilters->push_back (newfilter);
 		return *this;
 	}
-	/** Removes an EncodingFilter from this module's @see encodingfilters queue
+	/** Removes an EncodingFilter from this module's encodingfilters queue
 	* @param oldfilter the filter to remove
 	* @return *this
 	*/
@@ -436,7 +451,7 @@ protected:
 		encodingFilters->remove (oldfilter);
 		return *this;
 	}
-	/** Replaces an EncodingFilter in this module's @see encodingfilters queue
+	/** Replaces an EncodingFilter in this module's encodingfilters queue
 	* @param oldfilter the filter to remove
 	* @param newfilter the filter to add in its place
 	* @return *this
@@ -451,14 +466,13 @@ protected:
 	}
 	/** encodingFilter a text buffer
 	* @param buf the buffer to filter
-	* @param size the allocated size of the buffer
 	* @param key key location from where this buffer was extracted
 	* @return *this
 	*/
-	virtual void encodingFilter(char *buf, long size, SWKey *key) {
-		filterBuffer(encodingFilters, buf, size, key);
+	virtual void encodingFilter(SWBuf &buf, SWKey *key) {
+		filterBuffer(encodingFilters, buf, key);
 	}
-	/** Adds a StripFilter to this module's @ref stripfilters queue
+	/** Adds a StripFilter to this module's stripfilters queue
 	* @param newfilter the filter to add
 	* @return *this
 	*/
@@ -468,14 +482,13 @@ protected:
 	}
 	/** StripFilter a text buffer
 	* @param buf the buffer to filter
-	* @param size the allocated size of the buffer
 	* @param key key location from where this buffer was extracted
 	* @return *this
 	*/
-	virtual void stripFilter(char *buf, long size, SWKey *key) {
-		filterBuffer(stripFilters, buf, size, key);
+	virtual void stripFilter(SWBuf &buf, SWKey *key) {
+		filterBuffer(stripFilters, buf, key);
 	}
-	/** Adds a RawFilter to this module's @ref rawfilters queue
+	/** Adds a RawFilter to this module's rawfilters queue
 	* @param newfilter the filter to add
 	* @return *this
 	*/
@@ -486,26 +499,24 @@ protected:
 	/** FilterBuffer a text buffer
 	* @param filters the FilterList of filters to iterate
 	* @param buf the buffer to filter
-	* @param size the allocated size of the buffer
 	* @param key key location from where this buffer was extracted
 	* @return *this
 	*/
-	virtual void filterBuffer(FilterList *filters, char *buf, long size, SWKey *key) {
+	virtual void filterBuffer(FilterList *filters, SWBuf &buf, SWKey *key) {
 		FilterList::iterator it;
 		for (it = filters->begin(); it != filters->end(); it++) {
-			(*it)->ProcessText(buf, size, key, this);
+			(*it)->processText(buf, key, this);
 		}
 	}
 	/** RawFilter a text buffer
 	* @param buf the buffer to filter
-	* @param size the allocated size of the buffer
 	* @param key key location from where this buffer was extracted
 	* @return *this
 	*/
-	virtual void rawFilter(char *buf, long size, SWKey *key) {
-		filterBuffer(rawFilters, buf, size, key);
+	virtual void rawFilter(SWBuf &buf, SWKey *key) {
+		filterBuffer(rawFilters, buf, key);
 	}
-	/** Adds an OptionFilter to this module's @ref optionfilters queue
+	/** Adds an OptionFilter to this module's optionfilters queue
 	* @param newfilter the filter to add
 	* @return *this
 	*/
@@ -515,12 +526,11 @@ protected:
 	}
 	/** OptionFilter a text buffer
 	* @param buf the buffer to filter
-	* @param size the allocated size of the buffer
 	* @param key key location from where this buffer was extracted
 	* @return *this
 	*/
-	virtual void optionFilter(char *buf, long size, SWKey *key) {
-		filterBuffer(optionFilters, buf, size, key);
+	virtual void optionFilter(SWBuf &buf, SWKey *key) {
+		filterBuffer(optionFilters, buf, key);
 	}
 	/** calls all StripFilters on buffer or current text
 	*
@@ -529,15 +539,16 @@ protected:
 	* @param len max len of buf OR current text -- will be applied anyway
 	* @return this module's text at specified key location massaged by Strip filters
 	*/
-	virtual const char *StripText(char *buf = 0, int len = -1);
+	virtual const char *StripText(const char *buf = 0, int len = -1);
 	/** calls all RenderFilters on buffer or current text
 	*
 	* @param buf buffer to Render instead of current module position;
 	*   if buf is NULL, the current text will be used
 	* @param len max len of buf OR current text -- will be applied anyway
+	* @param render Perform the rendering.
 	* @return this module's text at specified key location massaged by Render filters
 	*/
-	virtual const char *RenderText(char *buf = 0, int len = -1, bool render = true);
+	virtual const char *RenderText(const char *buf = 0, int len = -1, bool render = true);
 	/** calls all StripFilters on current text
 	*
 	* @param tmpKey key to use to grab text
@@ -550,24 +561,31 @@ protected:
 	* @return this module's text at specified key location massaged by Render filters
 	*/
 	virtual const char *RenderText(SWKey * tmpKey);
-
-	/** 
+	/**
 	*
 	*  option to specify behaviour when iterating over consecutive entried linked
 	*  to same text
 	* @param val = true means only include entry once in iteration
 	*/
 	virtual void setSkipConsecutiveLinks(bool val) { skipConsecutiveLinks = val; }
+	/** DOCS NEEDED.
+	*/
 	virtual bool getSkipConsecutiveLinks() { return skipConsecutiveLinks; }
+	/** DOCS NEEDED.
+	*/
 	virtual AttributeTypeList &getEntryAttributes() const { return entryAttributes; }
+	/** DOCS NEEDED.
+	*/
 	virtual void processEntryAttributes(bool val) const { procEntAttr = val; }
+	/** DOCS NEEDED.
+	*/
 	virtual bool isProcessEntryAttributes() const { return procEntAttr; }
 
-
-
 	// OPERATORS -----------------------------------------------------------------
-	
+
 	SWMODULE_OPERATORS
 
 };
+
+SWORD_NAMESPACE_END
 #endif
