@@ -1,4 +1,6 @@
-/*
+/******************************************************************************
+ *
+ * $Id: utf8transliterator.h,v 1.19 2003/07/07 00:25:59 scribe Exp $
  *
  * Copyright 2001 CrossWire Bible Society (http://www.crosswire.org)
  *	CrossWire Bible Society
@@ -32,7 +34,12 @@ enum scriptEnum {SE_OFF, SE_LATIN, /*one-way (to) transliterators*/ SE_BASICLATI
 //#include <unicode/uchar.h>
 
 #include <unicode/unistr.h>
+
+//#ifdef _ICUSWORD_
+//#include <unicode/utrans.h>
+//#else
 #include <unicode/translit.h>
+//#endif
 
 #include <defs.h>
 #include <map>
@@ -51,47 +58,35 @@ typedef std::pair<UnicodeString, SWTransData> SWTransPair;
 
   /** This Filter uses ICU for transliteration
   */
-class SWDLLEXPORT UTF8Transliterator : public SWFilter
-{
+class SWDLLEXPORT UTF8Transliterator : public SWFilter {
 private:
 
-  char option;
+	char option;
 
-  static const char optionstring[NUMTARGETSCRIPTS][16];
+	static const char optionstring[NUMTARGETSCRIPTS][16];
 
-  static const char optName[];
-  static const char optTip[];
-  static const char SW_RB_RULE_BASED_IDS[];
-  static const char SW_RB_RULE[];
-  static const char SW_RESDATA[];
-  OptionsList options;
-  static SWTransMap transMap;
-  UErrorCode utf8status;
-  
-  void Load(UErrorCode &status);
-  void  registerTrans(const UnicodeString& ID, const UnicodeString& resource,
-		UTransDirection dir, UErrorCode &status );
-  bool checkTrans(const UnicodeString& ID, UErrorCode &status );
-  Transliterator * createTrans(const UnicodeString& preID, const UnicodeString& ID, 
-  	const UnicodeString& postID, UTransDirection dir, UErrorCode &status );
-  
- public:
-  UTF8Transliterator ();
-  virtual char ProcessText (char *text, int maxlen, const SWKey * key, const SWModule * = 0);
-  virtual const char *getOptionName ()
-    {
-      return optName;
-    }
-  virtual const char *getOptionTip ()
-    {
-      return optTip;
-    }
-  virtual void setOptionValue (const char *ival);
-  virtual const char *getOptionValue ();
-  virtual OptionsList getOptionValues ()
-    {
-      return options;
-    }
+	static const char optName[];
+	static const char optTip[];
+	static const char SW_RB_RULE_BASED_IDS[];
+	static const char SW_RB_RULE[];
+	static const char SW_RESDATA[];
+	StringList options;
+	static SWTransMap transMap;
+	UErrorCode utf8status;
+
+	void Load(UErrorCode &status);
+	void registerTrans(const UnicodeString& ID, const UnicodeString& resource, UTransDirection dir, UErrorCode &status);
+	bool checkTrans(const UnicodeString& ID, UErrorCode &status);
+	Transliterator *createTrans(const UnicodeString& preID, const UnicodeString& ID, const UnicodeString& postID, UTransDirection dir, UErrorCode &status);
+
+public:
+	UTF8Transliterator();
+	virtual char processText(SWBuf &text, const SWKey *key = 0, const SWModule *module = 0);
+	virtual const char *getOptionName() { return optName; }
+	virtual const char *getOptionTip() { return optTip; }
+	virtual void setOptionValue(const char *ival);
+	virtual const char *getOptionValue();
+	virtual StringList getOptionValues() { return options; }
 };
 
 SWORD_NAMESPACE_END
