@@ -2,7 +2,7 @@
  *  filemgr.cpp	- implementation of class FileMgr used for pooling file
  *  					handles
  *
- * $Id: filemgr.cpp,v 1.23 2002/10/01 19:52:40 dglassey Exp $
+ * $Id: filemgr.cpp,v 1.32 2003/08/27 21:47:56 chrislit Exp $
  *
  * Copyright 1998 CrossWire Bible Society (http://www.crosswire.org)
  *	CrossWire Bible Society
@@ -31,10 +31,23 @@
 #include <string.h>
 #ifndef __GNUC__
 #include <io.h>
+#include <direct.h>
 #else
 #include <unistd.h>
 #endif
 
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
+#ifndef S_IRGRP
+#define S_IRGRP 0
+#endif
+
+#ifndef S_IROTH
+#define S_IROTH 0
+#endif
+
 SWORD_NAMESPACE_START
 
 // ---------------- statics -----------------
@@ -43,7 +56,7 @@ FileMgr FileMgr::systemFileMgr;
 // --------------- end statics --------------
 
 
-FileDesc::FileDesc(FileMgr *parent, char *path, int mode, int perms, bool tryDowngrade) {
+FileDesc::FileDesc(FileMgr *parent, const char *path, int mode, int perms, bool tryDowngrade) {
 	this->parent = parent;
 	this->path = 0;
 	stdstr(&this->path, path);
@@ -88,11 +101,11 @@ FileMgr::~FileMgr() {
 }
 
 
-FileDesc *FileMgr::open(char *path, int mode, bool tryDowngrade) {
-	return open(path, mode, S_IREAD | S_IWRITE, tryDowngrade);
+FileDesc *FileMgr::open(const char *path, int mode, bool tryDowngrade) {
+	return open(path, mode, S_IREAD | S_IWRITE|S_IRGRP|S_IROTH, tryDowngrade);
 }
 
-FileDesc *FileMgr::open(char *path, int mode, int perms, bool tryDowngrade) {
+FileDesc *FileMgr::open(const char *path, int mode, int perms, bool tryDowngrade) {
 	FileDesc **tmp, *tmp2;
 	
 	for (tmp = &files; *tmp; tmp = &((*tmp)->next)) {
@@ -146,7 +159,7 @@ signed char FileMgr::trunc(FileDesc *file) {
 		if (i == 9999)
 			return -2;
 
-		int fd = ::open(buf, O_CREAT|O_RDWR, S_IREAD|S_IWRITE);
+		int fd = ::open(buf, O_CREAT|O_RDWR, S_IREAD|S_IWRITE|S_IRGRP|S_IROTH);
 		if (fd < 0)
 			return -3;
 	
@@ -158,7 +171,7 @@ signed char FileMgr::trunc(FileDesc *file) {
 		}
 		// zero out the file
 		::close(file->fd);
-		file->fd = ::open(file->path, O_TRUNC, S_IREAD|S_IWRITE);
+		file->fd = ::open(file->path, O_TRUNC, S_IREAD|S_IWRITE|S_IRGRP|S_IROTH);
 		::close(file->fd);
 		file->fd = -77;	// force file open by filemgr
 		// copy tmp file back (dumb, but must preserve file permissions)
@@ -170,7 +183,7 @@ signed char FileMgr::trunc(FileDesc *file) {
 		
 		::close(fd);
 		::close(file->fd);
-		unlink(buf);		// remove our tmp file
+		removeFile(buf);		// remove our tmp file
 		file->fd = -77;	// causes file to be swapped out forcing open on next call to getFd()
 	}
 	else { // put offset back and return failure
@@ -263,8 +276,82 @@ signed char FileMgr::existsDir(const char *ipath, const char *idirName)
 		sprintf(ch, "/%s", idirName);
 	}
 	signed char retVal = !access(path, 04);
-     delete [] path;
-     return retVal;
+	delete [] path;
+	return retVal;
 }
 
+
+int FileMgr::createParent(const char *pName) {
+	char *buf = new char [ strlen(pName) + 1 ];
+	int retCode = 0;
+	
+	strcpy(buf, pName);
+	int end = strlen(buf) - 1;
+	while (end) {
+		if ((buf[end] == '/') || (buf[end] == '\\'))
+			break;
+		end--;
+	}
+	buf[end] = 0;
+	if (strlen(buf)>0) {
+		if (access(buf, 02)) {  // not exists with write access?
+			if ((retCode = mkdir(buf
+#ifndef WIN32
+					, 0755
+#endif
+					))) {
+				createParent(buf);
+				retCode = mkdir(buf
+#ifndef WIN32
+					, 0755
+#endif
+					);
+			}
+		}
+	}
+	else retCode = -1;
+	delete [] buf;
+	return retCode;
+}
+	
+
+int FileMgr::createPathAndFile(const char *fName) {
+	int fd;
+	
+	fd = ::open(fName, O_CREAT|O_WRONLY|O_BINARY, S_IREAD|S_IWRITE|S_IRGRP|S_IROTH);
+	if (fd < 1) {
+		createParent(fName);
+		fd = ::open(fName, O_CREAT|O_WRONLY|O_BINARY, S_IREAD|S_IWRITE|S_IRGRP|S_IROTH);
+	}
+	return fd;
+}
+
+
+int FileMgr::copyFile(const char *sourceFile, const char *targetFile) {
+	int sfd, dfd, len;
+	char buf[4096];
+
+	if ((sfd = ::open(sourceFile, O_RDONLY|O_BINARY)) < 1)
+		return -1;
+	if ((dfd = createPathAndFile(targetFile)) < 1)
+		return -1;
+
+	do {
+		len = read(sfd, buf, 4096);
+		write(dfd, buf, len);
+	}
+	while(len == 4096);	
+	::close(dfd);
+	::close(sfd);
+	
+	return 0;
+}
+
+
+int FileMgr::removeFile(const char *fName) {
+	return ::remove(fName);
+}
+
+
+
 SWORD_NAMESPACE_END
