@@ -2,7 +2,7 @@
  *  swmgr.cpp   - implementaion of class SWMgr used to interact with an install
  *				base of sword modules.
  *
- * $Id: swmgr.cpp,v 1.81 2002/10/01 22:04:59 dglassey Exp $
+ * $Id: swmgr.cpp,v 1.94 2003/08/14 00:57:38 scribe Exp $
  *
  * Copyright 1998 CrossWire Bible Society (http://www.crosswire.org)
  *	CrossWire Bible Society
@@ -46,14 +46,24 @@
 #include <utilfuns.h>
 #include <gbfplain.h>
 #include <thmlplain.h>
+#include <osisplain.h>
 #include <gbfstrongs.h>
 #include <gbffootnotes.h>
 #include <gbfheadings.h>
+#include <gbfredletterwords.h>
 #include <gbfmorph.h>
+#include <osisheadings.h>
+#include <osisfootnotes.h>
+#include <osisstrongs.h>
+#include <osismorph.h>
+#include <osislemma.h>
+#include <osisredletterwords.h>
+#include <osisscripref.h>
 #include <thmlstrongs.h>
 #include <thmlfootnotes.h>
 #include <thmlheadings.h>
 #include <thmlmorph.h>
+#include <thmlvariants.h>
 #include <thmllemma.h>
 #include <thmlscripref.h>
 #include <cipherfil.h>
@@ -107,6 +117,10 @@ void SWMgr::init() {
 	optionFilters.clear();
 	cleanupFilters.clear();
 
+	tmpFilter = new ThMLVariants();
+	optionFilters.insert(FilterMap::value_type("ThMLVariants", tmpFilter));
+	cleanupFilters.push_back(tmpFilter);
+
 	tmpFilter = new GBFStrongs();
 	optionFilters.insert(FilterMap::value_type("GBFStrongs", tmpFilter));
 	cleanupFilters.push_back(tmpFilter);
@@ -115,6 +129,10 @@ void SWMgr::init() {
 	optionFilters.insert(FilterMap::value_type("GBFFootnotes", tmpFilter));
 	cleanupFilters.push_back(tmpFilter);
 
+	tmpFilter = new GBFRedLetterWords();
+	optionFilters.insert(FilterMap::value_type("GBFRedLetterWords", tmpFilter));
+	cleanupFilters.push_back(tmpFilter);
+
 	tmpFilter = new GBFMorph();
 	optionFilters.insert(FilterMap::value_type("GBFMorph", tmpFilter));
 	cleanupFilters.push_back(tmpFilter);
@@ -123,6 +141,34 @@ void SWMgr::init() {
 	optionFilters.insert(FilterMap::value_type("GBFHeadings", tmpFilter));
 	cleanupFilters.push_back(tmpFilter);
 
+	tmpFilter = new OSISHeadings();
+	optionFilters.insert(FilterMap::value_type("OSISHeadings", tmpFilter));
+	cleanupFilters.push_back(tmpFilter);
+
+	tmpFilter = new OSISStrongs();
+	optionFilters.insert(FilterMap::value_type("OSISStrongs", tmpFilter));
+	cleanupFilters.push_back(tmpFilter);
+
+	tmpFilter = new OSISMorph();
+	optionFilters.insert(FilterMap::value_type("OSISMorph", tmpFilter));
+	cleanupFilters.push_back(tmpFilter);
+
+	tmpFilter = new OSISLemma();
+	optionFilters.insert(FilterMap::value_type("OSISLemma", tmpFilter));
+	cleanupFilters.push_back(tmpFilter);
+
+	tmpFilter = new OSISFootnotes();
+	optionFilters.insert(FilterMap::value_type("OSISFootnotes", tmpFilter));
+	cleanupFilters.push_back(tmpFilter);
+
+	tmpFilter = new OSISScripref();
+	optionFilters.insert(FilterMap::value_type("OSISScripref", tmpFilter));
+	cleanupFilters.push_back(tmpFilter);
+
+	tmpFilter = new OSISRedLetterWords();
+	optionFilters.insert(FilterMap::value_type("OSISRedLetterWords", tmpFilter));
+	cleanupFilters.push_back(tmpFilter);
+
 	tmpFilter = new ThMLStrongs();
 	optionFilters.insert(FilterMap::value_type("ThMLStrongs", tmpFilter));
 	cleanupFilters.push_back(tmpFilter);
@@ -176,6 +222,9 @@ void SWMgr::init() {
 
 	thmlplain = new ThMLPlain();
 	cleanupFilters.push_back(thmlplain);
+
+	osisplain = new OSISPlain();
+	cleanupFilters.push_back(osisplain);
 }
 
 
@@ -214,7 +263,7 @@ void SWMgr::commonInit(SWConfig * iconfig, SWConfig * isysconfig, bool autoload,
 
 SWMgr::SWMgr(const char *iConfigPath, bool autoload, SWFilterMgr *filterMgr) {
 
-	std::string path;
+	SWBuf path;
 	
 	this->filterMgr = filterMgr;
 	if (filterMgr)
@@ -223,7 +272,8 @@ SWMgr::SWMgr(const char *iConfigPath, bool autoload, SWFilterMgr *filterMgr) {
 	init();
 	
 	path = iConfigPath;
-	if ((iConfigPath[strlen(iConfigPath)-1] != '\\') && (iConfigPath[strlen(iConfigPath)-1] != '/'))
+	int len = strlen(iConfigPath);
+	if ((len < 1) || (iConfigPath[len-1] != '\\') && (iConfigPath[len-1] != '/'))
 		path += "/";
 	if (FileMgr::existsFile(path.c_str(), "mods.conf")) {
 		stdstr(&prefixPath, path.c_str());
@@ -271,8 +321,8 @@ SWMgr::~SWMgr() {
 }
 
 
-void SWMgr::findConfig(char *configType, char **prefixPath, char **configPath, std::list<std::string> *augPaths) {
-	std::string path;
+void SWMgr::findConfig(char *configType, char **prefixPath, char **configPath, std::list<SWBuf> *augPaths) {
+	SWBuf path;
 	ConfigEntMap::iterator entry;
 	ConfigEntMap::iterator lastEntry;
 
@@ -516,7 +566,7 @@ void SWMgr::loadConfigDir(const char *ipath)
 {
    DIR *dir;
    struct dirent *ent;
-   std::string newmodfile;
+   SWBuf newmodfile;
  
 	if ((dir = opendir(ipath))) {
 		rewinddir(dir);
@@ -546,7 +596,7 @@ void SWMgr::loadConfigDir(const char *ipath)
 
 
 void SWMgr::augmentModules(const char *ipath) {
-	std::string path = ipath;
+	SWBuf path = ipath;
 	if ((ipath[strlen(ipath)-1] != '\\') && (ipath[strlen(ipath)-1] != '/'))
 		path += "/";
 	if (FileMgr::existsDir(path.c_str(), "mods.d")) {
@@ -614,13 +664,13 @@ signed char SWMgr::Load() {
 
 		CreateMods();
 
-		for (std::list<std::string>::iterator pathIt = augPaths.begin(); pathIt != augPaths.end(); pathIt++) {
+		for (std::list<SWBuf>::iterator pathIt = augPaths.begin(); pathIt != augPaths.end(); pathIt++) {
 			augmentModules(pathIt->c_str());
 		}
 //	augment config with ~/.sword/mods.d if it exists ---------------------
 		char *envhomedir  = getenv ("HOME");
 		if (envhomedir != NULL && configType != 2) { // 2 = user only
-			std::string path = envhomedir;
+			SWBuf path = envhomedir;
 			if ((envhomedir[strlen(envhomedir)-1] != '\\') && (envhomedir[strlen(envhomedir)-1] != '/'))
 				path += "/";
 			path += ".sword/";
@@ -639,41 +689,44 @@ signed char SWMgr::Load() {
 	return ret;
 }
 
-SWModule *SWMgr::CreateMod(std::string name, std::string driver, ConfigEntMap &section)
+SWModule *SWMgr::CreateMod(const char *name, const char *driver, ConfigEntMap &section)
 {
-	std::string description, datapath, misc1;
+	SWBuf description, datapath, misc1;
 	ConfigEntMap::iterator entry;
 	SWModule *newmod = 0;
-	std::string lang, sourceformat, encoding;
-        signed char direction, enc, markup;
+	SWBuf lang, sourceformat, encoding;
+	signed char direction, enc, markup;
 
-	description  = ((entry = section.find("Description")) != section.end()) ? (*entry).second : (std::string)"";
-	lang  = ((entry = section.find("Lang")) != section.end()) ? (*entry).second : (std::string)"en";
- 	sourceformat = ((entry = section.find("SourceType"))  != section.end()) ? (*entry).second : (std::string)"";
- 	encoding = ((entry = section.find("Encoding"))  != section.end()) ? (*entry).second : (std::string)"";
+	description  = ((entry = section.find("Description")) != section.end()) ? (*entry).second : (SWBuf)"";
+	lang  = ((entry = section.find("Lang")) != section.end()) ? (*entry).second : (SWBuf)"en";
+ 	sourceformat = ((entry = section.find("SourceType"))  != section.end()) ? (*entry).second : (SWBuf)"";
+ 	encoding = ((entry = section.find("Encoding"))  != section.end()) ? (*entry).second : (SWBuf)"";
 	datapath = prefixPath;
 	if ((prefixPath[strlen(prefixPath)-1] != '\\') && (prefixPath[strlen(prefixPath)-1] != '/'))
 		datapath += "/";
-	misc1 += ((entry = section.find("DataPath")) != section.end()) ? (*entry).second : (std::string)"";
+	misc1 += ((entry = section.find("DataPath")) != section.end()) ? (*entry).second : (SWBuf)"";
 	char *buf = new char [ strlen(misc1.c_str()) + 1 ];
 	char *buf2 = buf;
 	strcpy(buf, misc1.c_str());
 //	for (; ((*buf2) && ((*buf2 == '.') || (*buf2 == '/') || (*buf2 == '\\'))); buf2++);
 	for (; ((*buf2) && ((*buf2 == '/') || (*buf2 == '\\'))); buf2++);
+	if (!strncmp(buf2, "./", 2)) { //remove the leading ./ in the module data path to make it look better
+		buf2 += 2;
+	}
 	if (*buf2)
 		datapath += buf2;
 	delete [] buf;
 
 	section["AbsoluteDataPath"] = datapath;
 
-        if (!stricmp(sourceformat.c_str(), "GBF"))
-                markup = FMT_GBF;
-        else if (!stricmp(sourceformat.c_str(), "ThML"))
-                markup = FMT_THML;
-        else if (!stricmp(sourceformat.c_str(), "OSIS"))
-                markup = FMT_OSIS;
-        else
-                markup = FMT_PLAIN;
+	if (!stricmp(sourceformat.c_str(), "GBF"))
+		markup = FMT_GBF;
+	else if (!stricmp(sourceformat.c_str(), "ThML"))
+		markup = FMT_THML;
+	else if (!stricmp(sourceformat.c_str(), "OSIS"))
+		markup = FMT_OSIS;
+	else
+		markup = FMT_GBF;
 
 	if (!stricmp(encoding.c_str(), "SCSU"))
 		enc = ENC_SCSU;
@@ -683,22 +736,22 @@ SWModule *SWMgr::CreateMod(std::string name, std::string driver, ConfigEntMap &s
 	else enc = ENC_LATIN1;
 
 	if ((entry = section.find("Direction")) == section.end()) {
-                direction = DIRECTION_LTR;
-        }
-        else if (!stricmp((*entry).second.c_str(), "rtol")) {
-                direction = DIRECTION_RTL;
-        }
-        else if (!stricmp((*entry).second.c_str(), "bidi")) {
-                direction = DIRECTION_BIDI;
-        }
-        else {
-                direction = DIRECTION_LTR;
-        }
+		direction = DIRECTION_LTR;
+	}
+	else if (!stricmp((*entry).second.c_str(), "rtol")) {
+		direction = DIRECTION_RTL;
+	}
+	else if (!stricmp((*entry).second.c_str(), "bidi")) {
+		direction = DIRECTION_BIDI;
+	}
+	else {
+		direction = DIRECTION_LTR;
+	}
 
-	if ((!stricmp(driver.c_str(), "zText")) || (!stricmp(driver.c_str(), "zCom"))) {
+	if ((!stricmp(driver, "zText")) || (!stricmp(driver, "zCom"))) {
 		SWCompress *compress = 0;
 		int blockType = CHAPTERBLOCKS;
-		misc1 = ((entry = section.find("BlockType")) != section.end()) ? (*entry).second : (std::string)"CHAPTER";
+		misc1 = ((entry = section.find("BlockType")) != section.end()) ? (*entry).second : (SWBuf)"CHAPTER";
 		if (!stricmp(misc1.c_str(), "VERSE"))
 			blockType = VERSEBLOCKS;
 		else if (!stricmp(misc1.c_str(), "CHAPTER"))
@@ -706,7 +759,7 @@ SWModule *SWMgr::CreateMod(std::string name, std::string driver, ConfigEntMap &s
 		else if (!stricmp(misc1.c_str(), "BOOK"))
 			blockType = BOOKBLOCKS;
 
-		misc1 = ((entry = section.find("CompressType")) != section.end()) ? (*entry).second : (std::string)"LZSS";
+		misc1 = ((entry = section.find("CompressType")) != section.end()) ? (*entry).second : (SWBuf)"LZSS";
 #ifndef EXCLUDEZLIB
 		if (!stricmp(misc1.c_str(), "ZIP"))
 			compress = new ZipCompress();
@@ -716,48 +769,53 @@ SWModule *SWMgr::CreateMod(std::string name, std::string driver, ConfigEntMap &s
 			compress = new LZSSCompress();
 
 		if (compress) {
-			if (!stricmp(driver.c_str(), "zText"))
-				newmod = new zText(datapath.c_str(), name.c_str(), description.c_str(), blockType, compress, 0, enc, direction, markup, lang.c_str());
-			else	newmod = new zCom(datapath.c_str(), name.c_str(), description.c_str(), blockType, compress, 0, enc, direction, markup, lang.c_str());
+			if (!stricmp(driver, "zText"))
+				newmod = new zText(datapath.c_str(), name, description.c_str(), blockType, compress, 0, enc, direction, markup, lang.c_str());
+			else	newmod = new zCom(datapath.c_str(), name, description.c_str(), blockType, compress, 0, enc, direction, markup, lang.c_str());
 		}
 	}
 
-	if (!stricmp(driver.c_str(), "RawText")) {
-		newmod = new RawText(datapath.c_str(), name.c_str(), description.c_str(), 0, enc, direction, markup, lang.c_str());
+	if (!stricmp(driver, "RawText")) {
+		newmod = new RawText(datapath.c_str(), name, description.c_str(), 0, enc, direction, markup, lang.c_str());
 	}
 
 	// backward support old drivers
-	if (!stricmp(driver.c_str(), "RawGBF")) {
-		newmod = new RawText(datapath.c_str(), name.c_str(), description.c_str(), 0, enc, direction, markup, lang.c_str());
+	if (!stricmp(driver, "RawGBF")) {
+		newmod = new RawText(datapath.c_str(), name, description.c_str(), 0, enc, direction, markup, lang.c_str());
 	}
 
-	if (!stricmp(driver.c_str(), "RawCom")) {
-		newmod = new RawCom(datapath.c_str(), name.c_str(), description.c_str(), 0, enc, direction, markup, lang.c_str());
+	if (!stricmp(driver, "RawCom")) {
+		newmod = new RawCom(datapath.c_str(), name, description.c_str(), 0, enc, direction, markup, lang.c_str());
 	}
 
-	if (!stricmp(driver.c_str(), "RawFiles")) {
-		newmod = new RawFiles(datapath.c_str(), name.c_str(), description.c_str(), 0, enc, direction, markup, lang.c_str());
+	if (!stricmp(driver, "RawFiles")) {
+		newmod = new RawFiles(datapath.c_str(), name, description.c_str(), 0, enc, direction, markup, lang.c_str());
 	}
 
-	if (!stricmp(driver.c_str(), "HREFCom")) {
-		misc1 = ((entry = section.find("Prefix")) != section.end()) ? (*entry).second : (std::string)"";
-		newmod = new HREFCom(datapath.c_str(), misc1.c_str(), name.c_str(), description.c_str());
+	if (!stricmp(driver, "HREFCom")) {
+		misc1 = ((entry = section.find("Prefix")) != section.end()) ? (*entry).second : (SWBuf)"";
+		newmod = new HREFCom(datapath.c_str(), misc1.c_str(), name, description.c_str());
 	}
 
-	if (!stricmp(driver.c_str(), "RawLD"))
-		newmod = new RawLD(datapath.c_str(), name.c_str(), description.c_str(), 0, enc, direction, markup, lang.c_str());
+        int pos;  //used for position of final / in AbsoluteDataPath, but also set to 1 for modules types that need to strip module name
+	if (!stricmp(driver, "RawLD")) {
+		newmod = new RawLD(datapath.c_str(), name, description.c_str(), 0, enc, direction, markup, lang.c_str());
+                pos = 1;
+        }
 
-	if (!stricmp(driver.c_str(), "RawLD4"))
-		newmod = new RawLD4(datapath.c_str(), name.c_str(), description.c_str(), 0, enc, direction, markup, lang.c_str());
+	if (!stricmp(driver, "RawLD4")) {
+		newmod = new RawLD4(datapath.c_str(), name, description.c_str(), 0, enc, direction, markup, lang.c_str());
+                pos = 1;
+        }
 
-	if (!stricmp(driver.c_str(), "zLD")) {
+	if (!stricmp(driver, "zLD")) {
 		SWCompress *compress = 0;
 		int blockCount;
-		misc1 = ((entry = section.find("BlockCount")) != section.end()) ? (*entry).second : (std::string)"200";
+		misc1 = ((entry = section.find("BlockCount")) != section.end()) ? (*entry).second : (SWBuf)"200";
 		blockCount = atoi(misc1.c_str());
 		blockCount = (blockCount) ? blockCount : 200;
 
-		misc1 = ((entry = section.find("CompressType")) != section.end()) ? (*entry).second : (std::string)"LZSS";
+		misc1 = ((entry = section.find("CompressType")) != section.end()) ? (*entry).second : (SWBuf)"LZSS";
 #ifndef EXCLUDEZLIB
 		if (!stricmp(misc1.c_str(), "ZIP"))
 			compress = new ZipCompress();
@@ -767,18 +825,31 @@ SWModule *SWMgr::CreateMod(std::string name, std::string driver, ConfigEntMap &s
 			compress = new LZSSCompress();
 
 		if (compress) {
-			newmod = new zLD(datapath.c_str(), name.c_str(), description.c_str(), blockCount, compress, 0, enc, direction, markup, lang.c_str());
+			newmod = new zLD(datapath.c_str(), name, description.c_str(), blockCount, compress, 0, enc, direction, markup, lang.c_str());
+		}
+		pos = 1;
+	}
+
+	if (!stricmp(driver, "RawGenBook")) {
+		newmod = new RawGenBook(datapath.c_str(), name, description.c_str(), 0, enc, direction, markup, lang.c_str());
+		pos = 1;
+	}
+
+	if (pos == 1) {
+		SWBuf &dp = section["AbsoluteDataPath"];
+		for (int i = dp.length() - 1; i; i--) {
+			if (dp[i] == '/') {
+				dp.setSize(i);
+				break;
+			}
 		}
 	}
 
-	if (!stricmp(driver.c_str(), "RawGenBook")) {
-		newmod = new RawGenBook(datapath.c_str(), name.c_str(), description.c_str(), 0, enc, direction, markup, lang.c_str());
-	}
-    // if a specific module type is set in the config, use this
-    if ((entry = section.find("Type")) != section.end())
-        newmod->Type(entry->second.c_str());
+	// if a specific module type is set in the config, use this
+	if ((entry = section.find("Type")) != section.end())
+		newmod->Type(entry->second.c_str());
 
-     newmod->setConfig(&section);
+	newmod->setConfig(&section);
 	return newmod;
 }
 
@@ -789,7 +860,7 @@ void SWMgr::AddGlobalOptions(SWModule *module, ConfigEntMap &section, ConfigEntM
 		it = optionFilters.find((*start).second);
 		if (it != optionFilters.end()) {
 			module->AddOptionFilter((*it).second);	// add filter to module and option as a valid option
-			OptionsList::iterator loop;
+			StringList::iterator loop;
 			for (loop = options.begin(); loop != options.end(); loop++) {
 				if (!strcmp((*loop).c_str(), (*it).second->getOptionName()))
 					break;
@@ -822,11 +893,11 @@ void SWMgr::AddLocalOptions(SWModule *module, ConfigEntMap &section, ConfigEntMa
 
 
 void SWMgr::AddRawFilters(SWModule *module, ConfigEntMap &section) {
-	std::string sourceformat, cipherKey;
+	SWBuf sourceformat, cipherKey;
 	ConfigEntMap::iterator entry;
 
-	cipherKey = ((entry = section.find("CipherKey")) != section.end()) ? (*entry).second : (std::string)"";
-	if (!cipherKey.empty()) {
+	cipherKey = ((entry = section.find("CipherKey")) != section.end()) ? (*entry).second : (SWBuf)"";
+	if (cipherKey.length()) {
 		SWFilter *cipherFilter = new CipherFilter(cipherKey.c_str());
 		cipherFilters.insert(FilterMap::value_type(module->Name(), cipherFilter));
 		cleanupFilters.push_back(cipherFilter);
@@ -839,22 +910,21 @@ void SWMgr::AddRawFilters(SWModule *module, ConfigEntMap &section) {
 
 
 void SWMgr::AddEncodingFilters(SWModule *module, ConfigEntMap &section) {
-
 	if (filterMgr)
 		filterMgr->AddEncodingFilters(module, section);
 }
 
 
 void SWMgr::AddRenderFilters(SWModule *module, ConfigEntMap &section) {
-	std::string sourceformat;
+	SWBuf sourceformat;
 	ConfigEntMap::iterator entry;
 
-	sourceformat = ((entry = section.find("SourceType")) != section.end()) ? (*entry).second : (std::string)"";
+	sourceformat = ((entry = section.find("SourceType")) != section.end()) ? (*entry).second : (SWBuf)"";
 
 	// Temporary: To support old module types
-        // TODO: Remove at 1.6.0 release?
-	if (sourceformat.empty()) {
-		sourceformat = ((entry = section.find("ModDrv")) != section.end()) ? (*entry).second : (std::string)"";
+	// TODO: Remove at 1.6.0 release?
+	if (!sourceformat.length()) {
+		sourceformat = ((entry = section.find("ModDrv")) != section.end()) ? (*entry).second : (SWBuf)"";
 		if (!stricmp(sourceformat.c_str(), "RawGBF"))
 			sourceformat = "GBF";
 		else sourceformat = "";
@@ -873,13 +943,13 @@ void SWMgr::AddRenderFilters(SWModule *module, ConfigEntMap &section) {
 
 void SWMgr::AddStripFilters(SWModule *module, ConfigEntMap &section)
 {
-	std::string sourceformat;
+	SWBuf sourceformat;
 	ConfigEntMap::iterator entry;
 
-	sourceformat = ((entry = section.find("SourceType")) != section.end()) ? (*entry).second : (std::string)"";
+	sourceformat = ((entry = section.find("SourceType")) != section.end()) ? (*entry).second : (SWBuf)"";
 	// Temporary: To support old module types
-	if (sourceformat.empty()) {
-		sourceformat = ((entry = section.find("ModDrv")) != section.end()) ? (*entry).second : (std::string)"";
+	if (!sourceformat.length()) {
+		sourceformat = ((entry = section.find("ModDrv")) != section.end()) ? (*entry).second : (SWBuf)"";
 		if (!stricmp(sourceformat.c_str(), "RawGBF"))
 			sourceformat = "GBF";
 		else sourceformat = "";
@@ -891,6 +961,9 @@ void SWMgr::AddStripFilters(SWModule *module, ConfigEntMap &section)
 	else if (!stricmp(sourceformat.c_str(), "ThML")) {
 		module->AddStripFilter(thmlplain);
 	}
+	else if (!stricmp(sourceformat.c_str(), "OSIS")) {
+		module->AddStripFilter(osisplain);
+	}
 
 	if (filterMgr)
 		filterMgr->AddStripFilters(module, section);
@@ -904,13 +977,13 @@ void SWMgr::CreateMods() {
 	ConfigEntMap::iterator end;
 	ConfigEntMap::iterator entry;
 	SWModule *newmod;
-	std::string driver, misc1;
+	SWBuf driver, misc1;
 	for (it = config->Sections.begin(); it != config->Sections.end(); it++) {
 		ConfigEntMap &section = (*it).second;
 		newmod = 0;
 		
-		driver = ((entry = section.find("ModDrv")) != section.end()) ? (*entry).second : (std::string)"";
-		if (!driver.empty()) {
+		driver = ((entry = section.find("ModDrv")) != section.end()) ? (*entry).second : (SWBuf)"";
+		if (driver.length()) {
 			newmod = CreateMod((*it).first, driver, section);
 			if (newmod) {
 				start = (*it).second.lower_bound("GlobalOptionFilter");
@@ -949,8 +1022,8 @@ void SWMgr::InstallScan(const char *dirname)
    DIR *dir;
    struct dirent *ent;
    int conffd = 0;
-   std::string newmodfile;
-   std::string targetName;
+   SWBuf newmodfile;
+   SWBuf targetName;
  
 	if (!access(dirname, 04)) {
           if ((dir = opendir(dirname))) {
@@ -1042,15 +1115,15 @@ const char *SWMgr::getGlobalOptionTip(const char *option)
 }
 
 
-OptionsList SWMgr::getGlobalOptions()
+StringList SWMgr::getGlobalOptions()
 {
 	return options;
 }
 
 
-OptionsList SWMgr::getGlobalOptionValues(const char *option)
+StringList SWMgr::getGlobalOptionValues(const char *option)
 {
-	OptionsList options;
+	StringList options;
 	for (FilterMap::iterator it = optionFilters.begin(); it != optionFilters.end(); it++) {
 		if ((*it).second->getOptionName()) {
 			if (!stricmp(option, (*it).second->getOptionName())) {
