@@ -1,12 +1,11 @@
 /******************************************************************************
  *
- * gbffootnotes -	SWFilter decendant to hide or show footnotes
+ * gbffootnotes -	SWFilter descendant to hide or show footnotes
  *			in a GBF module.
  */
 
 
 #include <stdlib.h>
-#include <string.h>
 #include <gbffootnotes.h>
 #ifndef __GNUC__
 #else
@@ -15,49 +14,35 @@
 
 SWORD_NAMESPACE_START
 
-const char GBFFootnotes::on[] = "On";
-const char GBFFootnotes::off[] = "Off";
-const char GBFFootnotes::optName[] = "Footnotes";
-const char GBFFootnotes::optTip[] = "Toggles Footnotes On and Off if they exist";
+const char oName[] = "Footnotes";
+const char oTip[] = "Toggles Footnotes On and Off if they exist";
+
+const SWBuf choices[3] = {"On", "Off", ""};
+const StringList oValues(&choices[0], &choices[2]);
 
 
-GBFFootnotes::GBFFootnotes() {
-	option = false;
-	options.push_back(on);
-	options.push_back(off);
+GBFFootnotes::GBFFootnotes() : SWOptionFilter(oName, oTip, &oValues) {
+	setOptionValue("Off");
 }
 
 
 GBFFootnotes::~GBFFootnotes() {
 }
 
-void GBFFootnotes::setOptionValue(const char *ival)
-{
-	option = (!stricmp(ival, on));
-}
 
-const char *GBFFootnotes::getOptionValue()
-{
-	return (option) ? on:off;
-}
-
-char GBFFootnotes::ProcessText(char *text, int maxlen, const SWKey *key, const SWModule *module)
+char GBFFootnotes::processText (SWBuf &text, const SWKey *key, const SWModule *module)
 {
 	if (!option) {	// if we don't want footnotes
-		char *to, *from, token[4096]; // cheese.  Fix.
+		char token[4096]; // cheese.  Fix.
 		int tokpos = 0;
 		bool intoken = false;
 		int len;
 		bool hide = false;
 
-		len = strlen(text) + 1;	// shift string to right of buffer
-		if (len < maxlen) {
-			memmove(&text[maxlen - len], text, len);
-			from = &text[maxlen - len];
-		}
-		else	from = text;	// -------------------------------
-
-		for (to = text; *from; from++) {
+	const char *from;
+	SWBuf orig = text;
+	from = orig.c_str();
+	for (text = ""; *from; from++) {
 			if (*from == '<') {
 				intoken = true;
 				tokpos = 0;
@@ -94,10 +79,9 @@ char GBFFootnotes::ProcessText(char *text, int maxlen, const SWKey *key, const S
 				}
 				// if not a footnote token, keep token in text
 				if (!hide) {
-					*to++ = '<';
-					for (char *tok = token; *tok; tok++)
-						*to++ = *tok;
-					*to++ = '>';
+					text += '<';
+					text += token;
+					text += '>';
 				}
 				continue;
 			}
@@ -108,12 +92,10 @@ char GBFFootnotes::ProcessText(char *text, int maxlen, const SWKey *key, const S
 			}
 			else	{
 				if (!hide) {
-					*to++ = *from;
+					text += *from;
 				}
 			}
 		}
-		*to++ = 0;
-		*to = 0;
 	}
 	return 0;
 }
