@@ -15,12 +15,12 @@
  ***************************************************************************/
 
 #include <stdlib.h>
-#include <string.h>
 #include <thmlhtmlhref.h>
 #include <swmodule.h>
+#include <utilxml.h>
 
 SWORD_NAMESPACE_START
-
+ 
 ThMLHTMLHREF::ThMLHTMLHREF() {
 	setTokenStart("<");
 	setTokenEnd(">");
@@ -135,133 +135,142 @@ ThMLHTMLHREF::ThMLHTMLHREF() {
 }
 
 
-bool ThMLHTMLHREF::handleToken(char **buf, const char *token, DualStringMap &userData) {
+bool ThMLHTMLHREF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *userData) {
 	const char *tok;
-	if (!substituteToken(buf, token)) {
-	// manually process if it wasn't a simple substitution
-		if (!strncmp(token, "sync ", 5)) {
-			pushString(buf, "<a href=\"");
-			for (tok = token + 5; *(tok+1); tok++)
-				if(*tok != '\"')
-					*(*buf)++ = *tok;
-			*(*buf)++ = '\"';
-			*(*buf)++ = '>';
+	if (!substituteToken(buf, token)) { // manually process if it wasn't a simple substitution
+		MyUserData *u = (MyUserData *)userData;
+		XMLTag tag(token);
+		if (tag.getName() && !strcmp(tag.getName(), "sync")) {
+			if( tag.getAttribute("type") && !strcmp(tag.getAttribute("type"), "morph")) { //&gt;
+				buf += "<small><em>(<a href=\"";
+			}
+			else {
+				buf += "<small><em>&lt;<a href=\"";
+			}
 
-                        //scan for value and add it to the buffer
-			for (tok = token + 5; *tok; tok++) {
-				if (!strncmp(tok, "value=\"", 7)) {
-					tok += 7;
-					for (;*tok != '\"'; tok++)
-						*(*buf)++ = *tok;
-					break;
+			buf += "type=";
+			buf += tag.getAttribute("type");
+
+			const char* value = tag.getAttribute("value");
+			buf += " value=";
+			buf += value ? value : "";
+			buf += "\">";
+
+			//add the value
+			if (value && tag.getAttribute("type") && strcmp(tag.getAttribute("type"), "morph")) { //not morph type
+				value++;
+			}
+			buf += value ? value : "";
+
+			if(tag.getAttribute("type") && !strcmp(tag.getAttribute("type"), "morph"))
+				buf += "</a>) </em></small>";
+			else
+				buf += "</a>&gt; </em></small>";
+		}
+		else if (tag.getName() && !strcmp(tag.getName(), "scripture")) {
+			u->inscriptRef = true;
+			buf += "<i>";
+		}
+		else if (tag.getName() && !strcmp(tag.getName(), "scripRef")) {
+			if (tag.isEndTag()) {
+				if (u->inscriptRef) { // like  "<scripRef passage="John 3:16">John 3:16</scripRef>"
+					u->inscriptRef = false;
+					buf += "</a>";
+				}
+				else { // end of scripRef like "<scripRef>John 3:16</scripRef>"
+					buf += "<a href=\"passage=";
+					buf += u->lastTextNode.c_str();
+					buf += "\">";
+
+					buf += u->lastTextNode.c_str();
+					buf += "</a>";
+
+					// let's let text resume to output again
+					u->suspendTextPassThru = false;
 				}
 			}
-			pushString(buf, "</a>");
-		}
-		
-		else if (!strncmp(token, "scripture ", 10)) {
-			userData["inscriptRef"] = "true";
-			pushString(buf, "<i>");
-		} 
+			else if (tag.getAttribute("passage")) { //passage given
+				u->inscriptRef = true;
 
-		else if (!strncmp(token, "scripRef p", 10) || !strncmp(token, "scripRef v", 10)) {
-			userData["inscriptRef"] = "true";
-			pushString(buf, "<a href=\"");
-			for (const char *tok = token + 9; *(tok+1); tok++)				
-				if(*tok != '\"') 			
-					*(*buf)++ = *tok;
-			*(*buf)++ = '\"';
-			*(*buf)++ = '>';
-		} 
-
-		// we're starting a scripRef like "<scripRef>John 3:16</scripRef>"
-		else if (!strcmp(token, "scripRef")) {
-			userData["inscriptRef"] = "false";
-			// let's stop text from going to output
-			userData["suspendTextPassThru"] = "true";
-		}
-
-		// we've ended a scripRef 
-		else if (!strcmp(token, "/scripRef")) {
-			if (userData["inscriptRef"] == "true") { // like  "<scripRef passage="John 3:16">John 3:16</scripRef>"
-				userData["inscriptRef"] = "false";
-				pushString(buf, "</a>");
+				buf += "<a href=\"";
+				if (const char* version = tag.getAttribute("version")) {
+					buf += "version=";
+					buf += version;
+					buf += " ";
+				}
+				if (const char* passage = tag.getAttribute("passage")) {
+					buf += "passage=";
+					buf += passage;
+				}
+				buf += "\">";
 			}
-			
-			else { // like "<scripRef>John 3:16</scripRef>"
-				pushString(buf, "<a href=\"passage=");
-				//char *strbuf = (char *)userData["lastTextNode"].c_str();
-				pushString(buf, userData["lastTextNode"].c_str());
-				*(*buf)++ = '\"';
-				*(*buf)++ = '>';
-				pushString(buf, userData["lastTextNode"].c_str());
-				// let's let text resume to output again
-				userData["suspendTextPassThru"] = "false";	
-				pushString(buf, "</a>");
+			else { //no passage or version given
+				u->inscriptRef = false;
+				// let's stop text from going to output
+				u->suspendTextPassThru = true;
 			}
 		}
-			
-		else if (!strncmp(token, "div class=\"sechead\"", 19)) {
-		        userData["SecHead"] = "true";
-			pushString(buf, "<br /><b><i>");
-		}
-		else if (!strncmp(token, "div class=\"title\"", 19)) {
-		        userData["SecHead"] = "true";
-			pushString(buf, "<br /><b><i>");
-		}
-		else if (!strncmp(token, "/div", 4)) {
-		        if (userData["SecHead"] == "true") {
-			        pushString(buf, "</i></b><br />");
-				userData["SecHead"] = "false";
+		else if (tag.getName() && !strcmp(tag.getName(), "div")) {
+			if (tag.isEndTag() && u->SecHead) {
+				buf += "</i></b><br />";
+				u->SecHead = false;
+			}
+			else if (tag.getAttribute("class")) {
+				if (!stricmp(tag.getAttribute("class"), "sechead")) {
+					u->SecHead = true;
+					buf += "<br /><b><i>";
+				}
+				else if (!stricmp(tag.getAttribute("class"), "title")) {
+					u->SecHead = true;
+					buf += "<br /><b><i>";
+				}
 			}
 		}
-
-		else if (!strncmp(token, "sync type=\"Strongs\" value=\"T", 28)) {
-			pushString(buf, "<a href=\"");
-			for (tok = token + 5; *(tok+1); tok++)				
-				if(*tok != '\"') 			
+/*		else if (!strncmp(token, "sync type=\"Strongs\" value=\"T", 28)) {
+			buf +="<a href=\"");
+			for (tok = token + 5; *(tok+1); tok++)
+				if(*tok != '\"')
 					*(*buf)++ = *tok;
 			*(*buf)++ = '\"';
 			*(*buf)++ = '>';
 			for (tok = token + 29; *(tok+2); tok++)				
 				if(*tok != '\"') 			
 					*(*buf)++ = *tok;		
-			pushString(buf, "</a>");
+			buf +="</a>");
 		}
-		else if (!strncmp(token, "img ", 4)) {
+*/
+		else if (tag.getName() && !strcmp(tag.getName(), "img")) {
 			const char *src = strstr(token, "src");
 			if (!src)		// assert we have a src attribute
 				return false;
 
-			*(*buf)++ = '<';
+			buf += '<';
 			for (const char *c = token; *c; c++) {
 				if (c == src) {
 					for (;((*c) && (*c != '"')); c++)
-						*(*buf)++ = *c;
+						buf += *c;
 
 					if (!*c) { c--; continue; }
 
-					*(*buf)++ = '"';
+					buf += '"';
 					if (*(c+1) == '/') {
-						pushString(buf, "file:");
-						pushString(buf, module->getConfigEntry("AbsoluteDataPath"));
-						if (*((*buf)-1) == '/')
+						buf += "file:";
+						buf += userData->module->getConfigEntry("AbsoluteDataPath");
+						if (buf[buf.length()-2] == '/')
 							c++;		// skip '/'
 					}
 					continue;
 				}
-				*(*buf)++ = *c;
+				buf += *c;
 			}
-			*(*buf)++ = '>';
+			buf += '>';
 		}
-		else if (!strncmp(token, "note", 4)) {
-			pushString(buf, " <small><font color=\"#800000\">(");
-		}                
 		else {
-			*(*buf)++ = '<';
-			for (const char *tok = token; *tok; tok++)
-				*(*buf)++ = *tok;
-			*(*buf)++ = '>';
+			buf += '<';
+			/*for (const char *tok = token; *tok; tok++)
+				buf += *tok;*/
+			buf += token;
+			buf += '>';
 			//return false;  // we still didn't handle token
 		}
 	}
