@@ -15,8 +15,8 @@
  ***************************************************************************/
 
 #include <stdlib.h>
-#include <string.h>
 #include <thmlrtf.h>
+#include <swmodule.h>
 
 SWORD_NAMESPACE_START
 
@@ -31,6 +31,7 @@ ThMLRTF::ThMLRTF()
 	setEscapeStringCaseSensitive(true);
 
 	addEscapeStringSubstitute("nbsp", " ");
+	addEscapeStringSubstitute("apos", "'");
 	addEscapeStringSubstitute("quot", "\"");
 	addEscapeStringSubstitute("amp", "&");
 	addEscapeStringSubstitute("lt", "<");
@@ -148,71 +149,137 @@ ThMLRTF::ThMLRTF()
         addTokenSubstitute("P", "\\par ");
 }
 
-bool ThMLRTF::handleToken(char **buf, const char *token, DualStringMap &userData) {
+char ThMLRTF::processText(SWBuf &text, const SWKey *key, const SWModule *module)
+{
+        SWBasicFilter::processText(text, key, module);  //handle tokens as usual
+	const char *from;
+	SWBuf orig = text;
+	from = orig.c_str();
+	for (text = ""; *from; from++) {  //loop to remove extra spaces
+                if ((strchr(" \t\n\r", *from))) {
+                        while (*(from+1) && (strchr(" \t\n\r", *(from+1)))) {
+                                from++;
+                        }
+                        text += " ";
+                }
+			 else {
+                        text += *from;
+                }
+        }
+        text += (char)0;
+        return 0;
+}
+
+bool ThMLRTF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *userData) {
+	MyUserData *u = (MyUserData *)userData;
 	if (!substituteToken(buf, token)) {
 	// manually process if it wasn't a simple substitution
 		if (!strncmp(token, "sync type=\"Strongs\" value=\"", 27)) {
-                        if (token[27] == 'H' || token[27] == 'G' || token[27] == 'A') {
-        			pushString(buf, " {\\fs15 <");
-                                for (unsigned int i = 28; token[i] != '\"'; i++)
-                		        *(*buf)++ = token[i];
-				pushString(buf, ">}");
-				    }
-				    else if (token[27] == 'T') {
-						  pushString(buf, " {\\fs15 (");
+			if (token[27] == 'H' || token[27] == 'G' || token[27] == 'A') {
+				buf += " {\\fs15 <";
 				for (unsigned int i = 28; token[i] != '\"'; i++)
-					*(*buf)++ = token[i];
-				pushString(buf, ")}");
-				    }
+					buf += token[i];
+				buf += ">}";
+			}
+			else if (token[27] == 'T') {
+				buf += " {\\fs15 (";
+				for (unsigned int i = 28; token[i] != '\"'; i++)
+					buf += token[i];
+				buf += ")}";
+			}
 		}
 		else if (!strncmp(token, "sync type=\"morph\" ", 18)) {
-			pushString(buf, " {\\fs15 (");
+			buf += " {\\fs15 (";
 			for (const char *tok = token + 5; *tok; tok++) {
 				if (!strncmp(tok, "value=\"", 7)) {
 					tok += 7;
 					for (;*tok != '\"'; tok++)
-						*(*buf)++ = *tok;
+						buf += *tok;
 					break;
 				}
 			}
 
-			pushString(buf, ")}");
+			buf += ")}";
 		}
 		else if (!strncmp(token, "sync type=\"lemma\" value=\"", 25)) {
-			pushString(buf, "{\\fs15 (");
+			buf += "{\\fs15 (";
 			for (unsigned int i = 25; token[i] != '\"'; i++)
-				*(*buf)++ = token[i];
-			pushString(buf, ")}");
+				buf += token[i];
+			buf += ")}";
 		}
 		else if (!strncmp(token, "scripRef", 8)) {
-//			pushString(buf, "{\\cf2 #");
-			pushString(buf, "<a href=\"\">");
+//			buf += "{\\cf2 #";
+			buf += "<a href=\"\">";
 		}
 		else if (!strncmp(token, "/scripRef", 9)) {
-			pushString(buf, "</a>");
+			buf += "</a>";
 		}
 		else if (!strncmp(token, "div", 3)) {
-			*(*buf)++ = '{';
+			buf += '{';
 			if (!strncmp(token, "div class=\"title\"", 17)) {
-				pushString(buf, "\\par\\i1\\b1 ");
-				userData["sechead"] = "true";
+				buf += "\\par\\i1\\b1 ";
+				u->sechead = true;
 			}
 			else if (!strncmp(token, "div class=\"sechead\"", 19)) {
-				pushString(buf, "\\par\\i1\\b1 ");
-				userData["sechead"] = "true";
+				buf += "\\par\\i1\\b1 ";
+				u->sechead = true;
 			}
 		}
 		else if (!strncmp(token, "/div", 4)) {
-			*(*buf)++ = '}';
-			if (userData["sechead"] == "true") {
-				pushString(buf, "\\par ");
-				userData["sechead"] == "false";
+			if (u->sechead) {
+				buf += "\\par ";
+				u->sechead = false;
 			}
+			buf += '}';
 		}
 		else if (!strncmp(token, "note", 4)) {
-			pushString(buf, " {\\i1\\fs15 (");
+			buf += " {\\i1\\fs15 (";
 		}
+		else if (!strncmp(token, "img ", 4)) {
+			const char *src = strstr(token, "src");
+			if (!src)		// assert we have a src attribute
+				return false;
 
+			char* filepath = new char[strlen(u->module->getConfigEntry("AbsoluteDataPath")) + strlen(token)];
+			*filepath = 0;
+			strcpy(filepath, userData->module->getConfigEntry("AbsoluteDataPath"));
+			unsigned long i = strlen(filepath);
+			const char *c;
+			for (c = (src + 5); *c != '"'; c++) {
+				filepath[i] = *c;
+				i++;
+			}
+			filepath[i] = 0;
+
+			buf+="<figure src=\"";
+			buf+=filepath;
+			buf+="\" />";
+/*
+			char imgc;
+			FILE* imgfile;
+			for (c = filepath + strlen(filepath); c > filepath && *c != '.'; c--);
+			c++;
+
+
+			if (stricmp(c, "jpg") || stricmp(c, "jpeg")) {
+				imgfile = fopen(filepath, "r");
+				if (imgfile != NULL) {
+					buf += "{\\nonshppict {\\pict\\jpegblip ";
+					while (feof(imgfile) != EOF) {
+						buf.appendFormatted("%2x", fgetc(imgfile));
+					}
+					fclose(imgfile);
+					buf += "}}";
+				}
+			}
+			else if (stricmp(c, "png")) {
+				buf += "{\\*\\shppict {\\pict\\pngblip ";
+
+				buf += "}}";
+			}
+*/
+			delete [] filepath;
+		}
 		else {
 			return false;  // we still didn't handle token
 		}
