@@ -1,13 +1,12 @@
 /******************************************************************************
 *
-* utf8transliterators - SWFilter decendant to transliterate between
+* utf8transliterators - SWFilter descendant to transliterate between
 *			 ICU-supported scripts.
 */
 
 #ifdef _ICU_
 
 #include <stdlib.h>
-#include <string.h>
 
 #ifdef __GNUC__
 #include <unixstr.h>
@@ -319,7 +318,7 @@ const char *UTF8Transliterator::getOptionValue()
 	return (NUMTARGETSCRIPTS > option) ? optionstring[option] : 0;
 }
 
-char UTF8Transliterator::ProcessText(char *text, int maxlen, const SWKey *key, const SWModule *module)
+char UTF8Transliterator::processText(SWBuf &text, const SWKey *key, const SWModule *module)
 {
 	if (option) {	// if we want transliteration
 		unsigned long i, j;
@@ -712,11 +711,11 @@ char UTF8Transliterator::ProcessText(char *text, int maxlen, const SWKey *key, c
 				Transliterator * trans = createTrans(preid, id, postid, UTRANS_FORWARD, err);
                 if (trans && !U_FAILURE(err)) {
                         UnicodeString target = UnicodeString(source);
-                        trans->transliterate(target);
-                        len = ucnv_fromUChars(conv, text, maxlen, target.getBuffer(), target.length(), &err);
-                        if (len < maxlen) *(text + len) = 0;
-                        else *(text + maxlen) = 0;
-                        delete trans;
+				    trans->transliterate(target);
+				    text.setSize(text.size()*2);
+				    len = ucnv_fromUChars(conv, text.getRawData(), text.size(), target.getBuffer(), target.length(), &err);
+					text.setSize(len);
+				    delete trans;
                 }
                 ucnv_close(conv);
         }
