@@ -31,11 +31,11 @@ int main(int argc, char **argv)
 	long maxoff = lseek(idxfd->getFd(), 0, SEEK_END) - 6;
 	FileMgr::systemFileMgr.close(idxfd);
 
-	std::string last = "";
+	SWBuf last = "";
 	bool first = true;
 	char *trybuf = 0;
 	for (long index = 0; index < maxoff; index+=6) {
-		mod.getidxbuf(index, &trybuf);
+		mod.getIDXBuf(index, &trybuf);
 		if (!first) {
 			if (strcmp(trybuf, last.c_str()) < 0) {
 				printf("entry %ld(offset: %ld) (%s) is less than previous entry (%s)\n\n", index/6, index, trybuf, last.c_str());
